@@ -19,11 +19,12 @@
 
 package org.apache.samza.metrics;
 
+import java.io.Serializable;
 import org.apache.samza.config.Config;
 
 /**
  * Build a {@link org.apache.samza.metrics.MetricsReporter}
  */
-public interface MetricsReporterFactory {
-  MetricsReporter getMetricsReporter(String name, String containerName, Config config);
+public interface MetricsReporterFactory extends Serializable {
+  MetricsReporter getMetricsReporter(String name, String processorId, Config config);
 }
