@@ -18,12 +18,8 @@
  */
 package org.apache.samza.table;
 
-import java.util.Map;
-
 import org.apache.samza.annotation.InterfaceStability;
-import org.apache.samza.container.SamzaContainerContext;
-import org.apache.samza.task.TaskContext;
-
+import org.apache.samza.context.Context;
 
 /**
  * A table provider provides the implementation for a table. It ensures a table is
@@ -33,26 +29,15 @@ import org.apache.samza.task.TaskContext;
 public interface TableProvider {
   /**
    * Initialize TableProvider with container and task context
-   * @param containerContext Samza container context
-   * @param taskContext nullable for global table
+   * @param context context for the task
    */
-  void init(SamzaContainerContext containerContext, TaskContext taskContext);
+  void init(Context context);
 
   /**
-   * Get an instance of the table for read/write operations
+   * Get an instance of the {@link ReadWriteTable}
    * @return the underlying table
    */
-  Table getTable();
-
-  /**
-   * Generate any configuration for this table, the generated configuration
-   * is used by Samza container to construct this table and any components
-   * necessary
-   * .
-   * @param config the current configuration
-   * @return configuration for this table
-   */
-  Map<String, String> generateConfig(Map<String, String> config);
+  ReadWriteTable getTable();
 
   /**
    * Shutdown the underlying table
