@@ -21,7 +21,6 @@ package org.apache.samza.util;
 
 import org.apache.samza.metrics.Counter;
 import org.apache.samza.metrics.Gauge;
-import org.apache.samza.metrics.ListGauge;
 import org.apache.samza.metrics.MetricsRegistry;
 import org.apache.samza.metrics.Timer;
 
@@ -51,11 +50,6 @@ public class NoOpMetricsRegistry implements MetricsRegistry {
     return gauge;
   }
 
-  @Override
-  public <T> ListGauge<T> newListGauge(String group, ListGauge<T> listGauge) {
-    return listGauge;
-  }
-
   @Override
   public Timer newTimer(String group, String name) {
     return new Timer(name);
