@@ -19,17 +19,16 @@
 
 package org.apache.samza.coordinator;
 
-import org.apache.commons.lang3.StringUtils;
 import org.apache.samza.AzureClient;
-import org.apache.samza.config.ApplicationConfig;
 import org.apache.samza.config.AzureConfig;
 import org.apache.samza.config.Config;
-import org.apache.samza.config.ConfigException;
 import org.apache.samza.config.JobConfig;
 import org.apache.samza.config.TaskConfig;
 import org.apache.samza.container.TaskName;
 import org.apache.samza.container.grouper.stream.SystemStreamPartitionGrouper;
 import org.apache.samza.container.grouper.stream.SystemStreamPartitionGrouperFactory;
+import org.apache.samza.container.grouper.task.GrouperMetadata;
+import org.apache.samza.container.grouper.task.GrouperMetadataImpl;
 import org.apache.samza.coordinator.data.BarrierState;
 import org.apache.samza.coordinator.data.ProcessorEntity;
 import org.apache.samza.coordinator.scheduler.HeartbeatScheduler;
@@ -40,7 +39,7 @@ import org.apache.samza.coordinator.scheduler.LivenessCheckScheduler;
 import org.apache.samza.coordinator.scheduler.RenewLeaseScheduler;
 import org.apache.samza.coordinator.scheduler.SchedulerStateChangeListener;
 import org.apache.samza.job.model.JobModel;
-import org.apache.samza.runtime.ProcessorIdGenerator;
+import org.apache.samza.metrics.MetricsRegistry;
 import org.apache.samza.system.StreamMetadataCache;
 import org.apache.samza.system.SystemAdmins;
 import org.apache.samza.system.SystemStream;
@@ -48,13 +47,12 @@ import org.apache.samza.system.SystemStreamMetadata;
 import org.apache.samza.system.SystemStreamPartition;
 import org.apache.samza.util.BlobUtils;
 import org.apache.samza.util.LeaseBlobManager;
+import org.apache.samza.util.ReflectionUtil;
 import org.apache.samza.util.SystemClock;
 import org.apache.samza.util.TableUtils;
-import org.apache.samza.util.Util;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 import scala.collection.JavaConverters;
-
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.List;
@@ -100,10 +98,10 @@ public class AzureJobCoordinator implements JobCoordinator {
    * Creates an instance of Azure job coordinator, along with references to Azure leader elector, Azure Blob and Azure Table.
    * @param config User defined config
    */
-  public AzureJobCoordinator(Config config) {
+  public AzureJobCoordinator(String processorId, Config config, MetricsRegistry metricsRegistry) {
     //TODO: Cleanup previous values in the table when barrier times out.
+    this.processorId = processorId;
     this.config = config;
-    processorId = createProcessorId(config);
     currentJMVersion = new AtomicReference<>(INITIAL_STATE);
     AzureConfig azureConfig = new AzureConfig(config);
     AzureClient client = new AzureClient(azureConfig.getAzureConnectionString());
@@ -273,7 +271,8 @@ public class AzureJobCoordinator implements JobCoordinator {
    */
   private Set<SystemStreamPartition> getInputStreamPartitions() {
     TaskConfig taskConfig = new TaskConfig(config);
-    scala.collection.immutable.Set<SystemStream> inputSystemStreams = taskConfig.getInputStreams();
+    scala.collection.immutable.Set<SystemStream> inputSystemStreams =
+        JavaConverters.asScalaSetConverter(taskConfig.getInputStreams()).asScala().toSet();
 
     // Get the set of partitions for each SystemStream from the stream metadata
     Set<SystemStreamPartition>
@@ -300,8 +299,9 @@ public class AzureJobCoordinator implements JobCoordinator {
   private SystemStreamPartitionGrouper getSystemStreamPartitionGrouper() {
     JobConfig jobConfig = new JobConfig(config);
     String factoryString = jobConfig.getSystemStreamPartitionGrouperFactory();
-    SystemStreamPartitionGrouper grouper = Util.getObj(factoryString, SystemStreamPartitionGrouperFactory.class)
-        .getSystemStreamPartitionGrouper(jobConfig);
+    SystemStreamPartitionGrouper grouper =
+        ReflectionUtil.getObj(factoryString, SystemStreamPartitionGrouperFactory.class)
+            .getSystemStreamPartitionGrouper(jobConfig);
     return grouper;
   }
 
@@ -365,8 +365,9 @@ public class AzureJobCoordinator implements JobCoordinator {
     }
 
     // Generate the new JobModel
-    JobModel newJobModel = JobModelManager.readJobModel(this.config, Collections.emptyMap(),
-        null, streamMetadataCache, currentProcessorIds);
+    GrouperMetadata grouperMetadata = new GrouperMetadataImpl(Collections.emptyMap(), Collections.emptyMap(), Collections.emptyMap(), Collections.emptyMap());
+    JobModel newJobModel =
+        JobModelManager.readJobModel(this.config, Collections.emptyMap(), streamMetadataCache, grouperMetadata);
     LOG.info("pid=" + processorId + "Generated new Job Model. Version = " + nextJMVersion);
 
     // Publish the new job model
@@ -472,22 +473,6 @@ public class AzureJobCoordinator implements JobCoordinator {
     }
   }
 
-  private String createProcessorId(Config config) {
-    // TODO: This check to be removed after 0.13+
-    ApplicationConfig appConfig = new ApplicationConfig(config);
-    if (appConfig.getProcessorId() != null) {
-      return appConfig.getProcessorId();
-    } else if (StringUtils.isNotBlank(appConfig.getAppProcessorIdGeneratorClass())) {
-      ProcessorIdGenerator idGenerator =
-          Util.getObj(appConfig.getAppProcessorIdGeneratorClass(), ProcessorIdGenerator.class);
-      return idGenerator.generateProcessorId(config);
-    } else {
-      throw new ConfigException(String
-          .format("Expected either %s or %s to be configured", ApplicationConfig.PROCESSOR_ID,
-              ApplicationConfig.APP_PROCESSOR_ID_GENERATOR_CLASS));
-    }
-  }
-
   public class AzureLeaderElectorListener implements LeaderElectorListener {
     /**
      * Keep renewing the lease and do the required tasks as a leader.
@@ -500,12 +485,12 @@ public class AzureJobCoordinator implements JobCoordinator {
       // Schedule a task to renew the lease after a fixed time interval
       LOG.info("Starting scheduler to keep renewing lease held by the leader.");
       renewLease = new RenewLeaseScheduler((errorMsg) -> {
-          LOG.error(errorMsg);
-          table.updateIsLeader(currentJMVersion.get(), processorId, false);
-          azureLeaderElector.resignLeadership();
-          renewLease.shutdown();
-          liveness.shutdown();
-        }, azureLeaderElector.getLeaseBlobManager(), azureLeaderElector.getLeaseId());
+        LOG.error(errorMsg);
+        table.updateIsLeader(currentJMVersion.get(), processorId, false);
+        azureLeaderElector.resignLeadership();
+        renewLease.shutdown();
+        liveness.shutdown();
+      }, azureLeaderElector.getLeaseBlobManager(), azureLeaderElector.getLeaseId());
       renewLease.scheduleTask();
 
       doOnProcessorChange(new ArrayList<>());
