@@ -40,7 +40,7 @@ import org.apache.samza.metrics.MetricsRegistry;
 import org.apache.samza.system.OutgoingMessageEnvelope;
 import org.apache.samza.system.SystemProducer;
 import org.apache.samza.system.eventhub.EventHubConfig;
-import org.apache.samza.system.eventhub.metrics.SamzaHistogram;
+import org.apache.samza.metrics.SamzaHistogram;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -140,25 +140,25 @@ public abstract class AsyncSystemProducer implements SystemProducer {
 
     // Auto update the metrics and possible throwable when futures are complete.
     sendResult.handle((aVoid, throwable) -> {
-        long callbackLatencyMs = System.currentTimeMillis() - afterSendTimeMs;
-        sendCallbackLatency.get(streamId).update(callbackLatencyMs);
-        aggSendCallbackLatency.update(callbackLatencyMs);
-        if (throwable != null) {
-          sendErrors.get(streamId).inc();
-          aggSendErrors.inc();
-          LOG.error("Send message to event hub: {} failed with exception: ", streamId, throwable);
-          sendExceptionOnCallback.compareAndSet(null, throwable);
-        }
-        return aVoid;
-      });
+      long callbackLatencyMs = System.currentTimeMillis() - afterSendTimeMs;
+      sendCallbackLatency.get(streamId).update(callbackLatencyMs);
+      aggSendCallbackLatency.update(callbackLatencyMs);
+      if (throwable != null) {
+        sendErrors.get(streamId).inc();
+        aggSendErrors.inc();
+        LOG.error("Send message to event hub: {} failed with exception: ", streamId, throwable);
+        sendExceptionOnCallback.compareAndSet(null, throwable);
+      }
+      return aVoid;
+    });
   }
 
   public void start() {
     streamIds.forEach(streamId -> {
-        sendCallbackLatency.put(streamId, new SamzaHistogram(metricsRegistry, streamId, SEND_CALLBACK_LATENCY));
-        sendLatency.put(streamId, new SamzaHistogram(metricsRegistry, streamId, SEND_LATENCY));
-        sendErrors.put(streamId, metricsRegistry.newCounter(streamId, SEND_ERRORS));
-      });
+      sendCallbackLatency.put(streamId, new SamzaHistogram(metricsRegistry, streamId, SEND_CALLBACK_LATENCY));
+      sendLatency.put(streamId, new SamzaHistogram(metricsRegistry, streamId, SEND_LATENCY));
+      sendErrors.put(streamId, metricsRegistry.newCounter(streamId, SEND_ERRORS));
+    });
 
     if (aggSendLatency == null) {
       aggSendLatency = new SamzaHistogram(metricsRegistry, AGGREGATE, SEND_LATENCY);
