@@ -29,6 +29,7 @@ import org.apache.samza.system.SystemStreamPartition;
 import org.apache.samza.system.eventhub.*;
 import org.apache.samza.system.eventhub.admin.PassThroughInterceptor;
 import org.apache.samza.system.eventhub.producer.SwapFirstLastByteInterceptor;
+import org.apache.samza.testUtils.TestClock;
 import org.junit.Assert;
 import org.junit.Test;
 import org.junit.runner.RunWith;
@@ -36,6 +37,8 @@ import org.powermock.core.classloader.annotations.PrepareForTest;
 import org.powermock.modules.junit4.PowerMockRunner;
 
 import java.util.*;
+import java.util.concurrent.Future;
+import java.util.concurrent.TimeUnit;
 import java.util.function.Function;
 import java.util.stream.Collectors;
 
@@ -349,4 +352,95 @@ public class TestEventHubSystemConsumer {
     Assert.assertEquals(counters2.get(EventHubSystemConsumer.EVENT_READ_RATE).getCount(), numEvents);
     Assert.assertEquals(counters2.get(EventHubSystemConsumer.READ_ERRORS).getCount(), 0);
   }
+
+  @Test
+  public void testNonTransientErrorRetry() throws Exception {
+    String systemName = "eventhubs";
+    String streamName = "testNonTransientErrorRetry";
+    int numEvents = 10; // needs to be less than BLOCKING_QUEUE_SIZE
+    int partitionId = 0;
+    TestClock testClock = new TestClock();
+
+    TestMetricsRegistry testMetrics = new TestMetricsRegistry();
+    Map<SystemStreamPartition, List<EventData>> eventData = new HashMap<>();
+    SystemStreamPartition ssp = new SystemStreamPartition(systemName, streamName, new Partition(partitionId));
+    Map<String, Interceptor> interceptors = new HashMap<>();
+    interceptors.put(streamName, new PassThroughInterceptor());
+
+    // create EventData
+    List<EventData> singlePartitionEventData = MockEventData.generateEventData(numEvents);
+    eventData.put(ssp, singlePartitionEventData);
+
+    // Set configs
+    Map<String, String> configMap = new HashMap<>();
+    configMap.put(String.format(EventHubConfig.CONFIG_STREAM_LIST, systemName), streamName);
+    configMap.put(String.format(EventHubConfig.CONFIG_STREAM_NAMESPACE, streamName), EVENTHUB_NAMESPACE);
+    configMap.put(String.format(EventHubConfig.CONFIG_STREAM_SAS_KEY_NAME, streamName), EVENTHUB_KEY_NAME);
+    configMap.put(String.format(EventHubConfig.CONFIG_STREAM_SAS_TOKEN, streamName), EVENTHUB_KEY);
+    configMap.put(String.format(EventHubConfig.CONFIG_STREAM_ENTITYPATH, streamName), MOCK_ENTITY_1);
+    configMap.put(String.format(EventHubConfig.CONFIG_MAX_RETRY_COUNT, systemName), "1");
+    MapConfig config = new MapConfig(configMap);
+
+    MockEventHubClientManagerFactory eventHubClientWrapperFactory = new MockEventHubClientManagerFactory(eventData);
+
+    EventHubSystemConsumer consumer =
+        new EventHubSystemConsumer(new EventHubConfig(config), systemName, eventHubClientWrapperFactory, interceptors,
+            testMetrics, testClock);
+    consumer.register(ssp, EventHubSystemConsumer.END_OF_STREAM);
+    consumer.start();
+
+    // 1st error should retry instead of throw
+    testClock.advanceTime(System.currentTimeMillis());
+    eventHubClientWrapperFactory.triggerError(consumer.streamPartitionHandlers,
+        new EventHubException(false /* is transient */, "test"));
+    consumer.poll(Collections.singleton(ssp), 0).get(ssp);
+    // assert that the reconnect task was submitted and completed eventually
+    Assert.assertNotNull("reconnect task should have been submitted", consumer.reconnectTaskStatus);
+    Future lastReconnectTask = consumer.reconnectTaskStatus;
+    lastReconnectTask.get(10000, TimeUnit.MILLISECONDS); // should return instantaneously
+    Assert.assertEquals(consumer.recentRetryAttempts.size(), 1);
+
+    // after retry should receive events normally
+    testClock.advanceTime(1);
+    eventHubClientWrapperFactory.sendToHandlers(consumer.streamPartitionHandlers);
+    List<IncomingMessageEnvelope> result = consumer.poll(Collections.singleton(ssp), 0).get(ssp);
+    verifyEvents(result, singlePartitionEventData);
+    Assert.assertEquals(testMetrics.getCounters(streamName).size(), 3);
+    Assert.assertEquals(testMetrics.getGauges(streamName).size(), 2);
+    Map<String, Counter> counters =
+        testMetrics.getCounters(streamName).stream().collect(Collectors.toMap(Counter::getName, Function.identity()));
+    Assert.assertEquals(counters.get(EventHubSystemConsumer.EVENT_READ_RATE).getCount(), numEvents);
+
+    // 2nd error: advance into next window, the older retry should have been evicted so this error should cause retry
+    testClock.advanceTime(EventHubConfig.DEFAULT_CONFIG_RETRY_WINDOW_MS + 1);
+    Assert.assertEquals(consumer.recentRetryAttempts.size(), 0);
+    eventHubClientWrapperFactory.triggerError(consumer.streamPartitionHandlers,
+        new EventHubException(false /* is transient */, "test"));
+    consumer.poll(Collections.singleton(ssp), 0).get(ssp);
+    Assert.assertNotNull("reconnect task should have been submitted", consumer.reconnectTaskStatus);
+    lastReconnectTask = consumer.reconnectTaskStatus;
+    lastReconnectTask.get(10000, TimeUnit.MILLISECONDS); // should return instantaneously
+    Assert.assertEquals(consumer.recentRetryAttempts.size(), 1);
+
+    // 3rd error: 1 ms is within the min retry interval; so poll should do nothing
+    testClock.advanceTime(1);
+    eventHubClientWrapperFactory.triggerError(consumer.streamPartitionHandlers,
+        new EventHubException(false /* is transient */, "test"));
+    consumer.poll(Collections.singleton(ssp), 0).get(ssp);
+    Assert.assertEquals("there shouldn't be another retry task within min retry interval", consumer.reconnectTaskStatus,
+        lastReconnectTask);
+
+    // 4th error: now the poll should throw
+    testClock.advanceTime(EventHubConfig.DEFAULT_CONFIG_RETRY_INTERVAL_MS + 1);
+    eventHubClientWrapperFactory.triggerError(consumer.streamPartitionHandlers,
+        new EventHubException(false /* is transient */, "test"));
+    try {
+      consumer.poll(Collections.singleton(ssp), 0).get(ssp);
+      Assert.fail("poll should have thrown");
+    } catch (Exception e) {
+      Assert.assertEquals(e.getCause().getMessage(), "test");
+    }
+
+    Assert.assertEquals(counters.get(EventHubSystemConsumer.READ_ERRORS).getCount(), 4);
+  }
 }
