@@ -18,6 +18,7 @@
  */
 package org.apache.samza.config;
 
+import org.apache.samza.clustermanager.DefaultApplicationMain;
 import org.apache.samza.runtime.UUIDGenerator;
 
 
@@ -56,6 +57,11 @@ public class ApplicationConfig extends MapConfig {
   public static final String APP_CLASS = "app.class";
   public static final String APP_MODE = "app.mode";
   public static final String APP_RUN_ID = "app.run.id";
+  public static final String APP_MAIN_CLASS = "app.main.class";
+  public static final String APP_MAIN_ARGS = "app.main.args";
+  public static final String APP_RUNNER_CLASS = "app.runner.class";
+
+  private static final String DEFAULT_APP_RUNNER = "org.apache.samza.runtime.RemoteApplicationRunner";
 
   public ApplicationConfig(Config config) {
     super(config);
@@ -66,11 +72,11 @@ public class ApplicationConfig extends MapConfig {
   }
 
   public String getAppName() {
-    return get(APP_NAME, get(JobConfig.JOB_NAME()));
+    return get(APP_NAME, get(JobConfig.JOB_NAME));
   }
 
   public String getAppId() {
-    return get(APP_ID, get(JobConfig.JOB_ID(), "1"));
+    return get(APP_ID, get(JobConfig.JOB_ID, "1"));
   }
 
   public String getAppClass() {
@@ -85,7 +91,6 @@ public class ApplicationConfig extends MapConfig {
     return String.format("app-%s-%s", getAppName(), getAppId());
   }
 
-  @Deprecated
   public String getProcessorId() {
     return get(PROCESSOR_ID, null);
   }
@@ -98,4 +103,11 @@ public class ApplicationConfig extends MapConfig {
     return ApplicationMode.valueOf(get(APP_MODE, ApplicationMode.STREAM.name()).toUpperCase());
   }
 
+  public String getAppMainClass() {
+    return get(APP_MAIN_CLASS, DefaultApplicationMain.class.getName());
+  }
+
+  public String getAppRunnerClass() {
+    return get(APP_RUNNER_CLASS, DEFAULT_APP_RUNNER);
+  }
 }
