@@ -22,6 +22,7 @@ package org.apache.samza.coordinator.stream;
 import joptsimple.OptionSet;
 import org.apache.samza.config.Config;
 import org.apache.samza.coordinator.stream.messages.SetConfig;
+import org.apache.samza.execution.JobPlanner;
 import org.apache.samza.metrics.MetricsRegistryMap;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
@@ -102,11 +103,9 @@ public class CoordinatorStreamWriter {
    * Main function for using the CoordinatorStreamWriter. The main function starts a CoordinatorStreamWriter
    * and sends control messages.
    * To run the code use the following command:
-   * {path to samza deployment}/samza/bin/run-coordinator-stream-writer.sh  --config-factory={config-factory} --config-path={path to config file of a job} --type={type of the message} --key={[optional] key of the message} --value={[optional] value of the message}
+   * {path to samza deployment}/samza/bin/run-coordinator-stream-writer.sh  --config job.config.loader.factory={config-loader-factory} --config job.config.loader.properties{properties needed for config loader to load config} --type={type of the message} --key={[optional] key of the message} --value={[optional] value of the message}
    *
    * @param args input arguments for running the writer. These arguments are:
-   *             "config-factory" = The config file factory
-   *             "config-path" = The path to config file of a job
    *             "type" = type of the message being written
    *             "key" = [optional] key of the message being written
    *             "value" = [optional] value of the message being written
@@ -114,12 +113,13 @@ public class CoordinatorStreamWriter {
   public static void main(String[] args) {
     CoordinatorStreamWriterCommandLine cmdline = new CoordinatorStreamWriterCommandLine();
     OptionSet options = cmdline.parser().parse(args);
-    Config config = cmdline.loadConfig(options);
+    Config userConfig = cmdline.loadConfig(options);
+    Config generatedConfig = JobPlanner.generateSingleJobConfig(userConfig);
     String type = cmdline.loadType(options);
     String key = cmdline.loadKey(options);
     String value = cmdline.loadValue(options);
 
-    CoordinatorStreamWriter writer = new CoordinatorStreamWriter(config);
+    CoordinatorStreamWriter writer = new CoordinatorStreamWriter(generatedConfig);
     writer.start();
     writer.sendMessage(type, key, value);
     writer.stop();
