@@ -32,6 +32,7 @@ import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
+import java.util.function.Function;
 import java.util.stream.Collectors;
 import org.apache.commons.collections4.ListUtils;
 import org.apache.samza.SamzaException;
@@ -42,13 +43,14 @@ import org.apache.samza.config.ApplicationConfig;
 import org.apache.samza.config.ClusterManagerConfig;
 import org.apache.samza.config.Config;
 import org.apache.samza.config.JobConfig;
+import org.apache.samza.config.MapConfig;
 import org.apache.samza.config.StreamConfig;
-import org.apache.samza.table.descriptors.BaseTableDescriptor;
 import org.apache.samza.operators.spec.InputOperatorSpec;
 import org.apache.samza.operators.spec.OperatorSpec;
 import org.apache.samza.operators.spec.StreamTableJoinOperatorSpec;
 import org.apache.samza.system.StreamSpec;
-import org.apache.samza.table.TableSpec;
+import org.apache.samza.table.descriptors.LocalTableDescriptor;
+import org.apache.samza.table.descriptors.TableDescriptor;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -103,7 +105,7 @@ public class ExecutionPlanner {
     // currently we don't support host-affinity in batch mode
     if (appConfig.getAppMode() == ApplicationConfig.ApplicationMode.BATCH && clusterConfig.getHostAffinityEnabled()) {
       throw new SamzaException(String.format("Host affinity is not supported in batch mode. Please configure %s=false.",
-          ClusterManagerConfig.CLUSTER_MANAGER_HOST_AFFINITY_ENABLED));
+          ClusterManagerConfig.JOB_HOST_AFFINITY_ENABLED));
     }
   }
 
@@ -122,12 +124,14 @@ public class ExecutionPlanner {
     Set<StreamSpec> inputStreams = Sets.difference(sourceStreams, intermediateStreams);
     Set<StreamSpec> outputStreams = Sets.difference(sinkStreams, intermediateStreams);
 
-    Set<TableSpec> tables = appDesc.getTableDescriptors().stream()
-        .map(tableDescriptor -> ((BaseTableDescriptor) tableDescriptor).getTableSpec()).collect(Collectors.toSet());
+    Set<TableDescriptor> tables = appDesc.getTableDescriptors();
+
+    // Generate job.id and job.name configs from app.id and app.name if defined
+    MapConfig generatedJobConfigs = JobPlanner.generateSingleJobConfig(config);
+    String jobName = generatedJobConfigs.get(JobConfig.JOB_NAME);
+    String jobId = generatedJobConfigs.get(JobConfig.JOB_ID, "1");
 
     // For this phase, we have a single job node for the whole DAG
-    String jobName = config.get(JobConfig.JOB_NAME());
-    String jobId = config.get(JobConfig.JOB_ID(), "1");
     JobNode node = jobGraph.getOrCreateJobNode(jobName, jobId);
 
     // Add input streams
@@ -140,12 +144,15 @@ public class ExecutionPlanner {
     intermediateStreams.forEach(spec -> jobGraph.addIntermediateStream(spec, node, node));
 
     // Add tables
-    for (TableSpec table : tables) {
+    for (TableDescriptor table : tables) {
       jobGraph.addTable(table, node);
       // Add side-input streams (if any)
-      Iterable<String> sideInputs = ListUtils.emptyIfNull(table.getSideInputs());
-      for (String sideInput : sideInputs) {
-        jobGraph.addSideInputStream(getStreamSpec(sideInput, streamConfig));
+      if (table instanceof LocalTableDescriptor) {
+        LocalTableDescriptor localTable = (LocalTableDescriptor) table;
+        Iterable<String> sideInputs = ListUtils.emptyIfNull(localTable.getSideInputs());
+        for (String sideInput : sideInputs) {
+          jobGraph.addSideInputStream(getStreamSpec(sideInput, streamConfig));
+        }
       }
     }
 
@@ -208,6 +215,9 @@ public class ExecutionPlanner {
         OperatorSpecGraphAnalyzer.getJoinToInputOperatorSpecs(
             jobGraph.getApplicationDescriptorImpl().getInputOperators().values());
 
+    Map<String, TableDescriptor> tableDescriptors = jobGraph.getTables().stream()
+        .collect(Collectors.toMap(TableDescriptor::getTableId, Function.identity()));
+
     // Convert every group of input operator specs into a group of corresponding stream edges.
     List<StreamSet> streamSets = new ArrayList<>();
     for (OperatorSpec joinOpSpec : joinOpSpecToInputOpSpecs.keySet()) {
@@ -218,11 +228,14 @@ public class ExecutionPlanner {
       // streams associated with the joined table (if any).
       if (joinOpSpec instanceof StreamTableJoinOperatorSpec) {
         StreamTableJoinOperatorSpec streamTableJoinOperatorSpec = (StreamTableJoinOperatorSpec) joinOpSpec;
-
-        Collection<String> sideInputs = ListUtils.emptyIfNull(streamTableJoinOperatorSpec.getTableSpec().getSideInputs());
-        Iterable<StreamEdge> sideInputStreams = sideInputs.stream().map(jobGraph::getStreamEdge)::iterator;
-        Iterable<StreamEdge> streams = streamSet.getStreamEdges();
-        streamSet = new StreamSet(streamSet.getSetId(), Iterables.concat(streams, sideInputStreams));
+        TableDescriptor tableDescriptor = tableDescriptors.get(streamTableJoinOperatorSpec.getTableId());
+        if (tableDescriptor instanceof LocalTableDescriptor) {
+          LocalTableDescriptor localTableDescriptor = (LocalTableDescriptor) tableDescriptor;
+          Collection<String> sideInputs = ListUtils.emptyIfNull(localTableDescriptor.getSideInputs());
+          Iterable<StreamEdge> sideInputStreams = sideInputs.stream().map(jobGraph::getStreamEdge)::iterator;
+          Iterable<StreamEdge> streams = streamSet.getStreamEdges();
+          streamSet = new StreamSet(streamSet.getSetId(), Iterables.concat(streams, sideInputStreams));
+        }
       }
 
       streamSets.add(streamSet);
