@@ -19,6 +19,7 @@
 
 package org.apache.samza.execution;
 
+import com.google.common.annotations.VisibleForTesting;
 import java.io.ByteArrayOutputStream;
 import java.util.ArrayList;
 import java.util.Collection;
@@ -35,8 +36,10 @@ import org.apache.samza.operators.spec.OperatorSpec;
 import org.apache.samza.operators.spec.OutputOperatorSpec;
 import org.apache.samza.operators.spec.OutputStreamImpl;
 import org.apache.samza.operators.spec.PartitionByOperatorSpec;
+import org.apache.samza.operators.spec.SendToTableOperatorSpec;
 import org.apache.samza.operators.spec.StreamTableJoinOperatorSpec;
-import org.apache.samza.table.TableSpec;
+import org.apache.samza.table.descriptors.BaseTableDescriptor;
+import org.apache.samza.table.descriptors.TableDescriptor;
 import org.codehaus.jackson.annotate.JsonProperty;
 import org.codehaus.jackson.map.ObjectMapper;
 
@@ -59,10 +62,8 @@ import org.codehaus.jackson.map.ObjectMapper;
   static final class TableSpecJson {
     @JsonProperty("id")
     String id;
-    @JsonProperty("tableProviderFactory")
-    String tableProviderFactory;
-    @JsonProperty("config")
-    Map<String, String> config;
+    @JsonProperty("providerFactory")
+    String providerFactory;
   }
 
   static final class StreamEdgeJson {
@@ -165,7 +166,8 @@ import org.codehaus.jackson.map.ObjectMapper;
    * @param spec a {@link OperatorSpec} instance
    * @return map of the operator properties
    */
-  private Map<String, Object> operatorToMap(OperatorSpec spec) {
+  @VisibleForTesting
+  Map<String, Object> operatorToMap(OperatorSpec spec) {
     Map<String, Object> map = new HashMap<>();
     map.put("opCode", spec.getOpCode().name());
     map.put("opId", spec.getOpId());
@@ -183,13 +185,13 @@ import org.codehaus.jackson.map.ObjectMapper;
     }
 
     if (spec instanceof StreamTableJoinOperatorSpec) {
-      TableSpec tableSpec = ((StreamTableJoinOperatorSpec) spec).getTableSpec();
-      map.put("tableId", tableSpec.getId());
+      String tableId = ((StreamTableJoinOperatorSpec) spec).getTableId();
+      map.put("tableId", tableId);
     }
 
-    if (spec instanceof StreamTableJoinOperatorSpec) {
-      TableSpec tableSpec = ((StreamTableJoinOperatorSpec) spec).getTableSpec();
-      map.put("tableId", tableSpec.getId());
+    if (spec instanceof SendToTableOperatorSpec) {
+      String tableId = ((SendToTableOperatorSpec) spec).getTableId();
+      map.put("tableId", tableId);
     }
 
     if (spec instanceof JoinOperatorSpec) {
@@ -223,19 +225,19 @@ import org.codehaus.jackson.map.ObjectMapper;
     OperatorGraphJson opGraph = new OperatorGraphJson();
     opGraph.inputStreams = new ArrayList<>();
     jobNode.getInEdges().values().forEach(inStream -> {
-        StreamJson inputJson = new StreamJson();
-        opGraph.inputStreams.add(inputJson);
-        inputJson.streamId = inStream.getStreamSpec().getId();
-        inputJson.nextOperatorIds = jobNode.getNextOperatorIds(inputJson.streamId);
-        updateOperatorGraphJson(jobNode.getInputOperator(inputJson.streamId), opGraph);
-      });
+      StreamJson inputJson = new StreamJson();
+      opGraph.inputStreams.add(inputJson);
+      inputJson.streamId = inStream.getStreamSpec().getId();
+      inputJson.nextOperatorIds = jobNode.getNextOperatorIds(inputJson.streamId);
+      updateOperatorGraphJson(jobNode.getInputOperator(inputJson.streamId), opGraph);
+    });
 
     opGraph.outputStreams = new ArrayList<>();
     jobNode.getOutEdges().values().forEach(outStream -> {
-        StreamJson outputJson = new StreamJson();
-        outputJson.streamId = outStream.getStreamSpec().getId();
-        opGraph.outputStreams.add(outputJson);
-      });
+      StreamJson outputJson = new StreamJson();
+      outputJson.streamId = outStream.getStreamSpec().getId();
+      opGraph.outputStreams.add(outputJson);
+    });
     return opGraph;
   }
 
@@ -270,27 +272,15 @@ import org.codehaus.jackson.map.ObjectMapper;
     return edgeJson;
   }
 
-  /**
-   * Get or create the JSON POJO for a {@link TableSpec}
-   * @param tableSpec the {@link TableSpec}
-   * @param tableSpecs a map of tableId to {@link TableSpecJson}
-   * @return JSON representation of the {@link TableSpec}
-   */
-  private TableSpecJson buildTableJson(TableSpec tableSpec, Map<String, TableSpecJson> tableSpecs) {
-    String tableId = tableSpec.getId();
-    return tableSpecs.computeIfAbsent(tableId, k -> buildTableJson(tableSpec));
+  private TableSpecJson buildTableJson(TableDescriptor tableDescriptor, Map<String, TableSpecJson> tableSpecs) {
+    String tableId = tableDescriptor.getTableId();
+    return tableSpecs.computeIfAbsent(tableId, k -> buildTableJson(tableDescriptor));
   }
 
-  /**
-   * Create the JSON POJO for a {@link TableSpec}
-   * @param tableSpec the {@link TableSpec}
-   * @return JSON representation of the {@link TableSpec}
-   */
-  private TableSpecJson buildTableJson(TableSpec tableSpec) {
+  private TableSpecJson buildTableJson(TableDescriptor tableDescriptor) {
     TableSpecJson tableSpecJson = new TableSpecJson();
-    tableSpecJson.id = tableSpec.getId();
-    tableSpecJson.tableProviderFactory = tableSpec.getTableProviderFactoryClassName();
-    tableSpecJson.config = tableSpec.getConfig();
+    tableSpecJson.id = tableDescriptor.getTableId();
+    tableSpecJson.providerFactory = ((BaseTableDescriptor) tableDescriptor).getProviderFactoryClassName();
     return tableSpecJson;
   }
 }
