@@ -18,12 +18,17 @@
  */
 package org.apache.samza.operators.impl;
 
+import com.google.common.annotations.VisibleForTesting;
+import java.util.concurrent.CompletableFuture;
+import java.util.concurrent.CompletionStage;
 import org.apache.samza.SamzaException;
 import org.apache.samza.config.Config;
 import org.apache.samza.config.MetricsConfig;
 import org.apache.samza.container.TaskName;
+import org.apache.samza.context.ContainerContext;
 import org.apache.samza.context.Context;
-import org.apache.samza.context.TaskContextImpl;
+import org.apache.samza.context.InternalTaskContext;
+import org.apache.samza.context.TaskContext;
 import org.apache.samza.job.model.TaskModel;
 import org.apache.samza.metrics.Counter;
 import org.apache.samza.metrics.MetricsRegistry;
@@ -88,9 +93,11 @@ public abstract class OperatorImpl<M, RM> {
   /**
    * Initialize this {@link OperatorImpl} and its user-defined functions.
    *
-   * @param context the {@link Context} for the task
+   * @param internalTaskContext the {@link InternalTaskContext} for the task
    */
-  public final void init(Context context) {
+  public final void init(InternalTaskContext internalTaskContext) {
+    final Context context = internalTaskContext.getContext();
+
     String opId = getOpImplId();
 
     if (initialized) {
@@ -105,17 +112,18 @@ public abstract class OperatorImpl<M, RM> {
     registeredOperators = new HashSet<>();
     prevOperators = new HashSet<>();
     inputStreams = new HashSet<>();
-    // TODO SAMZA-1935: the objects that are only accessible through TaskContextImpl should be moved somewhere else
-    TaskContextImpl taskContext = (TaskContextImpl) context.getTaskContext();
-    MetricsRegistry metricsRegistry = taskContext.getTaskMetricsRegistry();
+
+    final ContainerContext containerContext = context.getContainerContext();
+    final MetricsRegistry metricsRegistry = containerContext.getContainerMetricsRegistry();
     this.numMessage = metricsRegistry.newCounter(METRICS_GROUP, opId + "-messages");
     this.handleMessageNs = metricsRegistry.newTimer(METRICS_GROUP, opId + "-handle-message-ns");
     this.handleTimerNs = metricsRegistry.newTimer(METRICS_GROUP, opId + "-handle-timer-ns");
-    this.taskName = taskContext.getTaskModel().getTaskName();
 
-    this.eosStates = (EndOfStreamStates) taskContext.fetchObject(EndOfStreamStates.class.getName());
-    this.watermarkStates = (WatermarkStates) taskContext.fetchObject(WatermarkStates.class.getName());
-    this.controlMessageSender = new ControlMessageSender(taskContext.getStreamMetadataCache());
+    final TaskContext taskContext =  context.getTaskContext();
+    this.taskName = taskContext.getTaskModel().getTaskName();
+    this.eosStates = (EndOfStreamStates) internalTaskContext.fetchObject(EndOfStreamStates.class.getName());
+    this.watermarkStates = (WatermarkStates) internalTaskContext.fetchObject(WatermarkStates.class.getName());
+    this.controlMessageSender = new ControlMessageSender(internalTaskContext.getStreamMetadataCache());
     this.taskModel = taskContext.getTaskModel();
     this.callbackScheduler = taskContext.getCallbackScheduler();
     handleInit(context);
@@ -156,21 +164,13 @@ public abstract class OperatorImpl<M, RM> {
         || taskModel.getSystemStreamPartitions().stream().anyMatch(ssp -> ssp.getSystemStream().equals(input));
   }
 
-  /**
-   * Handle the incoming {@code message} for this {@link OperatorImpl} and propagate results to registered operators.
-   * <p>
-   * Delegates to {@link #handleMessage(Object, MessageCollector, TaskCoordinator)} for handling the message.
-   *
-   * @param message  the input message
-   * @param collector  the {@link MessageCollector} for this message
-   * @param coordinator  the {@link TaskCoordinator} for this message
-   */
-  public final void onMessage(M message, MessageCollector collector, TaskCoordinator coordinator) {
+  public final CompletionStage<Void> onMessageAsync(M message, MessageCollector collector,
+      TaskCoordinator coordinator) {
     this.numMessage.inc();
     long startNs = this.highResClock.nanoTime();
-    Collection<RM> results;
+    CompletionStage<Collection<RM>> completableResultsFuture;
     try {
-      results = handleMessage(message, collector, coordinator);
+      completableResultsFuture = handleMessageAsync(message, collector, coordinator);
     } catch (ClassCastException e) {
       String actualType = e.getMessage().replaceFirst(" cannot be cast to .*", "");
       String expectedType = e.getMessage().replaceFirst(".* cannot be cast to ", "");
@@ -181,30 +181,37 @@ public abstract class OperatorImpl<M, RM> {
               getOpImplId(), getOperatorSpec().getSourceLocation(), expectedType, actualType), e);
     }
 
-    long endNs = this.highResClock.nanoTime();
-    this.handleMessageNs.update(endNs - startNs);
+    CompletionStage<Void> result = completableResultsFuture.thenCompose(results -> {
+      long endNs = this.highResClock.nanoTime();
+      this.handleMessageNs.update(endNs - startNs);
 
-    results.forEach(rm ->
-        this.registeredOperators.forEach(op ->
-            op.onMessage(rm, collector, coordinator)));
+      return CompletableFuture.allOf(results.stream()
+          .flatMap(r -> this.registeredOperators.stream()
+            .map(op -> op.onMessageAsync(r, collector, coordinator)))
+          .toArray(CompletableFuture[]::new));
+    });
 
     WatermarkFunction watermarkFn = getOperatorSpec().getWatermarkFn();
     if (watermarkFn != null) {
       // check whether there is new watermark emitted from the user function
       Long outputWm = watermarkFn.getOutputWatermark();
-      propagateWatermark(outputWm, collector, coordinator);
+      return result.thenCompose(ignored -> propagateWatermark(outputWm, collector, coordinator));
     }
+
+    return result;
   }
 
   /**
-   * Handle the incoming {@code message} and return the results to be propagated to registered operators.
+   * Handle the incoming {@code message} asynchronously and return a {@link CompletionStage} of the results to be propagated
+   * to the registered operators.
    *
-   * @param message  the input message
-   * @param collector  the {@link MessageCollector} in the context
-   * @param coordinator  the {@link TaskCoordinator} in the context
-   * @return  results of the transformation
+   * @param message the input message
+   * @param collector the {@link MessageCollector} in the context
+   * @param coordinator the {@link TaskCoordinator} in the context
+   *
+   * @return a {@code CompletionStage} of the results of the transformation
    */
-  protected abstract Collection<RM> handleMessage(M message, MessageCollector collector,
+  protected abstract CompletionStage<Collection<RM>> handleMessageAsync(M message, MessageCollector collector,
       TaskCoordinator coordinator);
 
   /**
@@ -215,17 +222,23 @@ public abstract class OperatorImpl<M, RM> {
    * @param collector  the {@link MessageCollector} in the context
    * @param coordinator  the {@link TaskCoordinator} in the context
    */
-  public final void onTimer(MessageCollector collector, TaskCoordinator coordinator) {
+  public final CompletionStage<Void> onTimer(MessageCollector collector, TaskCoordinator coordinator) {
     long startNs = this.highResClock.nanoTime();
     Collection<RM> results = handleTimer(collector, coordinator);
     long endNs = this.highResClock.nanoTime();
     this.handleTimerNs.update(endNs - startNs);
 
-    results.forEach(rm ->
-        this.registeredOperators.forEach(op ->
-            op.onMessage(rm, collector, coordinator)));
-    this.registeredOperators.forEach(op ->
-        op.onTimer(collector, coordinator));
+    CompletionStage<Void> resultFuture = CompletableFuture.allOf(
+        results.stream()
+            .flatMap(r -> this.registeredOperators.stream()
+                .map(op -> op.onMessageAsync(r, collector, coordinator)))
+            .toArray(CompletableFuture[]::new));
+
+    return resultFuture.thenCompose(x ->
+        CompletableFuture.allOf(this.registeredOperators
+            .stream()
+            .map(op -> op.onTimer(collector, coordinator))
+            .toArray(CompletableFuture[]::new)));
   }
 
   /**
@@ -249,12 +262,14 @@ public abstract class OperatorImpl<M, RM> {
    * @param collector message collector
    * @param coordinator task coordinator
    */
-  public final void aggregateEndOfStream(EndOfStreamMessage eos, SystemStreamPartition ssp, MessageCollector collector,
+  public final CompletionStage<Void> aggregateEndOfStream(EndOfStreamMessage eos, SystemStreamPartition ssp, MessageCollector collector,
       TaskCoordinator coordinator) {
     LOG.info("Received end-of-stream message from task {} in {}", eos.getTaskName(), ssp);
     eosStates.update(eos, ssp);
 
     SystemStream stream = ssp.getSystemStream();
+    CompletionStage<Void> endOfStreamFuture = CompletableFuture.completedFuture(null);
+
     if (eosStates.isEndOfStream(stream)) {
       LOG.info("Input {} reaches the end for task {}", stream.toString(), taskName.getTaskName());
       if (eos.getTaskName() != null) {
@@ -262,16 +277,20 @@ public abstract class OperatorImpl<M, RM> {
         // broadcast the end-of-stream to all the peer partitions
         controlMessageSender.broadcastToOtherPartitions(new EndOfStreamMessage(), ssp, collector);
       }
-      // populate the end-of-stream through the dag
-      onEndOfStream(collector, coordinator);
 
-      if (eosStates.allEndOfStream()) {
-        // all inputs have been end-of-stream, shut down the task
-        LOG.info("All input streams have reached the end for task {}", taskName.getTaskName());
-        coordinator.commit(TaskCoordinator.RequestScope.CURRENT_TASK);
-        coordinator.shutdown(TaskCoordinator.RequestScope.CURRENT_TASK);
-      }
+      // populate the end-of-stream through the dag
+      endOfStreamFuture = onEndOfStream(collector, coordinator)
+          .thenAccept(result -> {
+            if (eosStates.allEndOfStream()) {
+              // all inputs have been end-of-stream, shut down the task
+              LOG.info("All input streams have reached the end for task {}", taskName.getTaskName());
+              coordinator.commit(TaskCoordinator.RequestScope.CURRENT_TASK);
+              coordinator.shutdown(TaskCoordinator.RequestScope.CURRENT_TASK);
+            }
+          });
     }
+
+    return endOfStreamFuture;
   }
 
   /**
@@ -280,16 +299,25 @@ public abstract class OperatorImpl<M, RM> {
    * @param collector message collector
    * @param coordinator task coordinator
    */
-  private final void onEndOfStream(MessageCollector collector, TaskCoordinator coordinator) {
+  private CompletionStage<Void> onEndOfStream(MessageCollector collector, TaskCoordinator coordinator) {
+    CompletionStage<Void> endOfStreamFuture = CompletableFuture.completedFuture(null);
+
     if (inputStreams.stream().allMatch(input -> eosStates.isEndOfStream(input))) {
       Collection<RM> results = handleEndOfStream(collector, coordinator);
 
-      results.forEach(rm ->
-          this.registeredOperators.forEach(op ->
-              op.onMessage(rm, collector, coordinator)));
+      CompletionStage<Void> resultFuture = CompletableFuture.allOf(
+          results.stream()
+              .flatMap(r -> this.registeredOperators.stream()
+                  .map(op -> op.onMessageAsync(r, collector, coordinator)))
+              .toArray(CompletableFuture[]::new));
 
-      this.registeredOperators.forEach(op -> op.onEndOfStream(collector, coordinator));
+      endOfStreamFuture = resultFuture.thenCompose(x ->
+          CompletableFuture.allOf(this.registeredOperators.stream()
+              .map(op -> op.onEndOfStream(collector, coordinator))
+              .toArray(CompletableFuture[]::new)));
     }
+
+    return endOfStreamFuture;
   }
 
   /**
@@ -313,11 +341,13 @@ public abstract class OperatorImpl<M, RM> {
    * @param collector message collector
    * @param coordinator task coordinator
    */
-  public final void aggregateWatermark(WatermarkMessage watermarkMessage, SystemStreamPartition ssp,
+  public final CompletionStage<Void> aggregateWatermark(WatermarkMessage watermarkMessage, SystemStreamPartition ssp,
       MessageCollector collector, TaskCoordinator coordinator) {
     LOG.debug("Received watermark {} from {}", watermarkMessage.getTimestamp(), ssp);
     watermarkStates.update(watermarkMessage, ssp);
     long watermark = watermarkStates.getWatermark(ssp.getSystemStream());
+    CompletionStage<Void> watermarkFuture = CompletableFuture.completedFuture(null);
+
     if (currentWatermark < watermark) {
       LOG.debug("Got watermark {} from stream {}", watermark, ssp.getSystemStream());
 
@@ -327,11 +357,11 @@ public abstract class OperatorImpl<M, RM> {
         controlMessageSender.broadcastToOtherPartitions(new WatermarkMessage(watermark), ssp, collector);
       }
       // populate the watermark through the dag
-      onWatermark(watermark, collector, coordinator);
-
-      // update metrics
-      watermarkStates.updateAggregateMetric(ssp, watermark);
+      watermarkFuture = onWatermark(watermark, collector, coordinator)
+          .thenAccept(ignored -> watermarkStates.updateAggregateMetric(ssp, watermark));
     }
+
+    return watermarkFuture;
   }
 
   /**
@@ -342,16 +372,20 @@ public abstract class OperatorImpl<M, RM> {
    * @param collector message collector
    * @param coordinator task coordinator
    */
-  private final void onWatermark(long watermark, MessageCollector collector, TaskCoordinator coordinator) {
+  private CompletionStage<Void> onWatermark(long watermark, MessageCollector collector, TaskCoordinator coordinator) {
     final long inputWatermarkMin;
     if (prevOperators.isEmpty()) {
       // for input operator, use the watermark time coming from the source input
       inputWatermarkMin = watermark;
     } else {
       // InputWatermark(op) = min { OutputWatermark(op') | op' is upstream of op}
-      inputWatermarkMin = prevOperators.stream().map(op -> op.getOutputWatermark()).min(Long::compare).get();
+      inputWatermarkMin = prevOperators.stream()
+          .map(op -> op.getOutputWatermark())
+          .min(Long::compare)
+          .get();
     }
 
+    CompletionStage<Void> watermarkFuture = CompletableFuture.completedFuture(null);
     if (currentWatermark < inputWatermarkMin) {
       // advance the watermark time of this operator
       currentWatermark = inputWatermarkMin;
@@ -372,26 +406,38 @@ public abstract class OperatorImpl<M, RM> {
       }
 
       if (!output.isEmpty()) {
-        output.forEach(rm ->
-            this.registeredOperators.forEach(op ->
-                op.onMessage(rm, collector, coordinator)));
+        watermarkFuture = CompletableFuture.allOf(
+            output.stream()
+                .flatMap(rm -> this.registeredOperators.stream()
+                    .map(op -> op.onMessageAsync(rm, collector, coordinator)))
+                .toArray(CompletableFuture[]::new));
       }
 
-      propagateWatermark(outputWm, collector, coordinator);
+      watermarkFuture = watermarkFuture.thenCompose(res -> propagateWatermark(outputWm, collector, coordinator));
     }
+
+    return watermarkFuture;
   }
 
-  private void propagateWatermark(Long outputWm, MessageCollector collector, TaskCoordinator coordinator) {
+  private CompletionStage<Void> propagateWatermark(Long outputWm, MessageCollector collector, TaskCoordinator coordinator) {
+    CompletionStage<Void> watermarkFuture = CompletableFuture.completedFuture(null);
+
     if (outputWm != null) {
       if (outputWatermark < outputWm) {
         // advance the watermark
         outputWatermark = outputWm;
         LOG.debug("Advance output watermark to {} in operator {}", outputWatermark, getOpImplId());
-        this.registeredOperators.forEach(op -> op.onWatermark(outputWatermark, collector, coordinator));
+        watermarkFuture = CompletableFuture.allOf(
+            this.registeredOperators
+                .stream()
+                .map(op -> op.onWatermark(outputWatermark, collector, coordinator))
+                .toArray(CompletableFuture[]::new));
       } else if (outputWatermark > outputWm) {
         LOG.warn("Ignore watermark {} that is smaller than the previous watermark {}.", outputWm, outputWatermark);
       }
     }
+
+    return watermarkFuture;
   }
 
   /**
@@ -439,21 +485,24 @@ public abstract class OperatorImpl<M, RM> {
       @Override
       public void schedule(K key, long time) {
         callbackScheduler.scheduleCallback(key, time, (k, collector, coordinator) -> {
-            final ScheduledFunction<K, RM> scheduledFn = getOperatorSpec().getScheduledFn();
-            if (scheduledFn != null) {
-              final Collection<RM> output = scheduledFn.onCallback(key, time);
+          final ScheduledFunction<K, RM> scheduledFn = getOperatorSpec().getScheduledFn();
+          if (scheduledFn != null) {
+            final Collection<RM> output = scheduledFn.onCallback(key, time);
 
-              if (!output.isEmpty()) {
-                output.forEach(rm ->
-                    registeredOperators.forEach(op ->
-                        op.onMessage(rm, collector, coordinator)));
-              }
-            } else {
-              throw new SamzaException(
-                  String.format("Operator %s id %s (created at %s) must implement ScheduledFunction to use system timer.",
-                      getOperatorSpec().getOpCode().name(), getOpImplId(), getOperatorSpec().getSourceLocation()));
+            if (!output.isEmpty()) {
+              CompletableFuture<Void> timerFuture = CompletableFuture.allOf(output.stream()
+                  .flatMap(r -> registeredOperators.stream()
+                      .map(op -> op.onMessageAsync(r, collector, coordinator)))
+                  .toArray(CompletableFuture[]::new));
+
+              timerFuture.join();
             }
-          });
+          } else {
+            throw new SamzaException(
+                String.format("Operator %s id %s (created at %s) must implement ScheduledFunction to use system timer.",
+                    getOperatorSpec().getOpCode().name(), getOpImplId(), getOperatorSpec().getSourceLocation()));
+          }
+        });
       }
 
       @Override
@@ -494,8 +543,29 @@ public abstract class OperatorImpl<M, RM> {
     return getOperatorSpec().getOpId();
   }
 
+  /* Package Private helper method for tests to perform onMessage synchronously
+   * Note: It is only intended for test use
+   */
+  @VisibleForTesting
+  final void onMessage(M message, MessageCollector collector, TaskCoordinator coordinator) {
+    onMessageAsync(message, collector, coordinator)
+        .toCompletableFuture().join();
+  }
+
+  /* Package Private helper method for tests to perform handleMessage synchronously
+   * Note: It is only intended for test use
+   */
+  @VisibleForTesting
+  final Collection<RM> handleMessage(M message, MessageCollector collector, TaskCoordinator coordinator) {
+    return handleMessageAsync(message, collector, coordinator)
+        .toCompletableFuture().join();
+  }
+
   private HighResolutionClock createHighResClock(Config config) {
-    if (new MetricsConfig(config).getMetricsTimerEnabled()) {
+    MetricsConfig metricsConfig = new MetricsConfig(config);
+    // The timer metrics calculation here is only enabled for debugging
+    if (metricsConfig.getMetricsTimerEnabled()
+        && metricsConfig.getMetricsTimerDebugEnabled()) {
       return System::nanoTime;
     } else {
       return () -> 0;
