@@ -18,11 +18,14 @@
  */
 package org.apache.samza.operators.impl;
 
+import java.util.Optional;
+import java.util.concurrent.CompletableFuture;
+import java.util.concurrent.CompletionStage;
 import org.apache.samza.context.Context;
 import org.apache.samza.operators.KV;
 import org.apache.samza.operators.spec.OperatorSpec;
 import org.apache.samza.operators.spec.StreamTableJoinOperatorSpec;
-import org.apache.samza.table.ReadableTable;
+import org.apache.samza.table.ReadWriteTable;
 import org.apache.samza.task.MessageCollector;
 import org.apache.samza.task.TaskCoordinator;
 
@@ -42,11 +45,11 @@ import java.util.Collections;
 class StreamTableJoinOperatorImpl<K, M, R extends KV, JM> extends OperatorImpl<M, JM> {
 
   private final StreamTableJoinOperatorSpec<K, M, R, JM> joinOpSpec;
-  private final ReadableTable<K, ?> table;
+  private final ReadWriteTable<K, ?> table;
 
   StreamTableJoinOperatorImpl(StreamTableJoinOperatorSpec<K, M, R, JM> joinOpSpec, Context context) {
     this.joinOpSpec = joinOpSpec;
-    this.table = (ReadableTable) context.getTaskContext().getTable(joinOpSpec.getTableSpec().getId());
+    this.table = context.getTaskContext().getTable(joinOpSpec.getTableId());
   }
 
   @Override
@@ -55,17 +58,30 @@ class StreamTableJoinOperatorImpl<K, M, R extends KV, JM> extends OperatorImpl<M
   }
 
   @Override
-  public Collection<JM> handleMessage(M message, MessageCollector collector, TaskCoordinator coordinator) {
+  protected CompletionStage<Collection<JM>> handleMessageAsync(M message, MessageCollector collector,
+      TaskCoordinator coordinator) {
+    if (message == null) {
+      return CompletableFuture.completedFuture(Collections.emptyList());
+    }
+
     K key = joinOpSpec.getJoinFn().getMessageKey(message);
-    Object recordValue = table.get(key);
-    R record = recordValue != null ? (R) KV.of(key, recordValue) : null;
+    Object[] args = joinOpSpec.getArgs();
+
+    return Optional.ofNullable(key)
+        .map(joinKey -> table.getAsync(joinKey, args)
+            .thenApply(val -> getJoinOutput(joinKey, val, message)))
+        .orElseGet(() -> CompletableFuture.completedFuture(getJoinOutput(key, null, message)));
+  }
+
+  private Collection<JM> getJoinOutput(K key, Object value, M message) {
+    R record = value == null ? null : (R) KV.of(key, value);
+
     JM output = joinOpSpec.getJoinFn().apply(message, record);
 
     // The support for inner and outer join will be provided in the jonFn. For inner join, the joinFn might
     // return null, when the corresponding record is absent in the table.
     return output != null ?
-        Collections.singletonList(output)
-      : Collections.emptyList();
+        Collections.singletonList(output) : Collections.emptyList();
   }
 
   @Override
