@@ -20,7 +20,7 @@
 
 package org.apache.samza.operators.spec;
 
-import org.apache.samza.operators.functions.TimerFunction;
+import org.apache.samza.operators.functions.ScheduledFunction;
 import org.apache.samza.operators.functions.WatermarkFunction;
 
 public class BroadcastOperatorSpec<M> extends OperatorSpec<M, Void> {
@@ -43,7 +43,7 @@ public class BroadcastOperatorSpec<M> extends OperatorSpec<M, Void> {
   }
 
   @Override
-  public TimerFunction getTimerFn() {
+  public ScheduledFunction getScheduledFn() {
     return null;
   }
 }
