@@ -22,8 +22,6 @@ import org.apache.samza.annotation.InterfaceStability;
 import org.apache.samza.operators.functions.ScheduledFunction;
 import org.apache.samza.operators.functions.StreamTableJoinFunction;
 import org.apache.samza.operators.functions.WatermarkFunction;
-import org.apache.samza.table.TableSpec;
-
 
 /**
  * The spec for stream-table join operator that retrieves a record from the table using key
@@ -36,30 +34,37 @@ import org.apache.samza.table.TableSpec;
 @InterfaceStability.Unstable
 public class StreamTableJoinOperatorSpec<K, M, R, JM> extends OperatorSpec<M, JM> {
 
-  private final TableSpec tableSpec;
+  private final String tableId;
   private final StreamTableJoinFunction<K, M, R, JM> joinFn;
+  private final Object[] args;
 
   /**
    * Constructor for {@link StreamTableJoinOperatorSpec}.
    *
-   * @param tableSpec  the table spec for the table on the right side of the join
+   * @param tableId  the Id of the table on the right side of the join
    * @param joinFn  the user-defined join function to get join keys and results
    * @param opId  the unique ID for this operator
+   * @param args additional arguments passed to the table
    */
-  StreamTableJoinOperatorSpec(TableSpec tableSpec, StreamTableJoinFunction<K, M, R, JM> joinFn, String opId) {
+  StreamTableJoinOperatorSpec(String tableId, StreamTableJoinFunction<K, M, R, JM> joinFn, String opId, Object ... args) {
     super(OpCode.JOIN, opId);
-    this.tableSpec = tableSpec;
+    this.tableId = tableId;
     this.joinFn = joinFn;
+    this.args = args;
   }
 
-  public TableSpec getTableSpec() {
-    return tableSpec;
+  public String getTableId() {
+    return tableId;
   }
 
   public StreamTableJoinFunction<K, M, R, JM> getJoinFn() {
     return this.joinFn;
   }
 
+  public Object[] getArgs() {
+    return args;
+  }
+
   @Override
   public WatermarkFunction getWatermarkFn() {
     return joinFn instanceof WatermarkFunction ? (WatermarkFunction) joinFn : null;
