@@ -23,7 +23,7 @@ import java.util.Map;
 import java.util.Set;
 import org.apache.samza.SamzaException;
 import org.apache.samza.config.Config;
-import org.apache.samza.config.JavaSystemConfig;
+import org.apache.samza.config.SystemConfig;
 import org.apache.samza.config.MapConfig;
 
 
@@ -34,10 +34,15 @@ public class SystemAdmins {
   private final Map<String, SystemAdmin> systemAdminMap;
 
   public SystemAdmins(Config config) {
-    JavaSystemConfig systemConfig = new JavaSystemConfig(config);
+    SystemConfig systemConfig = new SystemConfig(config);
     this.systemAdminMap = systemConfig.getSystemAdmins();
   }
 
+  public SystemAdmins(Config config, String adminLabel) {
+    SystemConfig systemConfig = new SystemConfig(config);
+    this.systemAdminMap = systemConfig.getSystemAdmins(adminLabel);
+  }
+
   /**
    * Creates a new instance of {@link SystemAdmins} with an empty admin mapping.
    * @return New empty instance of {@link SystemAdmins}
