@@ -20,7 +20,9 @@
 package org.apache.samza.system.inmemory;
 
 import com.google.common.base.Preconditions;
+import java.util.Arrays;
 import org.apache.samza.Partition;
+import org.apache.samza.system.IncomingMessageEnvelope;
 import org.apache.samza.system.OutgoingMessageEnvelope;
 import org.apache.samza.system.SystemProducer;
 import org.apache.samza.system.SystemStreamPartition;
@@ -89,12 +91,26 @@ public class InMemorySystemProducer implements SystemProducer {
     Preconditions.checkNotNull(partitionKey, "Failed to compute partition key for the message: " + envelope);
 
     int partition =
-        Math.abs(partitionKey.hashCode()) % memoryManager.getPartitionCountForSystemStream(envelope.getSystemStream());
+        Math.abs(hashCode(partitionKey)) % memoryManager.getPartitionCountForSystemStream(envelope.getSystemStream());
 
     SystemStreamPartition ssp = new SystemStreamPartition(envelope.getSystemStream(), new Partition(partition));
     memoryManager.put(ssp, key, message);
   }
 
+  /**
+   * Populates the IME to the ssp configured, this gives user more control to set up Test environment partition.
+   * The offset in the envelope needs to adhere to a rule that for messages in the same system stream partition the
+   * offset needs to start at 0 for the first and be monotonically increasing for the following messages.
+   * If not the {@link InMemoryManager#put(SystemStreamPartition, IncomingMessageEnvelope)} will fail.
+   *
+   * Note: Please DO NOT use this in production use cases, this is only meant to set-up more flexible tests.
+   * This function is not thread safe.
+   * @param envelope incoming message envelope
+   */
+  public void send(IncomingMessageEnvelope envelope) {
+    memoryManager.put(envelope.getSystemStreamPartition(), envelope);
+  }
+
   /**
    * If the SystemProducer buffers messages before sending them to its underlying system, it should flush those
    * messages and leave no messages remaining to be sent.
@@ -106,4 +122,12 @@ public class InMemorySystemProducer implements SystemProducer {
   public void flush(String source) {
     // nothing to do
   }
+
+  /**
+   * Return the hash code of the partitionKey. When partitionKey is a byte array, it returns a hash code based on
+   * the contents of the byte array. This guarantees that byte arrays with same contents get the same hash code.
+   */
+  private int hashCode(Object partitionKey) {
+    return (partitionKey instanceof byte[]) ? Arrays.hashCode((byte[]) partitionKey) : partitionKey.hashCode();
+  }
 }
