@@ -32,7 +32,7 @@ import org.apache.samza.SamzaException;
 import org.apache.samza.container.TaskName;
 import org.apache.samza.system.IncomingMessageEnvelope;
 import org.apache.samza.util.HighResolutionClock;
-import org.apache.samza.util.Util;
+import org.apache.samza.util.ThreadUtil;
 
 
 /**
@@ -41,7 +41,7 @@ import org.apache.samza.util.Util;
  * for the callbacks based on the sequence number, and updates the offsets for checkpointing
  * by always moving forward to the latest contiguous callback (uses the high watermark).
  */
-class TaskCallbackManager {
+public class TaskCallbackManager {
 
   private static final class TaskCallbacks {
     private final Queue<TaskCallbackImpl> callbacks = new PriorityQueue<>();
@@ -101,7 +101,7 @@ class TaskCallbackManager {
       Runnable timerTask = new Runnable() {
         @Override
         public void run() {
-          Util.logThreadDump("Thread dump at task callback timeout");
+          ThreadUtil.logThreadDump("Thread dump at task callback timeout");
           String msg = "Callback for task {} " + callback.taskName + " timed out after " + timeout + " ms.";
           callback.failure(new SamzaException(msg));
         }
