@@ -19,12 +19,14 @@
 package org.apache.samza.util;
 
 import java.util.HashMap;
+import java.util.List;
 import java.util.Map;
+import java.util.stream.Collectors;
 import org.apache.samza.SamzaException;
 import org.apache.samza.config.MetricsConfig;
 import org.apache.samza.metrics.MetricsReporter;
 import org.apache.samza.metrics.MetricsReporterFactory;
-import scala.collection.JavaConverters;
+
 
 /**
  * Helper class that instantiates the MetricsReporter.
@@ -34,19 +36,28 @@ public class MetricsReporterLoader {
   private MetricsReporterLoader() {
   }
 
-  public static Map<String, MetricsReporter> getMetricsReporters(MetricsConfig config, String containerName) {
+  public static Map<String, MetricsReporter> getMetricsReporters(MetricsConfig metricsConfig, String containerName) {
     Map<String, MetricsReporter> metricsReporters = new HashMap<>();
 
-    for (String metricsReporterName : JavaConverters.seqAsJavaListConverter(config.getMetricReporterNames()).asJava()) {
-      String metricsFactoryClassName = config.getMetricsFactoryClass(metricsReporterName).get();
-      if (metricsFactoryClassName == null) {
-        throw new SamzaException(String.format("Metrics reporter %s missing .class config", metricsReporterName));
-      }
-      MetricsReporterFactory metricsReporterFactory = Util.getObj(metricsFactoryClassName, MetricsReporterFactory.class);
+    String diagnosticsReporterName = MetricsConfig.METRICS_SNAPSHOT_REPORTER_NAME_FOR_DIAGNOSTICS;
+
+    // Exclude creation of diagnostics-reporter, because it is created manually in SamzaContainer (to allow sharing of
+    // sysProducer between reporter and diagnosticsManager
+    List<String> metricsReporterNames = metricsConfig.getMetricReporterNames()
+        .stream()
+        .filter(reporterName -> !reporterName.equals(diagnosticsReporterName))
+        .collect(Collectors.toList());
+
+    for (String metricsReporterName : metricsReporterNames) {
+      String metricsFactoryClassName = metricsConfig.getMetricsFactoryClass(metricsReporterName)
+          .orElseThrow(() -> new SamzaException(
+              String.format("Metrics reporter %s missing .class config", metricsReporterName)));
+      MetricsReporterFactory metricsReporterFactory =
+          ReflectionUtil.getObj(metricsFactoryClassName, MetricsReporterFactory.class);
       metricsReporters.put(metricsReporterName,
                            metricsReporterFactory.getMetricsReporter(metricsReporterName,
                                                                      containerName,
-                                                                     config));
+                                                                     metricsConfig));
     }
     return metricsReporters;
   }
