@@ -19,43 +19,54 @@
 package org.apache.samza.zk;
 
 import com.google.common.annotations.VisibleForTesting;
-
+import com.google.common.base.Preconditions;
+import java.io.IOException;
 import java.util.ArrayList;
-import java.util.HashSet;
 import java.util.HashMap;
+import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
 import java.util.Objects;
 import java.util.Set;
 import java.util.concurrent.atomic.AtomicBoolean;
 import org.I0Itec.zkclient.IZkStateListener;
-import org.apache.commons.lang3.StringUtils;
-import org.apache.samza.checkpoint.CheckpointManager;
-import org.apache.samza.config.ApplicationConfig;
+import org.apache.samza.SamzaException;
 import org.apache.samza.config.Config;
-import org.apache.samza.config.ConfigException;
 import org.apache.samza.config.JobConfig;
 import org.apache.samza.config.MapConfig;
-import org.apache.samza.config.MetricsConfig;
-import org.apache.samza.config.TaskConfigJava;
+import org.apache.samza.config.StorageConfig;
+import org.apache.samza.config.TaskConfig;
 import org.apache.samza.config.ZkConfig;
 import org.apache.samza.container.TaskName;
+import org.apache.samza.container.grouper.task.GrouperMetadata;
+import org.apache.samza.container.grouper.task.GrouperMetadataImpl;
 import org.apache.samza.coordinator.JobCoordinator;
 import org.apache.samza.coordinator.JobCoordinatorListener;
 import org.apache.samza.coordinator.JobModelManager;
 import org.apache.samza.coordinator.LeaderElectorListener;
+import org.apache.samza.coordinator.MetadataResourceUtil;
+import org.apache.samza.coordinator.StreamPartitionCountMonitor;
+import org.apache.samza.coordinator.metadatastore.CoordinatorStreamStore;
+import org.apache.samza.coordinator.metadatastore.NamespaceAwareCoordinatorStreamStore;
+import org.apache.samza.coordinator.stream.CoordinatorStreamValueSerde;
+import org.apache.samza.coordinator.stream.messages.SetConfig;
 import org.apache.samza.job.model.ContainerModel;
 import org.apache.samza.job.model.JobModel;
+import org.apache.samza.job.model.JobModelUtil;
+import org.apache.samza.job.model.TaskModel;
+import org.apache.samza.metadatastore.MetadataStore;
 import org.apache.samza.metrics.MetricsRegistry;
-import org.apache.samza.metrics.MetricsReporter;
-import org.apache.samza.metrics.ReadableMetricsRegistry;
-import org.apache.samza.runtime.ProcessorIdGenerator;
-import org.apache.samza.storage.ChangelogStreamManager;
+import org.apache.samza.runtime.LocationId;
+import org.apache.samza.runtime.LocationIdProvider;
+import org.apache.samza.runtime.LocationIdProviderFactory;
+import org.apache.samza.startpoint.StartpointManager;
 import org.apache.samza.system.StreamMetadataCache;
 import org.apache.samza.system.SystemAdmins;
-import org.apache.samza.util.MetricsReporterLoader;
+import org.apache.samza.system.SystemStream;
+import org.apache.samza.system.SystemStreamPartition;
+import org.apache.samza.util.ReflectionUtil;
 import org.apache.samza.util.SystemClock;
-import org.apache.samza.util.Util;
+import org.apache.samza.zk.ZkUtils.ProcessorNode;
 import org.apache.zookeeper.Watcher;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
@@ -89,28 +100,39 @@ public class ZkJobCoordinator implements JobCoordinator {
   private final Config config;
   private final ZkBarrierForVersionUpgrade barrier;
   private final ZkJobCoordinatorMetrics metrics;
-  private final Map<String, MetricsReporter> reporters;
   private final ZkLeaderElector leaderElector;
   private final AtomicBoolean initiatedShutdown = new AtomicBoolean(false);
   private final StreamMetadataCache streamMetadataCache;
   private final SystemAdmins systemAdmins;
   private final int debounceTimeMs;
   private final Map<TaskName, Integer> changeLogPartitionMap = new HashMap<>();
+  private final LocationId locationId;
+  private final MetadataStore jobModelMetadataStore;
+  private final CoordinatorStreamStore coordinatorStreamStore;
 
   private JobCoordinatorListener coordinatorListener = null;
   private JobModel newJobModel;
-  private boolean hasCreatedStreams = false;
+  private boolean hasLoadedMetadataResources = false;
   private String cachedJobModelVersion = null;
 
+  @VisibleForTesting
+  ZkSessionMetrics zkSessionMetrics;
+
   @VisibleForTesting
   ScheduleAfterDebounceTime debounceTimer;
 
-  ZkJobCoordinator(Config config, MetricsRegistry metricsRegistry, ZkUtils zkUtils) {
+  @VisibleForTesting
+  StreamPartitionCountMonitor streamPartitionCountMonitor = null;
+
+  ZkJobCoordinator(String processorId, Config config, MetricsRegistry metricsRegistry, ZkUtils zkUtils, MetadataStore jobModelMetadataStore, MetadataStore coordinatorStreamStore) {
+    // TODO: When we consolidate metadata stores for standalone, this check can be removed. For now, we expect this type.
+    //   Keeping method signature as MetadataStore to avoid public API changes in the future
+    Preconditions.checkArgument(coordinatorStreamStore instanceof CoordinatorStreamStore);
+
     this.config = config;
-
     this.metrics = new ZkJobCoordinatorMetrics(metricsRegistry);
-
-    this.processorId = createProcessorId(config);
+    this.zkSessionMetrics = new ZkSessionMetrics(metricsRegistry);
+    this.processorId = processorId;
     this.zkUtils = zkUtils;
     // setup a listener for a session state change
     // we are mostly interested in "session closed" and "new session created" events
@@ -118,25 +140,29 @@ public class ZkJobCoordinator implements JobCoordinator {
     leaderElector = new ZkLeaderElector(processorId, zkUtils);
     leaderElector.setLeaderElectorListener(new LeaderElectorListenerImpl());
     this.debounceTimeMs = new JobConfig(config).getDebounceTimeMs();
-    this.reporters = MetricsReporterLoader.getMetricsReporters(new MetricsConfig(config), processorId);
     debounceTimer = new ScheduleAfterDebounceTime(processorId);
     debounceTimer.setScheduledTaskCallback(throwable -> {
-        LOG.error("Received exception in debounce timer! Stopping the job coordinator", throwable);
-        stop();
-      });
+      LOG.error("Received exception in debounce timer! Stopping the job coordinator", throwable);
+      stop();
+    });
     this.barrier =  new ZkBarrierForVersionUpgrade(zkUtils.getKeyBuilder().getJobModelVersionBarrierPrefix(), zkUtils, new ZkBarrierListenerImpl(), debounceTimer);
-    systemAdmins = new SystemAdmins(config);
+    systemAdmins = new SystemAdmins(config, this.getClass().getSimpleName());
     streamMetadataCache = new StreamMetadataCache(systemAdmins, METADATA_CACHE_TTL_MS, SystemClock.instance());
+    LocationIdProviderFactory locationIdProviderFactory =
+        ReflectionUtil.getObj(new JobConfig(config).getLocationIdProviderFactory(), LocationIdProviderFactory.class);
+    LocationIdProvider locationIdProvider = locationIdProviderFactory.getLocationIdProvider(config);
+    this.locationId = locationIdProvider.getLocationId();
+    this.coordinatorStreamStore = (CoordinatorStreamStore) coordinatorStreamStore;
+    this.jobModelMetadataStore = jobModelMetadataStore;
   }
 
   @Override
   public void start() {
     ZkKeyBuilder keyBuilder = zkUtils.getKeyBuilder();
     zkUtils.validateZkVersion();
-    zkUtils.validatePaths(new String[]{keyBuilder.getProcessorsPath(), keyBuilder.getJobModelVersionPath(), keyBuilder
-        .getJobModelPathPrefix()});
+    zkUtils.validatePaths(new String[]{keyBuilder.getProcessorsPath(), keyBuilder.getJobModelVersionPath(), keyBuilder.getJobModelPathPrefix(), keyBuilder.getTaskLocalityPath()});
 
-    startMetrics();
+    this.jobModelMetadataStore.init();
     systemAdmins.start();
     leaderElector.tryBecomeLeader();
     zkUtils.subscribeToJobModelVersionChange(new ZkJobModelVersionChangeHandler(zkUtils));
@@ -152,7 +178,7 @@ public class ZkJobCoordinator implements JobCoordinator {
 
       // Notify the metrics about abandoning the leadership. Moving it up the chain in the shutdown sequence so that
       // in case of unclean shutdown, we get notified about lack of leader and we can set up some alerts around the absence of leader.
-      metrics.isLeader.set(false);
+      metrics.isLeader.set(0);
 
       try {
         // todo: what does it mean for coordinator listener to be null? why not have it part of constructor?
@@ -176,13 +202,15 @@ public class ZkJobCoordinator implements JobCoordinator {
         LOG.debug("Shutting down system admins.");
         systemAdmins.stop();
 
-        LOG.debug("Shutting down metrics.");
-        shutdownMetrics();
+        if (streamPartitionCountMonitor != null) {
+          streamPartitionCountMonitor.stop();
+        }
 
         if (coordinatorListener != null) {
           coordinatorListener.onCoordinatorStop();
         }
 
+        jobModelMetadataStore.close();
         shutdownSuccessful = true;
       } catch (Throwable t) {
         LOG.error("Encountered errors during job coordinator stop.", t);
@@ -197,19 +225,6 @@ public class ZkJobCoordinator implements JobCoordinator {
     }
   }
 
-  private void startMetrics() {
-    for (MetricsReporter reporter: reporters.values()) {
-      reporter.register("job-coordinator-" + processorId, (ReadableMetricsRegistry) metrics.getMetricsRegistry());
-      reporter.start();
-    }
-  }
-
-  private void shutdownMetrics() {
-    for (MetricsReporter reporter: reporters.values()) {
-      reporter.stop();
-    }
-  }
-
   @Override
   public void setListener(JobCoordinatorListener listener) {
     this.coordinatorListener = listener;
@@ -233,14 +248,18 @@ public class ZkJobCoordinator implements JobCoordinator {
   public void onProcessorChange(List<String> processors) {
     if (leaderElector.amILeader()) {
       LOG.info("ZkJobCoordinator::onProcessorChange - list of processors changed. List size=" + processors.size());
-      debounceTimer.scheduleAfterDebounceTime(ON_PROCESSOR_CHANGE, debounceTimeMs, () -> doOnProcessorChange(processors));
+      debounceTimer.scheduleAfterDebounceTime(ON_PROCESSOR_CHANGE, debounceTimeMs, this::doOnProcessorChange);
     }
   }
 
-  void doOnProcessorChange(List<String> processors) {
-    // if list of processors is empty - it means we are called from 'onBecomeLeader'
-    // TODO: Handle empty currentProcessorIds.
-    List<String> currentProcessorIds = zkUtils.getSortedActiveProcessorsIDs();
+  void doOnProcessorChange() {
+    List<ProcessorNode> processorNodes = zkUtils.getAllProcessorNodes();
+
+    List<String> currentProcessorIds = new ArrayList<>();
+    for (ProcessorNode processorNode : processorNodes) {
+      currentProcessorIds.add(processorNode.getProcessorData().getProcessorId());
+    }
+
     Set<String> uniqueProcessorIds = new HashSet<>(currentProcessorIds);
 
     if (currentProcessorIds.size() != uniqueProcessorIds.size()) {
@@ -250,18 +269,12 @@ public class ZkJobCoordinator implements JobCoordinator {
 
     // Generate the JobModel
     LOG.info("Generating new JobModel with processors: {}.", currentProcessorIds);
-    JobModel jobModel = generateNewJobModel(currentProcessorIds);
+    JobModel jobModel = generateNewJobModel(processorNodes);
 
     // Create checkpoint and changelog streams if they don't exist
-    if (!hasCreatedStreams) {
-      CheckpointManager checkpointManager = new TaskConfigJava(config).getCheckpointManager(metrics.getMetricsRegistry());
-      if (checkpointManager != null) {
-        checkpointManager.createResources();
-      }
-
-      // Pass in null Coordinator consumer and producer because ZK doesn't have coordinator streams.
-      ChangelogStreamManager.createChangelogStreams(config, jobModel.maxChangeLogStreamPartitions);
-      hasCreatedStreams = true;
+    if (!hasLoadedMetadataResources) {
+      loadMetadataResources(jobModel);
+      hasLoadedMetadataResources = true;
     }
 
     // Assign the next version of JobModel
@@ -270,7 +283,7 @@ public class ZkJobCoordinator implements JobCoordinator {
     LOG.info("pid=" + processorId + "Generated new JobModel with version: " + nextJMVersion + " and processors: " + currentProcessorIds);
 
     // Publish the new job model
-    zkUtils.publishJobModel(nextJMVersion, jobModel);
+    publishJobModelToMetadataStore(jobModel, nextJMVersion);
 
     // Start the barrier for the job model update
     barrier.create(nextJMVersion, currentProcessorIds);
@@ -283,53 +296,151 @@ public class ZkJobCoordinator implements JobCoordinator {
     debounceTimer.scheduleAfterDebounceTime(ON_ZK_CLEANUP, 0, () -> zkUtils.cleanupZK(NUM_VERSIONS_TO_LEAVE));
   }
 
-  private String createProcessorId(Config config) {
-    // TODO: This check to be removed after 0.13+
-    ApplicationConfig appConfig = new ApplicationConfig(config);
-    if (appConfig.getProcessorId() != null) {
-      return appConfig.getProcessorId();
-    } else if (StringUtils.isNotBlank(appConfig.getAppProcessorIdGeneratorClass())) {
-      ProcessorIdGenerator idGenerator =
-          Util.getObj(appConfig.getAppProcessorIdGeneratorClass(), ProcessorIdGenerator.class);
-      return idGenerator.generateProcessorId(config);
-    } else {
-      throw new ConfigException(String
-          .format("Expected either %s or %s to be configured", ApplicationConfig.PROCESSOR_ID,
-              ApplicationConfig.APP_PROCESSOR_ID_GENERATOR_CLASS));
+  @VisibleForTesting
+  void publishJobModelToMetadataStore(JobModel jobModel, String nextJMVersion) {
+    JobModelUtil.writeJobModel(jobModel, nextJMVersion, jobModelMetadataStore);
+  }
+
+  @VisibleForTesting
+  JobModel readJobModelFromMetadataStore(String zkJobModelVersion) {
+    return JobModelUtil.readJobModel(zkJobModelVersion, jobModelMetadataStore);
+  }
+
+  /**
+   * Stores the configuration of the job in the coordinator stream.
+   */
+  @VisibleForTesting
+  void loadMetadataResources(JobModel jobModel) {
+    try {
+      MetadataResourceUtil metadataResourceUtil = createMetadataResourceUtil(jobModel, config);
+      metadataResourceUtil.createResources();
+
+      if (coordinatorStreamStore != null) {
+        // TODO: SAMZA-2273 - publish configs async
+        CoordinatorStreamValueSerde jsonSerde = new CoordinatorStreamValueSerde(SetConfig.TYPE);
+        NamespaceAwareCoordinatorStreamStore configStore =
+            new NamespaceAwareCoordinatorStreamStore(coordinatorStreamStore, SetConfig.TYPE);
+        for (Map.Entry<String, String> entry : config.entrySet()) {
+          byte[] serializedValue = jsonSerde.toBytes(entry.getValue());
+          configStore.put(entry.getKey(), serializedValue);
+        }
+        configStore.flush();
+
+        if (new JobConfig(config).getStartpointEnabled()) {
+          // fan out the startpoints
+          StartpointManager startpointManager = createStartpointManager();
+          startpointManager.start();
+          try {
+            startpointManager.fanOut(JobModelUtil.getTaskToSystemStreamPartitions(jobModel));
+          } finally {
+            startpointManager.stop();
+          }
+        }
+      } else {
+        LOG.warn("No metadata store registered to this job coordinator. Config not written to the metadata store and no Startpoints fan out.");
+      }
+    } catch (IOException ex) {
+      throw new SamzaException(String.format("IO exception while loading metadata resources."), ex);
     }
   }
 
+  @VisibleForTesting
+  MetadataResourceUtil createMetadataResourceUtil(JobModel jobModel, Config config) {
+    return new MetadataResourceUtil(jobModel, metrics.getMetricsRegistry(), config);
+  }
+
   /**
    * Generate new JobModel when becoming a leader or the list of processor changed.
    */
-  private JobModel generateNewJobModel(List<String> processors) {
+  @VisibleForTesting
+  JobModel generateNewJobModel(List<ProcessorNode> processorNodes) {
     String zkJobModelVersion = zkUtils.getJobModelVersion();
     // If JobModel exists in zookeeper && cached JobModel version is unequal to JobModel version stored in zookeeper.
     if (zkJobModelVersion != null && !Objects.equals(cachedJobModelVersion, zkJobModelVersion)) {
-      JobModel jobModel = zkUtils.getJobModel(zkJobModelVersion);
+      JobModel jobModel = readJobModelFromMetadataStore(zkJobModelVersion);
       for (ContainerModel containerModel : jobModel.getContainers().values()) {
         containerModel.getTasks().forEach((taskName, taskModel) -> changeLogPartitionMap.put(taskName, taskModel.getChangelogPartition().getPartitionId()));
       }
       cachedJobModelVersion = zkJobModelVersion;
     }
-    /**
-     * Host affinity is not supported in standalone. Hence, LocalityManager(which is responsible for container
-     * to host mapping) is passed in as null when building the jobModel.
-     */
-    JobModel model = JobModelManager.readJobModel(this.config, changeLogPartitionMap, null, streamMetadataCache, processors);
+
+    GrouperMetadata grouperMetadata = getGrouperMetadata(zkJobModelVersion, processorNodes);
+    JobModel model = JobModelManager.readJobModel(config, changeLogPartitionMap, streamMetadataCache, grouperMetadata);
     return new JobModel(new MapConfig(), model.getContainers());
   }
 
+  @VisibleForTesting
+  StreamPartitionCountMonitor getPartitionCountMonitor() {
+    StreamMetadataCache streamMetadata = new StreamMetadataCache(systemAdmins, 0, SystemClock.instance());
+    Set<SystemStream> inputStreamsToMonitor = new TaskConfig(config).getAllInputStreams();
+
+    return new StreamPartitionCountMonitor(
+        inputStreamsToMonitor,
+        streamMetadata,
+        metrics.getMetricsRegistry(),
+        new JobConfig(config).getMonitorPartitionChangeFrequency(),
+      streamsChanged -> {
+        if (leaderElector.amILeader()) {
+          debounceTimer.scheduleAfterDebounceTime(ON_PROCESSOR_CHANGE, 0, this::doOnProcessorChange);
+        }
+      });
+  }
+
+  @VisibleForTesting
+  StartpointManager createStartpointManager() {
+    // This method is for easy mocking.
+    return new StartpointManager(coordinatorStreamStore);
+  }
+
+  /**
+   * Builds the {@link GrouperMetadataImpl} based upon provided {@param jobModelVersion}
+   * and {@param processorNodes}.
+   * @param jobModelVersion the most recent jobModelVersion available in the zookeeper.
+   * @param processorNodes the list of live processors in the zookeeper.
+   * @return the built grouper metadata.
+   */
+  private GrouperMetadataImpl getGrouperMetadata(String jobModelVersion, List<ProcessorNode> processorNodes) {
+    Map<TaskName, String> taskToProcessorId = new HashMap<>();
+    Map<TaskName, List<SystemStreamPartition>> taskToSSPs = new HashMap<>();
+    if (jobModelVersion != null) {
+      JobModel jobModel = readJobModelFromMetadataStore(jobModelVersion);
+      for (ContainerModel containerModel : jobModel.getContainers().values()) {
+        for (TaskModel taskModel : containerModel.getTasks().values()) {
+          taskToProcessorId.put(taskModel.getTaskName(), containerModel.getId());
+          for (SystemStreamPartition partition : taskModel.getSystemStreamPartitions()) {
+            taskToSSPs.computeIfAbsent(taskModel.getTaskName(), k -> new ArrayList<>());
+            taskToSSPs.get(taskModel.getTaskName()).add(partition);
+          }
+        }
+      }
+    }
+
+    Map<String, LocationId> processorLocality = new HashMap<>();
+    for (ProcessorNode processorNode : processorNodes) {
+      ProcessorData processorData = processorNode.getProcessorData();
+      processorLocality.put(processorData.getProcessorId(), processorData.getLocationId());
+    }
+
+    Map<TaskName, LocationId> taskLocality = zkUtils.readTaskLocality();
+    return new GrouperMetadataImpl(processorLocality, taskLocality, taskToSSPs, taskToProcessorId);
+  }
+
   class LeaderElectorListenerImpl implements LeaderElectorListener {
     @Override
     public void onBecomingLeader() {
       LOG.info("ZkJobCoordinator::onBecomeLeader - I became the leader");
-      metrics.isLeader.set(true);
+      metrics.isLeader.set(1);
       zkUtils.subscribeToProcessorChange(new ProcessorChangeHandler(zkUtils));
-      debounceTimer.scheduleAfterDebounceTime(ON_PROCESSOR_CHANGE, debounceTimeMs, () -> {
-          // actual actions to do are the same as onProcessorChange
-          doOnProcessorChange(new ArrayList<>());
-        });
+      if (!new StorageConfig(config).hasDurableStores()) {
+        // 1. Stop if there's a existing StreamPartitionCountMonitor running.
+        if (streamPartitionCountMonitor != null) {
+          streamPartitionCountMonitor.stop();
+        }
+        // 2. Start a new instance of StreamPartitionCountMonitor.
+        streamPartitionCountMonitor = getPartitionCountMonitor();
+        streamPartitionCountMonitor.start();
+      }
+      debounceTimer.scheduleAfterDebounceTime(ON_PROCESSOR_CHANGE, debounceTimeMs, ZkJobCoordinator.this::doOnProcessorChange);
     }
   }
 
@@ -355,15 +466,20 @@ public class ZkJobCoordinator implements JobCoordinator {
       metrics.singleBarrierRebalancingTime.update(System.nanoTime() - startTime);
       if (ZkBarrierForVersionUpgrade.State.DONE.equals(state)) {
         debounceTimer.scheduleAfterDebounceTime(barrierAction, 0, () -> {
-            LOG.info("pid=" + processorId + "new version " + version + " of the job model got confirmed");
+          LOG.info("pid=" + processorId + "new version " + version + " of the job model got confirmed");
 
-            // read the new Model
-            JobModel jobModel = getJobModel();
-            // start the container with the new model
-            if (coordinatorListener != null) {
-              coordinatorListener.onNewJobModel(processorId, jobModel);
+          // read the new Model
+          JobModel jobModel = getJobModel();
+          // start the container with the new model
+          if (coordinatorListener != null) {
+            for (ContainerModel containerModel : jobModel.getContainers().values()) {
+              for (TaskName taskName : containerModel.getTasks().keySet()) {
+                zkUtils.writeTaskLocality(taskName, locationId);
+              }
             }
-          });
+            coordinatorListener.onNewJobModel(processorId, jobModel);
+          }
+        });
       } else {
         if (ZkBarrierForVersionUpgrade.State.TIMED_OUT.equals(state)) {
           // no-op for non-leaders
@@ -371,10 +487,8 @@ public class ZkJobCoordinator implements JobCoordinator {
           LOG.warn("Barrier for version " + version + " timed out.");
           if (leaderElector.amILeader()) {
             LOG.info("Leader will schedule a new job model generation");
-            debounceTimer.scheduleAfterDebounceTime(ON_PROCESSOR_CHANGE, debounceTimeMs, () -> {
-                // actual actions to do are the same as onProcessorChange
-                doOnProcessorChange(new ArrayList<>());
-              });
+            // actual actions to do are the same as onProcessorChange
+            debounceTimer.scheduleAfterDebounceTime(ON_PROCESSOR_CHANGE, debounceTimeMs, ZkJobCoordinator.this::doOnProcessorChange);
           }
         }
       }
@@ -425,26 +539,26 @@ public class ZkJobCoordinator implements JobCoordinator {
     @Override
     public void doHandleDataChange(String dataPath, Object data) {
       debounceTimer.scheduleAfterDebounceTime(JOB_MODEL_VERSION_CHANGE, 0, () -> {
-          String jobModelVersion = (String) data;
+        String jobModelVersion = (String) data;
 
-          LOG.info("Got a notification for new JobModel version. Path = {} Version = {}", dataPath, data);
+        LOG.info("Got a notification for new JobModel version. Path = {} Version = {}", dataPath, data);
 
-          newJobModel = zkUtils.getJobModel(jobModelVersion);
-          LOG.info("pid=" + processorId + ": new JobModel is available. Version =" + jobModelVersion + "; JobModel = " + newJobModel);
+        newJobModel = readJobModelFromMetadataStore(jobModelVersion);
+        LOG.info("pid=" + processorId + ": new JobModel is available. Version =" + jobModelVersion + "; JobModel = " + newJobModel);
 
-          if (!newJobModel.getContainers().containsKey(processorId)) {
-            LOG.info("New JobModel does not contain pid={}. Stopping this processor. New JobModel: {}",
-                processorId, newJobModel);
-            stop();
-          } else {
-            // stop current work
-            if (coordinatorListener != null) {
-              coordinatorListener.onJobModelExpired();
-            }
-            // update ZK and wait for all the processors to get this new version
-            barrier.join(jobModelVersion, processorId);
+        if (!newJobModel.getContainers().containsKey(processorId)) {
+          LOG.info("New JobModel does not contain pid={}. Stopping this processor. New JobModel: {}",
+              processorId, newJobModel);
+          stop();
+        } else {
+          // stop current work
+          if (coordinatorListener != null) {
+            coordinatorListener.onJobModelExpired();
           }
-        });
+          // update ZK and wait for all the processors to get this new version
+          barrier.join(jobModelVersion, processorId);
+        }
+      });
     }
 
     @Override
@@ -467,6 +581,7 @@ public class ZkJobCoordinator implements JobCoordinator {
       switch (state) {
         case Expired:
           // if the session has expired it means that all the registration's ephemeral nodes are gone.
+          zkSessionMetrics.zkSessionExpirations.inc();
           LOG.warn("Got " + state.toString() + " event for processor=" + processorId + ". Stopping the container and unregister the processor node.");
 
           // increase generation of the ZK session. All the callbacks from the previous generation will be ignored.
@@ -477,6 +592,11 @@ public class ZkJobCoordinator implements JobCoordinator {
           if (leaderElector.amILeader()) {
             leaderElector.resignLeadership();
           }
+
+          if (streamPartitionCountMonitor != null) {
+            streamPartitionCountMonitor.stop();
+          }
+
           /**
            * After this event, one amongst the following two things could potentially happen:
            * A. On successful reconnect to another zookeeper server in ensemble, this processor is going to
@@ -487,14 +607,15 @@ public class ZkJobCoordinator implements JobCoordinator {
           LOG.info("Cancelling all scheduled actions in session expiration for processorId: {}.", processorId);
           debounceTimer.cancelAllScheduledActions();
           debounceTimer.scheduleAfterDebounceTime(ZK_SESSION_EXPIRED, 0, () -> {
-              if (coordinatorListener != null) {
-                coordinatorListener.onJobModelExpired();
-              }
-            });
+            if (coordinatorListener != null) {
+              coordinatorListener.onJobModelExpired();
+            }
+          });
 
           return;
         case Disconnected:
           // if the session has expired it means that all the registration's ephemeral nodes are gone.
+          zkSessionMetrics.zkSessionDisconnects.inc();
           LOG.warn("Got " + state.toString() + " event for processor=" + processorId + ". Scheduling a coordinator stop.");
 
           // If the connection is not restored after debounceTimeMs, the process is considered dead.
@@ -503,22 +624,24 @@ public class ZkJobCoordinator implements JobCoordinator {
         case AuthFailed:
         case NoSyncConnected:
         case Unknown:
+          zkSessionMetrics.zkSessionErrors.inc();
           LOG.warn("Got unexpected failure event " + state.toString() + " for processor=" + processorId + ". Stopping the job coordinator.");
           debounceTimer.scheduleAfterDebounceTime(ZK_SESSION_ERROR, 0, () -> stop());
           return;
         case SyncConnected:
+          zkSessionMetrics.zkSyncConnected.inc();
           LOG.info("Got syncconnected event for processor=" + processorId + ".");
           debounceTimer.cancelAction(ZK_SESSION_ERROR);
           return;
         default:
           // received SyncConnected, ConnectedReadOnly, and SaslAuthenticated. NoOp
           LOG.info("Got ZK event " + state.toString() + " for processor=" + processorId + ". Continue");
-          return;
       }
     }
 
     @Override
     public void handleNewSession() {
+      zkSessionMetrics.zkNewSessions.inc();
       LOG.info("Got new session created event for processor=" + processorId);
       debounceTimer.cancelAllScheduledActions();
       LOG.info("register zk controller for the new session");
@@ -529,6 +652,7 @@ public class ZkJobCoordinator implements JobCoordinator {
     @Override
     public void handleSessionEstablishmentError(Throwable error) {
       // this means we cannot connect to zookeeper to establish a session
+      zkSessionMetrics.zkSessionErrors.inc();
       LOG.info("handleSessionEstablishmentError received for processor=" + processorId, error);
       debounceTimer.scheduleAfterDebounceTime(ZK_SESSION_ERROR, 0, () -> stop());
     }
