@@ -19,20 +19,17 @@
 
 package org.apache.samza.checkpoint
 
-
-
 import java.util.HashMap
 import java.util.concurrent.ConcurrentHashMap
 
-import org.apache.samza.system.IncomingMessageEnvelope
-import org.apache.samza.system.SystemAdmins
+import org.apache.commons.lang3.StringUtils
 import org.apache.samza.SamzaException
-import org.apache.samza.config.Config
-import org.apache.samza.config.StreamConfig.Config2Stream
-import org.apache.samza.config.SystemConfig.Config2System
+import org.apache.samza.annotation.InterfaceStability
+import org.apache.samza.config.{Config, StreamConfig, SystemConfig}
 import org.apache.samza.container.TaskName
+import org.apache.samza.startpoint.{Startpoint, StartpointManager}
 import org.apache.samza.system.SystemStreamMetadata.OffsetType
-import org.apache.samza.system.{SystemAdmin, SystemStream, SystemStreamMetadata, SystemStreamPartition}
+import org.apache.samza.system._
 import org.apache.samza.util.Logging
 
 import scala.collection.JavaConverters._
@@ -76,21 +73,24 @@ object OffsetManager extends Logging {
     systemStreamMetadata: Map[SystemStream, SystemStreamMetadata],
     config: Config,
     checkpointManager: CheckpointManager = null,
+    startpointManager: StartpointManager = null,
     systemAdmins: SystemAdmins = SystemAdmins.empty(),
     checkpointListeners: Map[String, CheckpointListener] = Map(),
     offsetManagerMetrics: OffsetManagerMetrics = new OffsetManagerMetrics) = {
     debug("Building offset manager for %s." format systemStreamMetadata)
 
+    val streamConfig = new StreamConfig(config)
+
     val offsetSettings = systemStreamMetadata
       .map {
         case (systemStream, systemStreamMetadata) =>
           // Get default offset.
-          val streamDefaultOffset = config.getDefaultStreamOffset(systemStream)
-          val systemDefaultOffset = config.getDefaultSystemOffset(systemStream.getSystem)
-          val defaultOffsetType = if (streamDefaultOffset.isDefined) {
+          val streamDefaultOffset = streamConfig.getDefaultStreamOffset(systemStream)
+          val systemDefaultOffset = new SystemConfig(config).getSystemOffsetDefault(systemStream.getSystem)
+          val defaultOffsetType = if (streamDefaultOffset.isPresent) {
             OffsetType.valueOf(streamDefaultOffset.get.toUpperCase)
-          } else if (systemDefaultOffset.isDefined) {
-            OffsetType.valueOf(systemDefaultOffset.get.toUpperCase)
+          } else if (systemDefaultOffset != null) {
+            OffsetType.valueOf(systemDefaultOffset.toUpperCase)
           } else {
             info("No default offset for %s defined. Using upcoming." format systemStream)
             OffsetType.UPCOMING
@@ -98,13 +98,13 @@ object OffsetManager extends Logging {
           debug("Using default offset %s for %s." format (defaultOffsetType, systemStream))
 
           // Get reset offset.
-          val resetOffset = config.getResetOffset(systemStream)
+          val resetOffset = streamConfig.getResetOffset(systemStream)
           debug("Using reset offset %s for %s." format (resetOffset, systemStream))
 
           // Build OffsetSetting so we can create a map for OffsetManager.
           (systemStream, OffsetSetting(systemStreamMetadata, defaultOffsetType, resetOffset))
       }.toMap
-    new OffsetManager(offsetSettings, checkpointManager, systemAdmins, checkpointListeners, offsetManagerMetrics)
+    new OffsetManager(offsetSettings, checkpointManager, startpointManager, systemAdmins, checkpointListeners, offsetManagerMetrics)
   }
 }
 
@@ -112,14 +112,15 @@ object OffsetManager extends Logging {
  * OffsetManager does several things:
  *
  * <ul>
- * <li>Loads last checkpointed offset for all input SystemStreamPartitions in a
- * SamzaContainer.</li>
- * <li>Uses last checkpointed offset to figure out the next offset to start
- * reading from for each input SystemStreamPartition in a SamzaContainer</li>
+ * <li>Loads last checkpointed offset and startpoints for all input SystemStreamPartitions in a
+ * SamzaContainer. See SEP-18 for details.</li>
+ * <li>Uses last checkpointed offset or startpoint to figure out the next offset to start
+ * reading from for each input SystemStreamPartition in a SamzaContainer. Startpoints have a higher precedence than
+ * checkpoints.</li>
  * <li>Keep track of the last processed offset for each SystemStreamPartitions
  * in a SamzaContainer.</li>
  * <li>Checkpoints the last processed offset for each SystemStreamPartitions
- * in a SamzaContainer periodically to the CheckpointManager.</li>
+ * in a SamzaContainer periodically to the CheckpointManager and deletes any associated startpoints.</li>
  * </ul>
  *
  * All partitions must be registered before start is called, and start must be
@@ -138,6 +139,11 @@ class OffsetManager(
    */
   val checkpointManager: CheckpointManager = null,
 
+  /**
+    * Optional startpoint manager for overridden offsets.
+    */
+  val startpointManager: StartpointManager = null,
+
   /**
    * SystemAdmins that are used to get next offsets from last checkpointed
    * offsets. Map is from system name to SystemAdmin class for the system.
@@ -166,6 +172,11 @@ class OffsetManager(
    */
   var startingOffsets = Map[TaskName, Map[SystemStreamPartition, String]]()
 
+  /**
+    * Startpoints loaded for each SystemStreamPartition.
+    */
+  var startpoints = Map[TaskName, Map[SystemStreamPartition, Startpoint]]()
+
   /**
    * The set of system stream partitions that have been registered with the
    * OffsetManager, grouped by the taskName they belong to. These are the SSPs
@@ -184,6 +195,7 @@ class OffsetManager(
     loadOffsetsFromCheckpointManager
     stripResetStreams
     loadStartingOffsets
+    loadStartpoints
     loadDefaults
 
     info("Successfully loaded last processed offsets: %s" format lastProcessedOffsets)
@@ -222,6 +234,34 @@ class OffsetManager(
     startingOffsets += taskName -> (startingOffsets(taskName) + (ssp -> offset))
   }
 
+  /**
+    * Gets Startpoints that are loaded into the OffsetManager after OffsetManager#start is called.
+    */
+  @InterfaceStability.Unstable
+  def getStartpoint(taskName: TaskName, systemStreamPartition: SystemStreamPartition): Option[Startpoint] = {
+    // Startpoints already loaded when this method is available for use. Similar to getStartingOffset above.
+    startpoints.get(taskName) match {
+      case Some(sspToStartpoint) => sspToStartpoint.get(systemStreamPartition)
+      case None => None
+    }
+  }
+
+  /**
+    * Sets the Startpoint into the OffsetManager. Does not write directly to the StartpointManager. This is to be
+    * used by the TaskContext only for setting Startpoints during processor initialization, similar to the
+    * OffsetManager#setStartingOffset method. To write Startpoints to the metadata store, use the StartpointManager.
+    */
+  @InterfaceStability.Unstable
+  def setStartpoint(taskName: TaskName, ssp: SystemStreamPartition, startpoint: Startpoint) = {
+    // Startpoints already loaded when this method is available for use. Similar to setStartingOffset above.
+    startpoints += {
+      startpoints.get(taskName) match {
+        case Some(sspToStartpont) => taskName -> (sspToStartpont + (ssp -> startpoint))
+        case None => taskName -> new ConcurrentHashMap[SystemStreamPartition, Startpoint]() { put(ssp, startpoint) }.asScala
+      }
+    }
+  }
+
   /**
     * Gets a snapshot of all the current offsets for the specified task. This is useful to
     * ensure there are no concurrent updates to the offsets between when this method is
@@ -231,18 +271,19 @@ class OffsetManager(
     if (checkpointManager != null || checkpointListeners.nonEmpty) {
       debug("Getting checkpoint offsets for taskName %s." format taskName)
 
-      val sspsForTaskName = systemStreamPartitions.getOrElse(taskName, throw new SamzaException("No SSPs registered for task: " + taskName)).toSet
-      val sspToOffsets = lastProcessedOffsets.get(taskName)
+      val taskStartingOffsets = startingOffsets.getOrElse(taskName,
+        throw new SamzaException("Couldn't find starting offsets for task: " + taskName))
 
-      val partitionOffsets = if (sspToOffsets != null) {
-        // Filter the offsets in case the task model changed since the last checkpoint was written.
-        sspToOffsets.asScala.filterKeys(sspsForTaskName.contains)
-      } else {
-        warn(taskName + " is not found... ")
-        Map[SystemStreamPartition, String]()
-      }
+      val taskSSPs = systemStreamPartitions.getOrElse(taskName,
+        throw new SamzaException("No SSPs registered for task: " + taskName))
 
-      new Checkpoint(new HashMap(partitionOffsets.asJava)) // Copy into new Map to prevent mutation
+      // filter the offsets in case the task model changed since the last checkpoint was written.
+      val taskLastProcessedOffsets =
+        lastProcessedOffsets.getOrDefault(taskName, new ConcurrentHashMap()).asScala
+          .filterKeys(taskSSPs.contains)
+
+      val modifiedTaskOffsets = getModifiedOffsets(taskStartingOffsets, taskLastProcessedOffsets)
+      new Checkpoint(new HashMap(modifiedTaskOffsets)) // Copy into new Map to prevent mutation
     } else {
       debug("Returning null checkpoint for taskName %s because no checkpoint manager/callback is defined." format taskName)
       null
@@ -250,7 +291,48 @@ class OffsetManager(
   }
 
   /**
-    * Write the specified checkpoint for the given task.
+    * Call the beforeCheckpoint() method on [[CheckpointListener]] SystemConsumers to give them a chance to inspect and
+    * potentially modify the offsets about to be checkpointed.
+    */
+  def getModifiedOffsets(taskStartingOffsets: Map[SystemStreamPartition, String],
+      taskLastProcessedOffsets: Map[SystemStreamPartition, String]): HashMap[SystemStreamPartition, String] = {
+    val modifiedOffsets = new HashMap[SystemStreamPartition, String](taskLastProcessedOffsets.asJava)
+
+    taskLastProcessedOffsets
+      .groupBy { case (ssp, offset) => ssp.getSystem }  // Map(systemName -> Map(ssp -> offset))
+      .foreach { case (systemName, systemSSPLastProcessedOffsets) =>
+      if (checkpointListeners.contains(systemName)) {
+        val checkpointListener = checkpointListeners(systemName)
+
+        // if we haven't processed any messages from the system since we started, we don't need to call
+        // the checkpoint listener. also, in case of Kafka, getting the safe offset is only possible after
+        // first poll() is successful. check if we need to adjust the offset for any of the ssp in the system.
+        val needModifiedOffsets = systemSSPLastProcessedOffsets.exists { case (ssp, sspLastProcessedOffset) =>
+          // starting offsets are checkpointed offsets + 1 (see 'loadStartingOffsets'),
+          // while on container start last processed offsets are equal to checkpointed offsets.
+          // compare last processed offsets with starting offsets to see if we've processed anything.
+          // last processed offsets < starting offsets after container start but before we've processed anything.
+          taskStartingOffsets.get(ssp) match {
+            case Some(startingOffset) if startingOffset != null =>
+              val systemAdmin = systemAdmins.getSystemAdmin(systemName)
+              val comparisionResult = systemAdmin.offsetComparator(sspLastProcessedOffset, startingOffset)
+              if (comparisionResult != null) comparisionResult < 0 else false
+            case _ => false
+          }
+        }
+
+        if (needModifiedOffsets) {
+          val systemModifiedOffsets = checkpointListener.beforeCheckpoint(systemSSPLastProcessedOffsets.asJava)
+          modifiedOffsets.putAll(systemModifiedOffsets)
+        }
+      }
+    }
+
+    modifiedOffsets
+  }
+
+  /**
+    * Write the specified checkpoint for the given task and delete the corresponding startpoint, if available.
     */
   def writeCheckpoint(taskName: TaskName, checkpoint: Checkpoint) {
     if (checkpoint != null && (checkpointManager != null || checkpointListeners.nonEmpty)) {
@@ -261,17 +343,39 @@ class OffsetManager(
         val sspToOffsets = checkpoint.getOffsets
         if(sspToOffsets != null) {
           sspToOffsets.asScala.foreach {
-            case (ssp, cp) => offsetManagerMetrics.checkpointedOffsets.get(ssp).set(cp)
+            case (ssp, cp) => {
+              val metric = offsetManagerMetrics.checkpointedOffsets.get(ssp)
+              // metric will be null for changelog SSPs since they're not registered with / tracked by the
+              // OffsetManager. Ignore such SSPs.
+              if (metric != null) metric.set(cp)
+            }
           }
         }
       }
 
-      // invoke checkpoint listeners
-      checkpoint.getOffsets.asScala.groupBy { case (ssp, _) => ssp.getSystem }.foreach {
-        case (systemName:String, offsets: Map[SystemStreamPartition, String]) => {
-          // Option is empty if there is no checkpointListener for this systemName
-          checkpointListeners.get(systemName).foreach(_.onCheckpoint(offsets.asJava))
+      // Invoke checkpoint listeners only for SSPs that are registered with the OffsetManager. For example,
+      // changelog SSPs are not registered but may be present in the Checkpoint if transactional state checkpointing
+      // is enabled.
+      val registeredSSPs = systemStreamPartitions.getOrElse(taskName, Set[SystemStreamPartition]())
+      checkpoint.getOffsets.asScala
+        .filterKeys(registeredSSPs.contains)
+        .groupBy { case (ssp, _) => ssp.getSystem }.foreach {
+          case (systemName:String, offsets: Map[SystemStreamPartition, String]) => {
+            // Option is empty if there is no checkpointListener for this systemName
+            checkpointListeners.get(systemName).foreach(_.onCheckpoint(offsets.asJava))
+          }
         }
+    }
+
+    // delete corresponding startpoints after checkpoint is supposed to be committed
+    if (startpointManager != null && startpoints.contains(taskName)) {
+      info("%d startpoint(s) for taskName: %s have been committed to the checkpoint." format (startpoints.get(taskName).size, taskName.getTaskName))
+      startpointManager.removeFanOutForTask(taskName)
+      startpoints -= taskName
+
+      if (startpoints.isEmpty) {
+        info("All outstanding startpoints have been committed to the checkpoint.")
+        startpointManager.stop
       }
     }
   }
@@ -284,6 +388,14 @@ class OffsetManager(
     } else {
       debug("Skipping checkpoint manager shutdown because no checkpoint manager is defined.")
     }
+
+    if (startpointManager != null) {
+      debug("Shutting down startpoint manager.")
+
+      startpointManager.stop
+    } else {
+      debug("Skipping startpoint manager shutdown because no startpoint manager is defined.")
+    }
   }
 
   /**
@@ -404,6 +516,79 @@ class OffsetManager(
     }
   }
 
+  /**
+    * Load Startpoints for each SystemStreamPartition+TaskName
+    */
+  private def loadStartpoints: Unit = {
+    if (startpointManager != null) {
+      info("Starting startpoint manager.")
+      startpointManager.start
+
+      systemStreamPartitions.foreach {
+        case (taskName, systemStreamPartitionSet) => {
+          Option(startpointManager.getFanOutForTask(taskName)) match {
+            case Some(fanOut) => {
+              val filteredFanOut = fanOut.asScala
+                .filter(f => systemStreamPartitionSet.contains(f._1))
+                .toMap
+              if (!filteredFanOut.isEmpty) {
+                startpoints += taskName -> filteredFanOut
+                info("Startpoint fan out for task: %s - %s" format (taskName, filteredFanOut))
+              }
+            }
+            case None => debug("No startpoints fanned out on taskName: %s" format taskName.getTaskName)
+          }
+        }
+      }
+
+      if (startpoints.isEmpty) {
+        info("No startpoints to consume.")
+        startpointManager.stop
+      } else {
+        startpoints
+          .foreach(taskMap => taskMap._2
+            .foreach(sspMap => info("Loaded startpoint: %s for SSP: %s and task: %s" format (sspMap._2, sspMap._1, taskMap._1))))
+        resolveStartpointsToStartingOffsets
+      }
+    }
+  }
+
+  /**
+    * Overwrite starting offsets with resolved offsets from startpoints
+    */
+  private def resolveStartpointsToStartingOffsets: Unit = {
+    startpoints.foreach {
+      case (taskName, sspToStartpoint) => {
+        var resolvedOffsets: Map[SystemStreamPartition, String] = Map()
+        sspToStartpoint.foreach {
+          case (ssp, startpoint) => {
+            try {
+              val systemAdmin: SystemAdmin = systemAdmins.getSystemAdmin(ssp.getSystem)
+              val resolvedOffset: String = systemAdmin.resolveStartpointToOffset(ssp, startpoint)
+              if (StringUtils.isNotBlank(resolvedOffset)) {
+                resolvedOffsets += ssp -> resolvedOffset
+                info("Resolved the startpoint: %s of system stream partition: %s to offset: %s." format(startpoint, ssp, resolvedOffset))
+              }
+            } catch {
+              case e: Exception => {
+                error("Exception occurred when resolving startpoint: %s of system stream partition: %s to offset." format(startpoint, ssp), e)
+              }
+            }
+          }
+        }
+
+        // copy starting offsets and overwrite with resolved offsets
+        var mergedOffsets: Map[SystemStreamPartition, String] = startingOffsets.getOrElse(taskName, Map())
+        resolvedOffsets.foreach {
+          case (ssp, resolvedOffset) => {
+            mergedOffsets += ssp -> resolvedOffset
+          }
+        }
+        startingOffsets += taskName -> mergedOffsets
+      }
+    }
+  }
+
   /**
    * Use defaultOffsets to get a next offset for every SystemStreamPartition
    * that was registered, but has no offset.
