@@ -21,7 +21,6 @@ package org.apache.samza.metrics
 
 import org.apache.samza.metrics.MetricGroup.ValueFunction
 
-
 /**
  * MetricsHelper is a little helper class to make it easy to register and
  * manage counters, gauges and timers.
@@ -32,6 +31,7 @@ import org.apache.samza.metrics.MetricGroup.ValueFunction
 trait MetricsHelper {
   val group = this.getClass.getName
   val registry: MetricsRegistry
+
   val metricGroup = new MetricGroup(group, getPrefix, registry)
 
   def newCounter(name: String) = metricGroup.newCounter(name)
@@ -40,8 +40,6 @@ trait MetricsHelper {
 
   def newGauge[T](name: String, value: T) = metricGroup.newGauge[T](name,value)
 
-  def newListGauge[T](name: String) = metricGroup.newListGauge[T](name)
-
   /**
    * Specify a dynamic gauge that always returns the latest value when polled.
    * The value closure must be thread safe, since metrics reporters may access
@@ -53,6 +51,8 @@ trait MetricsHelper {
     })
   }
 
+  def newHistogram(name: String, percentiles: java.util.List[java.lang.Double]) = metricGroup.newHistogram(name,percentiles)
+
   /**
    * Returns a prefix for metric names.
    */
