@@ -50,8 +50,10 @@ class MetricsRegistryMap(val name: String) extends ReadableMetricsRegistry with
   }
 
   def newGauge[T](group: String, gauge: Gauge[T]) = {
-    debug("Adding new gauge %s %s %s." format (group, gauge.getName, gauge))
-    putAndGetGroup(group).putIfAbsent(gauge.getName, gauge)
+    if (putAndGetGroup(group).containsKey(gauge.getName)) {
+      debug("Updating existing gauge %s %s %s" format (group, gauge.getName, gauge))
+    }
+    putAndGetGroup(group).put(gauge.getName, gauge)
     val realGauge = metrics.get(group).get(gauge.getName).asInstanceOf[Gauge[T]]
     listeners.foreach(_.onGauge(group, realGauge))
     realGauge
@@ -75,21 +77,6 @@ class MetricsRegistryMap(val name: String) extends ReadableMetricsRegistry with
     newTimer(group, new Timer(name))
   }
 
-  /**
-    * Register a {@link org.apache.samza.metrics.ListGauge}
-    *
-    * @param group     Group for this ListGauge
-    * @param listGauge the ListGauge to register
-    * @tparam T the type of the list gauge
-    */
-  def newListGauge[T](group: String, listGauge: ListGauge[T]) = {
-    debug("Adding new listgauge %s %s %s." format(group, listGauge.getName, listGauge))
-    putAndGetGroup(group).putIfAbsent(listGauge.getName, listGauge)
-    val realListGauge = metrics.get(group).get(listGauge.getName).asInstanceOf[ListGauge[T]]
-    listeners.foreach(_.onListGauge(group, realListGauge))
-    realListGauge
-  }
-
   private def putAndGetGroup(group: String) = {
     metrics.putIfAbsent(group, new ConcurrentHashMap[String, Metric])
     metrics.get(group)
