@@ -21,13 +21,11 @@
 
 package org.apache.samza.util
 
-import java.io.{File, FileInputStream, FileOutputStream, ObjectInputStream, ObjectOutputStream}
+import java.io._
 import java.nio.file._
 import java.util.zip.CRC32
 
-import org.apache.samza.util.Util.info
-
-object FileUtil {
+class FileUtil extends Logging {
   /**
     * Writes checksum & data to a file
     * Checksum is pre-fixed to the data and is a 32-bit long type data.
@@ -51,11 +49,52 @@ object FileUtil {
     }
 
     //atomic swap of tmp and real offset file
+    move(tmpFile, file)
+  }
+
+  /**
+    * Writes the data to a text file
+    * @param file The file handle to write to
+    * @param data The data to be written to the file
+    * @param append true for appending data to file, false otherwise
+    * */
+  def writeToTextFile(file: File, data: String, append: Boolean): Unit = {
+
+    val tmpFilePath = file.getAbsolutePath + ".tmp"
+    var fileWriter: FileWriter = null
+    val tmpFile = new File(tmpFilePath)
+
+    //atomic swap of tmp and real file if we need to append
+    if (append) {
+      move(file, tmpFile)
+    }
+
     try {
-      Files.move(tmpFile.toPath, file.toPath, StandardCopyOption.ATOMIC_MOVE, StandardCopyOption.REPLACE_EXISTING)
+      fileWriter = new FileWriter(tmpFile, append)
+      fileWriter.write(data)
+    } catch {
+      case e: Exception =>
+        error("Error in writing to file %s isAppend %s" format (file, append))
+        System.out.println(e)
+    } finally {
+      fileWriter.close()
+    }
+
+    //atomic swap of tmp and real file
+    move(tmpFile, file)
+  }
+
+  /**
+   * Moves source file to destination file, replacing destination file if it already exists.
+   */
+  def move(source: File, destination: File) : Unit = {
+    try {
+      if (source.exists()) {
+        Files.move(source.toPath, destination.toPath, StandardCopyOption.ATOMIC_MOVE, StandardCopyOption.REPLACE_EXISTING)
+      }
     } catch {
       case e: AtomicMoveNotSupportedException =>
-        Files.move(tmpFile.toPath, file.toPath, StandardCopyOption.REPLACE_EXISTING)
+        Files.move(source.toPath, destination.toPath, StandardCopyOption.REPLACE_EXISTING)
     }
   }
 
@@ -102,6 +141,14 @@ object FileUtil {
     }
   }
 
+  def exists(path: Path): Boolean = {
+    Files.exists(path)
+  }
+
+  def createDirectories(path: Path): Path = {
+    Files.createDirectories(path)
+  }
+
   /**
     * Generates the CRC32 checksum code for any given data
     * @param data The string for which checksum has to be generated
