@@ -21,8 +21,10 @@
 
 package org.apache.samza.util
 
-import java.util.function
+import java.util
+import java.util.{Optional, function}
 
+import scala.collection.JavaConverters
 import scala.collection.immutable.Map
 import scala.collection.JavaConverters._
 import scala.runtime.AbstractFunction0
@@ -36,6 +38,13 @@ object ScalaJavaUtil {
     javaMap.asScala.toMap
   }
 
+  /**
+    * Convert a scala iterable to a Java collection.
+    */
+  def toJavaCollection[T](iterable : Iterable[T]): util.Collection[T] = {
+    JavaConverters.asJavaCollectionConverter(iterable).asJavaCollection
+  }
+
   /**
     * Wraps the provided value in an Scala Function, e.g. for use in [[Option#getOrDefault]]
     *
@@ -71,4 +80,30 @@ object ScalaJavaUtil {
   def toScalaFunction[T, R](javaFunction: java.util.function.Function[T, R]): Function1[T, R] = {
     t => javaFunction.apply(t)
   }
+
+
+  /**
+    * Conversions between Scala Option and Java 8 Optional.
+    */
+  object JavaOptionals {
+    implicit def toRichOption[T](opt: Option[T]): RichOption[T] = new RichOption[T](opt)
+    implicit def toRichOptional[T](optional: Optional[T]): RichOptional[T] = new RichOptional[T](optional)
+  }
+
+  class RichOption[T] (opt: Option[T]) {
+
+    /**
+      * Transform this Option to an equivalent Java Optional
+      */
+    def toOptional: Optional[T] = Optional.ofNullable(opt.getOrElse(null).asInstanceOf[T])
+  }
+
+  class RichOptional[T] (opt: Optional[T]) {
+
+    /**
+      * Transform this Optional to an equivalent Scala Option
+      */
+    def toOption: Option[T] = if (opt.isPresent) Some(opt.get()) else None
+  }
+
 }
