@@ -19,6 +19,9 @@
 package org.apache.samza.execution;
 
 import com.google.common.base.Joiner;
+import com.google.common.collect.ImmutableMap;
+import java.util.Arrays;
+import org.apache.samza.SamzaException;
 import org.apache.samza.application.descriptors.StreamApplicationDescriptorImpl;
 import org.apache.samza.application.descriptors.TaskApplicationDescriptorImpl;
 import org.apache.samza.config.Config;
@@ -27,39 +30,31 @@ import org.apache.samza.config.JobConfig;
 import org.apache.samza.config.MapConfig;
 import org.apache.samza.config.SerializerConfig;
 import org.apache.samza.config.TaskConfig;
-import org.apache.samza.config.TaskConfigJava;
-import org.apache.samza.context.Context;
+import org.apache.samza.storage.SideInputsProcessor;
 import org.apache.samza.system.descriptors.GenericInputDescriptor;
 import org.apache.samza.table.descriptors.BaseTableDescriptor;
 import org.apache.samza.operators.KV;
+import org.apache.samza.table.descriptors.TestLocalTableDescriptor.MockLocalTableDescriptor;
+import org.apache.samza.table.descriptors.TestLocalTableDescriptor.MockTableProviderFactory;
 import org.apache.samza.table.descriptors.TableDescriptor;
 import org.apache.samza.operators.impl.store.TimestampedValueSerde;
 import org.apache.samza.serializers.JsonSerdeV2;
-import org.apache.samza.serializers.KVSerde;
 import org.apache.samza.serializers.Serde;
 import org.apache.samza.serializers.SerializableSerde;
 import org.apache.samza.serializers.StringSerde;
 import org.apache.samza.system.StreamSpec;
-import org.apache.samza.table.Table;
-import org.apache.samza.table.descriptors.TableProvider;
-import org.apache.samza.table.descriptors.TableProviderFactory;
-import org.apache.samza.table.TableSpec;
 import org.junit.Test;
 
-import java.util.ArrayList;
 import java.util.Base64;
 import java.util.HashMap;
 import java.util.HashSet;
-import java.util.List;
 import java.util.Map;
 import java.util.Set;
 import java.util.stream.Collectors;
 
 import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertTrue;
-import static org.mockito.Mockito.mock;
-import static org.mockito.Mockito.spy;
-import static org.mockito.Mockito.when;
+import static org.mockito.Mockito.*;
 
 
 /**
@@ -79,7 +74,7 @@ public class TestJobNodeConfigurationGenerator extends ExecutionPlannerTestBase
     Config expectedJobConfig = getExpectedJobConfig(mockConfig, mockJobNode.getInEdges());
     validateJobConfig(expectedJobConfig, jobConfig);
     // additional, check the computed window.ms for join
-    assertEquals("3600000", jobConfig.get(TaskConfig.WINDOW_MS()));
+    assertEquals("3600000", jobConfig.get(TaskConfig.WINDOW_MS));
     Map<String, Serde> deserializedSerdes = validateAndGetDeserializedSerdes(jobConfig, 5);
     validateStreamConfigures(jobConfig, deserializedSerdes);
     validateJoinStoreConfigures(jobConfig, deserializedSerdes);
@@ -161,17 +156,10 @@ public class TestJobNodeConfigurationGenerator extends ExecutionPlannerTestBase
     mockStreamAppDesc = new StreamApplicationDescriptorImpl(getRepartitionJoinStreamApplication(), mockConfig);
     // add table to the RepartitionJoinStreamApplication
     GenericInputDescriptor<KV<String, Object>> sideInput1 = inputSystemDescriptor.getInputDescriptor("sideInput1", defaultSerde);
-    BaseTableDescriptor mockTableDescriptor = mock(BaseTableDescriptor.class);
-    TableSpec mockTableSpec = mock(TableSpec.class);
-    when(mockTableSpec.getId()).thenReturn("testTable");
-    when(mockTableSpec.getSerde()).thenReturn((KVSerde) defaultSerde);
-    when(mockTableSpec.getTableProviderFactoryClassName()).thenReturn(MockTableProviderFactory.class.getName());
-    List<String> sideInputs = new ArrayList<>();
-    sideInputs.add(sideInput1.getStreamId());
-    when(mockTableSpec.getSideInputs()).thenReturn(sideInputs);
-    when(mockTableDescriptor.getTableId()).thenReturn("testTable");
-    when(mockTableDescriptor.getTableSpec()).thenReturn(mockTableSpec);
-    when(mockTableDescriptor.getSerde()).thenReturn(defaultSerde);
+    BaseTableDescriptor mockTableDescriptor = new MockLocalTableDescriptor("testTable", defaultSerde)
+        .withSideInputs(Arrays.asList(sideInput1.getStreamId()))
+        .withSideInputsProcessor(mock(SideInputsProcessor.class, withSettings().serializable()))
+        .withConfig("mock.table.provider.config", "mock.config.value");
     // add side input and terminate at table in the appplication
     mockStreamAppDesc.getInputStream(sideInput1).sendTo(mockStreamAppDesc.getTable(mockTableDescriptor));
     StreamEdge sideInputEdge = new StreamEdge(new StreamSpec(sideInput1.getStreamId(), "sideInput1",
@@ -198,17 +186,10 @@ public class TestJobNodeConfigurationGenerator extends ExecutionPlannerTestBase
   public void testTaskApplicationWithTableAndSideInput() {
     // add table to the RepartitionJoinStreamApplication
     GenericInputDescriptor<KV<String, Object>> sideInput1 = inputSystemDescriptor.getInputDescriptor("sideInput1", defaultSerde);
-    BaseTableDescriptor mockTableDescriptor = mock(BaseTableDescriptor.class);
-    TableSpec mockTableSpec = mock(TableSpec.class);
-    when(mockTableSpec.getId()).thenReturn("testTable");
-    when(mockTableSpec.getSerde()).thenReturn((KVSerde) defaultSerde);
-    when(mockTableSpec.getTableProviderFactoryClassName()).thenReturn(MockTableProviderFactory.class.getName());
-    List<String> sideInputs = new ArrayList<>();
-    sideInputs.add(sideInput1.getStreamId());
-    when(mockTableSpec.getSideInputs()).thenReturn(sideInputs);
-    when(mockTableDescriptor.getTableId()).thenReturn("testTable");
-    when(mockTableDescriptor.getTableSpec()).thenReturn(mockTableSpec);
-    when(mockTableDescriptor.getSerde()).thenReturn(defaultSerde);
+    BaseTableDescriptor mockTableDescriptor = new MockLocalTableDescriptor("testTable", defaultSerde)
+        .withSideInputs(Arrays.asList(sideInput1.getStreamId()))
+        .withSideInputsProcessor(mock(SideInputsProcessor.class, withSettings().serializable()))
+        .withConfig("mock.table.provider.config", "mock.config.value");
     StreamEdge sideInputEdge = new StreamEdge(new StreamSpec(sideInput1.getStreamId(), "sideInput1",
         inputSystemDescriptor.getSystemName()), false, false, mockConfig);
     // need to put the sideInput related stream configuration to the original config
@@ -242,8 +223,8 @@ public class TestJobNodeConfigurationGenerator extends ExecutionPlannerTestBase
   public void testConfigRewriter() {
     Map<String, String> configs = new HashMap<>(mockConfig);
     String streamCfgToOverride = String.format("streams.%s.samza.system", intermediateInputDescriptor.getStreamId());
-    configs.put(String.format(JobConfig.CONFIG_REWRITER_CLASS(), "mock"), MockConfigRewriter.class.getName());
-    configs.put(JobConfig.CONFIG_REWRITERS(), "mock");
+    configs.put(String.format(JobConfig.CONFIG_REWRITER_CLASS, "mock"), MockConfigRewriter.class.getName());
+    configs.put(JobConfig.CONFIG_REWRITERS, "mock");
     configs.put(String.format("job.config.rewriter.mock.%s", streamCfgToOverride), "rewritten-system");
     mockConfig = spy(new MapConfig(configs));
     mockStreamAppDesc = new StreamApplicationDescriptorImpl(getRepartitionJoinStreamApplication(), mockConfig);
@@ -256,13 +237,29 @@ public class TestJobNodeConfigurationGenerator extends ExecutionPlannerTestBase
     assertEquals("rewritten-system", jobConfig.get(streamCfgToOverride));
   }
 
+  @Test(expected = SamzaException.class)
+  public void testJobNameConfigValidation() {
+    ImmutableMap<String, String> userConfigs =
+        ImmutableMap.of("job.name", "samza-job", "job.id", "1", "app.name", "samza-app");
+    ImmutableMap<String, String> generatedConfigs =
+        ImmutableMap.of("job.name", "samza-app", "job.id", "1", "app.name", "samza-app");
+    JobNodeConfigurationGenerator.validateJobConfigs(userConfigs, generatedConfigs);
+  }
+
+  @Test(expected = SamzaException.class)
+  public void testJobIdConfigValidation() {
+    ImmutableMap<String, String> userConfigs =
+        ImmutableMap.of("job.id", "1", "app.id", "this-should-take-precedence", "app.name", "samza-app");
+    ImmutableMap<String, String> generatedConfigs =
+        ImmutableMap.of("job.name", "samza-app", "job.id", "this-should-take-precedence", "app.name", "samza-app");
+    JobNodeConfigurationGenerator.validateJobConfigs(userConfigs, generatedConfigs);
+  }
+
   private void validateTableConfigure(JobConfig jobConfig, Map<String, Serde> deserializedSerdes,
       TableDescriptor tableDescriptor) {
     Config tableConfig = jobConfig.subset(String.format("tables.%s.", tableDescriptor.getTableId()));
     assertEquals(MockTableProviderFactory.class.getName(), tableConfig.get("provider.factory"));
-    MockTableProvider mockTableProvider =
-        (MockTableProvider) new MockTableProviderFactory().getTableProvider(((BaseTableDescriptor) tableDescriptor).getTableSpec());
-    assertEquals(mockTableProvider.configMap.get("mock.table.provider.config"), jobConfig.get("mock.table.provider.config"));
+    assertEquals("mock.config.value", jobConfig.get("mock.table.provider.config"));
     validateTableSerdeConfigure(tableDescriptor.getTableId(), jobConfig, deserializedSerdes);
   }
 
@@ -278,10 +275,10 @@ public class TestJobNodeConfigurationGenerator extends ExecutionPlannerTestBase
       }
     }
     if (!inputs.isEmpty()) {
-      configMap.put(TaskConfig.INPUT_STREAMS(), Joiner.on(',').join(inputs));
+      configMap.put(TaskConfig.INPUT_STREAMS, Joiner.on(',').join(inputs));
     }
     if (!broadcasts.isEmpty()) {
-      configMap.put(TaskConfigJava.BROADCAST_INPUT_STREAMS, Joiner.on(',').join(broadcasts));
+      configMap.put(TaskConfig.BROADCAST_INPUT_STREAMS, Joiner.on(',').join(broadcasts));
     }
     return new MapConfig(configMap);
   }
@@ -292,17 +289,17 @@ public class TestJobNodeConfigurationGenerator extends ExecutionPlannerTestBase
     SerializableSerde<Serde> serializableSerde = new SerializableSerde<>();
     assertEquals(numSerdes, serializers.size());
     return serializers.entrySet().stream().collect(Collectors.toMap(
-        e -> e.getKey().replace(SerializerConfig.SERIALIZED_INSTANCE_SUFFIX(), ""),
-        e -> serializableSerde.fromBytes(Base64.getDecoder().decode(e.getValue().getBytes()))
+      e -> e.getKey().replace(SerializerConfig.SERIALIZED_INSTANCE_SUFFIX, ""),
+      e -> serializableSerde.fromBytes(Base64.getDecoder().decode(e.getValue().getBytes()))
     ));
   }
 
   private void validateJobConfig(Config expectedConfig, JobConfig jobConfig) {
-    assertEquals(expectedConfig.get(JobConfig.JOB_NAME()), jobConfig.getName().get());
-    assertEquals(expectedConfig.get(JobConfig.JOB_ID()), jobConfig.getJobId());
+    assertEquals(expectedConfig.get(JobConfig.JOB_NAME), jobConfig.getName().get());
+    assertEquals(expectedConfig.get(JobConfig.JOB_ID), jobConfig.getJobId());
     assertEquals("testJobGraphJson", jobConfig.get(JobNodeConfigurationGenerator.CONFIG_INTERNAL_EXECUTION_PLAN));
-    assertEquals(expectedConfig.get(TaskConfig.INPUT_STREAMS()), jobConfig.get(TaskConfig.INPUT_STREAMS()));
-    assertEquals(expectedConfig.get(TaskConfigJava.BROADCAST_INPUT_STREAMS), jobConfig.get(TaskConfigJava.BROADCAST_INPUT_STREAMS));
+    assertEquals(expectedConfig.get(TaskConfig.INPUT_STREAMS), jobConfig.get(TaskConfig.INPUT_STREAMS));
+    assertEquals(expectedConfig.get(TaskConfig.BROADCAST_INPUT_STREAMS), jobConfig.get(TaskConfig.BROADCAST_INPUT_STREAMS));
   }
 
   private void validateStreamSerdeConfigure(String streamId, Config config, Map<String, Serde> deserializedSerdes) {
@@ -316,9 +313,9 @@ public class TestJobNodeConfigurationGenerator extends ExecutionPlannerTestBase
   }
 
   private void validateTableSerdeConfigure(String tableId, Config config, Map<String, Serde> deserializedSerdes) {
-    Config streamConfig = config.subset(String.format("tables.%s.", tableId));
+    Config streamConfig = config.subset(String.format("stores.%s.", tableId));
     String keySerdeName = streamConfig.get("key.serde");
-    String valueSerdeName = streamConfig.get("value.serde");
+    String valueSerdeName = streamConfig.get("msg.serde");
     assertTrue(String.format("Serialized serdes should contain %s key serde", tableId), deserializedSerdes.containsKey(keySerdeName));
     assertTrue(String.format("Serialized %s key serde should be a StringSerde", tableId), keySerdeName.startsWith(StringSerde.class.getSimpleName()));
     assertTrue(String.format("Serialized serdes should contain %s value serde", tableId), deserializedSerdes.containsKey(valueSerdeName));
@@ -332,7 +329,6 @@ public class TestJobNodeConfigurationGenerator extends ExecutionPlannerTestBase
     assertEquals("true", intStreamConfig.get("samza.delete.committed.messages"));
     assertEquals(physicalName, intStreamConfig.get("samza.physical.name"));
     assertEquals("true", intStreamConfig.get("samza.intermediate"));
-    assertEquals("oldest", intStreamConfig.get("samza.offset.default"));
   }
 
   private void validateStreamConfigures(Config config, Map<String, Serde> deserializedSerdes) {
@@ -387,46 +383,7 @@ public class TestJobNodeConfigurationGenerator extends ExecutionPlannerTestBase
     assertEquals("3600000", joinStoreConfig.get("rocksdb.ttl.ms"));
   }
 
-  private static class MockTableProvider implements TableProvider {
-    private final Map<String, String> configMap;
-
-    MockTableProvider(Map<String, String> configMap) {
-      this.configMap = configMap;
-    }
-
-    @Override
-    public void init(Context context) {
-
-    }
-
-    @Override
-    public Table getTable() {
-      return null;
-    }
-
-    @Override
-    public Map<String, String> generateConfig(Config jobConfig, Map<String, String> generatedConfig) {
-      return configMap;
-    }
-
-    @Override
-    public void close() {
-
-    }
-  }
-
-  public static class MockTableProviderFactory implements TableProviderFactory {
-
-    @Override
-    public TableProvider getTableProvider(TableSpec tableSpec) {
-      Map<String, String> configMap = new HashMap<>();
-      configMap.put("mock.table.provider.config", "mock.config.value");
-      return new MockTableProvider(configMap);
-    }
-  }
-
   public static class MockConfigRewriter implements ConfigRewriter {
-
     @Override
     public Config rewrite(String name, Config config) {
       Map<String, String> configMap = new HashMap<>(config);
@@ -434,4 +391,5 @@ public class TestJobNodeConfigurationGenerator extends ExecutionPlannerTestBase
       return new MapConfig(configMap);
     }
   }
+
 }
