@@ -22,21 +22,24 @@ package org.apache.samza.container
 import java.util
 import java.util.concurrent.atomic.AtomicReference
 
-import org.apache.samza.config.{Config, MapConfig}
+import javax.servlet.http.{HttpServlet, HttpServletRequest, HttpServletResponse}
+import org.apache.samza.Partition
+import org.apache.samza.config.{ClusterManagerConfig, Config, MapConfig}
 import org.apache.samza.context.{ApplicationContainerContext, ContainerContext}
 import org.apache.samza.coordinator.JobModelManager
 import org.apache.samza.coordinator.server.{HttpServer, JobServlet}
 import org.apache.samza.job.model.{ContainerModel, JobModel, TaskModel}
-import org.apache.samza.metrics.{Gauge, Timer}
+import org.apache.samza.metrics.Gauge
+import org.apache.samza.serializers.model.SamzaObjectMapper
+import org.apache.samza.storage.ContainerStorageManager
 import org.apache.samza.system._
-import org.apache.samza.{Partition, SamzaContainerStatus}
 import org.junit.Assert._
 import org.junit.{Before, Test}
 import org.mockito.Matchers.{any, notNull}
 import org.mockito.Mockito._
 import org.mockito.invocation.InvocationOnMock
 import org.mockito.stubbing.Answer
-import org.mockito.{ArgumentCaptor, Mock, MockitoAnnotations}
+import org.mockito.{Mock, Mockito, MockitoAnnotations}
 import org.scalatest.junit.AssertionsForJUnit
 import org.scalatest.mockito.MockitoSugar
 
@@ -61,11 +64,15 @@ class TestSamzaContainer extends AssertionsForJUnit with MockitoSugar {
   @Mock
   private var metrics: SamzaContainerMetrics = null
   @Mock
+  private var localityManager: LocalityManager = null
+  @Mock
   private var containerContext: ContainerContext = null
   @Mock
   private var applicationContainerContext: ApplicationContainerContext = null
   @Mock
   private var samzaContainerListener: SamzaContainerListener = null
+  @Mock
+  private var containerStorageManager: ContainerStorageManager = null
 
   private var samzaContainer: SamzaContainer = null
 
@@ -73,7 +80,7 @@ class TestSamzaContainer extends AssertionsForJUnit with MockitoSugar {
   def setup(): Unit = {
     MockitoAnnotations.initMocks(this)
     setupSamzaContainer(Some(this.applicationContainerContext))
-    when(this.metrics.containerStartupTime).thenReturn(mock[Timer])
+    when(this.metrics.containerStartupTime).thenReturn(mock[Gauge[Long]])
   }
 
   @Test
@@ -121,6 +128,45 @@ class TestSamzaContainer extends AssertionsForJUnit with MockitoSugar {
     verify(this.runLoop).run()
   }
 
+
+  @Test
+  def testShutDownSequenceForStandbyContainers() {
+    class ShutDownSignal(container: SamzaContainer) extends Runnable {
+      def run(): Unit = {
+        Thread.sleep(2000)
+        container.shutdown();
+      }
+    }
+
+    this.samzaContainer = new SamzaContainer(
+      this.config,
+      Map.empty[TaskName, TaskInstance],
+      Map.empty[TaskName, TaskInstanceMetrics],
+      this.runLoop,
+      this.systemAdmins,
+      this.consumerMultiplexer,
+      this.producerMultiplexer,
+      this.metrics,
+      localityManager = this.localityManager,
+      containerContext = this.containerContext,
+      applicationContainerContextOption = Some(this.applicationContainerContext),
+      externalContextOption = None,
+      containerStorageManager = containerStorageManager)
+    this.samzaContainer.setContainerListener(this.samzaContainerListener)
+
+    new ShutDownSignal(samzaContainer).run();
+    this.samzaContainer.run
+
+    verify(this.samzaContainerListener).beforeStart()
+    verify(this.samzaContainerListener).afterStart()
+    verify(this.samzaContainerListener).afterStop()
+    verify(this.runLoop, never()).run()
+    verify(this.systemAdmins).stop()
+    verify(this.containerStorageManager).shutdown()
+  }
+
+
+
   @Test
   def testCleanRun(): Unit = {
     doNothing().when(this.runLoop).run() // run loop completes successfully
@@ -136,6 +182,46 @@ class TestSamzaContainer extends AssertionsForJUnit with MockitoSugar {
     verify(this.runLoop).run()
   }
 
+  @Test
+  def testInterruptDuringStoreRestorationShutdownContainer(): Unit = {
+    when(this.containerStorageManager.start())
+      .thenAnswer(new Answer[Void] {
+        override def answer(mock: InvocationOnMock): Void = {
+        Thread.sleep(1000)
+        throw new InterruptedException("Injecting interrupt into container storage manager")
+      }
+      })
+
+    this.samzaContainer.run
+
+    assertEquals(SamzaContainerStatus.STOPPED, this.samzaContainer.getStatus())
+    verify(this.samzaContainerListener).beforeStart()
+    verify(this.samzaContainerListener).afterStop()
+    verify(this.samzaContainerListener, never()).afterFailure(any())
+    verify(this.runLoop, times(0)).run()
+  }
+
+  @Test
+  def testInterruptDuringStoreRestorationWithErrorsDuringContainerShutdown(): Unit = {
+    when(this.containerStorageManager.start())
+      .thenAnswer(new Answer[Void] {
+        override def answer(mock: InvocationOnMock): Void = {
+          Thread.sleep(1000)
+          throw new InterruptedException("Injecting interrupt into container storage manager")
+        }
+      })
+
+    when(this.taskInstance.shutdownTask).thenThrow(new RuntimeException("Trigger a shutdown, please."))
+
+    this.samzaContainer.run
+
+    assertEquals(SamzaContainerStatus.FAILED, this.samzaContainer.getStatus())
+    verify(this.samzaContainerListener).beforeStart()
+    verify(this.samzaContainerListener).afterFailure(any())
+    verify(this.samzaContainerListener, never()).afterStop()
+    verify(this.runLoop, times(0)).run()
+  }
+
   @Test
   def testFailureDuringShutdown(): Unit = {
     doNothing().when(this.runLoop).run() // run loop completes successfully
@@ -152,23 +238,6 @@ class TestSamzaContainer extends AssertionsForJUnit with MockitoSugar {
     verify(this.runLoop).run()
   }
 
-  @Test
-  def testStartStoresIncrementsCounter() {
-    when(this.taskInstance.taskName).thenReturn(TASK_NAME)
-    val restoreGauge = mock[Gauge[Long]]
-    when(this.metrics.taskStoreRestorationMetrics).thenReturn(Map(TASK_NAME -> restoreGauge))
-    when(this.taskInstance.startStores).thenAnswer(new Answer[Void] {
-      override def answer(invocation: InvocationOnMock): Void = {
-        Thread.sleep(1)
-        null
-      }
-    })
-    this.samzaContainer.startStores
-    val restoreGaugeValueCaptor = ArgumentCaptor.forClass(classOf[Long])
-    verify(restoreGauge).set(restoreGaugeValueCaptor.capture())
-    assertTrue(restoreGaugeValueCaptor.getValue >= 1)
-  }
-
   @Test
   def testApplicationContainerContext() {
     val orderVerifier = inOrder(this.applicationContainerContext, this.runLoop)
@@ -199,11 +268,11 @@ class TestSamzaContainer extends AssertionsForJUnit with MockitoSugar {
       "0" -> new ContainerModel("0", tasks),
       "1" -> new ContainerModel("1", tasks))
     val jobModel = new JobModel(config, containers)
-    def jobModelGenerator(): JobModel = jobModel
+    def jobModelGenerator(): Array[Byte] = SamzaObjectMapper.getObjectMapper.writeValueAsBytes(jobModel)
     val server = new HttpServer
     val coordinator = new JobModelManager(jobModel, server)
-    JobModelManager.jobModelRef.set(jobModelGenerator())
-    coordinator.server.addServlet("/*", new JobServlet(JobModelManager.jobModelRef))
+    JobModelManager.serializedJobModelRef.set(jobModelGenerator())
+    coordinator.server.addServlet("/*", new JobServlet(JobModelManager.serializedJobModelRef))
     try {
       coordinator.start
       assertEquals(jobModel, SamzaContainer.readJobModel(server.getUrl.toString))
@@ -224,11 +293,11 @@ class TestSamzaContainer extends AssertionsForJUnit with MockitoSugar {
       "0" -> new ContainerModel("0", tasks),
       "1" -> new ContainerModel("1", tasks))
     val jobModel = new JobModel(config, containers)
-    def jobModelGenerator(): JobModel = jobModel
+    def jobModelGenerator(): Array[Byte] = SamzaObjectMapper.getObjectMapper.writeValueAsBytes(jobModel)
     val server = new HttpServer
     val coordinator = new JobModelManager(jobModel, server)
-    JobModelManager.jobModelRef.set(jobModelGenerator())
-    val mockJobServlet = new MockJobServlet(2, JobModelManager.jobModelRef)
+    JobModelManager.serializedJobModelRef.set(jobModelGenerator())
+    val mockJobServlet = new MockJobServlet(2, JobModelManager.serializedJobModelRef)
     coordinator.server.addServlet("/*", mockJobServlet)
     try {
       coordinator.start
@@ -273,30 +342,55 @@ class TestSamzaContainer extends AssertionsForJUnit with MockitoSugar {
     assertEquals(Set(), SamzaContainer.getChangelogSSPsForContainer(containerModel, Map()))
   }
 
+  @Test
+  def testStoreContainerLocality():Unit = {
+    this.config = new MapConfig(Map(ClusterManagerConfig.JOB_HOST_AFFINITY_ENABLED -> "true"))
+    setupSamzaContainer(None) // re-init with an actual config
+    val containerModel: ContainerModel = Mockito.mock[ContainerModel](classOf[ContainerModel])
+    val testContainerId = "1"
+    Mockito.when(containerModel.getId).thenReturn(testContainerId)
+    Mockito.when(this.containerContext.getContainerModel).thenReturn(containerModel)
+
+    this.samzaContainer.storeContainerLocality
+    Mockito.verify(this.localityManager).writeContainerToHostMapping(any(), any())
+  }
+
   private def setupSamzaContainer(applicationContainerContext: Option[ApplicationContainerContext]) {
     this.samzaContainer = new SamzaContainer(
       this.config,
       Map(TASK_NAME -> this.taskInstance),
+      Map(TASK_NAME -> new TaskInstanceMetrics),
       this.runLoop,
       this.systemAdmins,
       this.consumerMultiplexer,
       this.producerMultiplexer,
-      metrics,
+      this.metrics,
+      localityManager = this.localityManager,
       containerContext = this.containerContext,
-      applicationContainerContextOption = applicationContainerContext)
+      applicationContainerContextOption = applicationContainerContext,
+      externalContextOption = None,
+      containerStorageManager = containerStorageManager)
     this.samzaContainer.setContainerListener(this.samzaContainerListener)
   }
 
-  class MockJobServlet(exceptionLimit: Int, jobModelRef: AtomicReference[JobModel]) extends JobServlet(jobModelRef) {
+  class MockJobServlet(exceptionLimit: Int, jobModelRef: AtomicReference[Array[Byte]]) extends HttpServlet {
     var exceptionCount = 0
 
-    override protected def getObjectToWrite(): JobModel = {
+    override protected def doGet(request: HttpServletRequest, response: HttpServletResponse) {
       if (exceptionCount < exceptionLimit) {
         exceptionCount += 1
         throw new java.io.IOException("Throwing exception")
       } else {
         val jobModel = jobModelRef.get()
-        jobModel
+
+        // This should never happen because JobServlet is instantiated only after a jobModel is generated and its reference is updated
+        if (jobModel == null) {
+          throw new IllegalStateException("No JobModel to serve in the JobCoordinator.")
+        }
+
+        response.setContentType("application/json")
+        response.setStatus(HttpServletResponse.SC_OK)
+        response.getOutputStream.write(jobModelRef.get())
       }
     }
   }
