@@ -23,9 +23,11 @@ import java.util
 
 import org.apache.samza.Partition
 import org.apache.samza.checkpoint.TestCheckpointTool.MockCheckpointManagerFactory
-import org.apache.samza.config.{JobConfig, MapConfig, SystemConfig, TaskConfig}
+import org.apache.samza.config._
 import org.apache.samza.container.{SamzaContainer, TaskName}
-import org.apache.samza.coordinator.stream.{CoordinatorStreamManager, MockCoordinatorStreamSystemFactory, MockCoordinatorStreamWrappedConsumer}
+import org.apache.samza.coordinator.metadatastore.{CoordinatorStreamStore, CoordinatorStreamStoreTestUtil, NamespaceAwareCoordinatorStreamStore}
+import org.apache.samza.coordinator.stream.messages.SetChangelogMapping
+import org.apache.samza.coordinator.stream.{MockCoordinatorStreamSystemFactory, MockCoordinatorStreamWrappedConsumer}
 import org.apache.samza.job.MockJobFactory
 import org.apache.samza.job.local.{ProcessJobFactory, ThreadJobFactory}
 import org.apache.samza.job.model.{ContainerModel, JobModel, TaskModel}
@@ -91,8 +93,8 @@ class TestJobCoordinator extends FlatSpec with PrivateMethodTester {
       JobConfig.JOB_COORDINATOR_SYSTEM -> "coordinator",
       JobConfig.JOB_CONTAINER_COUNT -> "2",
       TaskConfig.INPUT_STREAMS -> "test.stream1",
-      SystemConfig.SYSTEM_FACTORY.format("test") -> classOf[MockSystemFactory].getCanonicalName,
-      SystemConfig.SYSTEM_FACTORY.format("coordinator") -> classOf[MockCoordinatorStreamSystemFactory].getName,
+      SystemConfig.SYSTEM_FACTORY_FORMAT.format("test") -> classOf[MockSystemFactory].getCanonicalName,
+      SystemConfig.SYSTEM_FACTORY_FORMAT.format("coordinator") -> classOf[MockCoordinatorStreamSystemFactory].getName,
       TaskConfig.GROUPER_FACTORY -> "org.apache.samza.container.grouper.task.GroupByContainerCountFactory",
       JobConfig.MONITOR_PARTITION_CHANGE -> "true",
       JobConfig.MONITOR_PARTITION_CHANGE_FREQUENCY_MS -> "100"
@@ -105,17 +107,22 @@ class TestJobCoordinator extends FlatSpec with PrivateMethodTester {
     val expectedJobModel = new JobModel(new MapConfig(config.asJava), containers.asJava)
 
     // Verify that the atomicReference is initialized
-    assertNotNull(JobModelManager.jobModelRef.get())
-    assertEquals(expectedJobModel, JobModelManager.jobModelRef.get())
+    assertNotNull(JobModelManager.serializedJobModelRef.get())
+    val expectedContainerModels = new util.TreeMap[String, ContainerModel](expectedJobModel.getContainers)
+    val jobModel = SamzaObjectMapper.getObjectMapper.readValue(JobModelManager.serializedJobModelRef.get(), classOf[JobModel])
+    val actualContainerModels = new util.TreeMap[String, ContainerModel](jobModel.getContainers)
+    assertEquals(expectedContainerModels, actualContainerModels)
 
     coordinator.start
-    assertEquals(new MapConfig(config.asJava), coordinator.jobModel.getConfig)
-    assertEquals(expectedJobModel, coordinator.jobModel)
+    val expectedConfig: Config = coordinator.jobModel.getConfig
+    val actualConfig: Config = new MapConfig(config.asJava)
+    assertTrue(expectedConfig.entrySet().containsAll(actualConfig.entrySet()))
+    assertEquals(expectedJobModel.getContainers, coordinator.jobModel.getContainers)
 
     val response = HttpUtil.read(coordinator.server.getUrl)
     // Verify that the JobServlet is serving the correct jobModel
     val jobModelFromCoordinatorUrl = SamzaObjectMapper.getObjectMapper.readValue(response, classOf[JobModel])
-    assertEquals(expectedJobModel, jobModelFromCoordinatorUrl)
+    assertEquals(expectedJobModel.getContainers, jobModelFromCoordinatorUrl.getContainers)
 
     coordinator.stop
   }
@@ -153,8 +160,8 @@ class TestJobCoordinator extends FlatSpec with PrivateMethodTester {
       JobConfig.JOB_COORDINATOR_SYSTEM -> "coordinator",
       JobConfig.JOB_CONTAINER_COUNT -> "2",
       TaskConfig.INPUT_STREAMS -> "test.stream1",
-      SystemConfig.SYSTEM_FACTORY.format("test") -> classOf[MockSystemFactory].getCanonicalName,
-      SystemConfig.SYSTEM_FACTORY.format("coordinator") -> classOf[MockCoordinatorStreamSystemFactory].getName,
+      SystemConfig.SYSTEM_FACTORY_FORMAT.format("test") -> classOf[MockSystemFactory].getCanonicalName,
+      SystemConfig.SYSTEM_FACTORY_FORMAT.format("coordinator") -> classOf[MockCoordinatorStreamSystemFactory].getName,
       TaskConfig.GROUPER_FACTORY -> "org.apache.samza.container.grouper.task.GroupByContainerCountFactory"
       )
 
@@ -230,7 +237,7 @@ class TestJobCoordinator extends FlatSpec with PrivateMethodTester {
     */
   @Test
   def testWithPartitionAssignmentWithMockJobFactory {
-    val config = new SystemConfig(getTestConfig(classOf[MockJobFactory]))
+    val config = getTestConfig(classOf[MockJobFactory])
     val systemStream = new SystemStream("test", "stream1")
     val streamMetadataCache = mock(classOf[StreamMetadataCache])
     when(streamMetadataCache.getStreamMetadata(Set(systemStream), true)).thenReturn(
@@ -243,7 +250,7 @@ class TestJobCoordinator extends FlatSpec with PrivateMethodTester {
     val getMatchedInputStreamPartitions = PrivateMethod[immutable.Set[Any]]('getMatchedInputStreamPartitions)
 
     val allSSP = JobModelManager invokePrivate getInputStreamPartitions(config, streamMetadataCache)
-    val matchedSSP = JobModelManager invokePrivate  getMatchedInputStreamPartitions(config, streamMetadataCache)
+    val matchedSSP = JobModelManager invokePrivate getMatchedInputStreamPartitions(config, streamMetadataCache)
     assertEquals(matchedSSP, allSSP)
   }
 
@@ -256,8 +263,8 @@ class TestJobCoordinator extends FlatSpec with PrivateMethodTester {
       JobConfig.STREAM_JOB_FACTORY_CLASS -> clazz.getCanonicalName,
       JobConfig.SSP_MATCHER_CLASS -> JobConfig.SSP_MATCHER_CLASS_REGEX,
       JobConfig.SSP_MATCHER_CONFIG_REGEX -> "[1]",
-      SystemConfig.SYSTEM_FACTORY.format("test") -> classOf[MockSystemFactory].getCanonicalName,
-      SystemConfig.SYSTEM_FACTORY.format("coordinator") -> classOf[MockCoordinatorStreamSystemFactory].getName).asJava)
+      SystemConfig.SYSTEM_FACTORY_FORMAT.format("test") -> classOf[MockSystemFactory].getCanonicalName,
+      SystemConfig.SYSTEM_FACTORY_FORMAT.format("coordinator") -> classOf[MockCoordinatorStreamSystemFactory].getName).asJava)
     config
   }
 
@@ -270,22 +277,25 @@ class TestJobCoordinator extends FlatSpec with PrivateMethodTester {
   }
 
   def getTestJobModelManager(config: MapConfig) = {
-    val coordinatorStreamManager = new CoordinatorStreamManager(config, new MetricsRegistryMap)
-    coordinatorStreamManager.register("TestJobCoordinator")
-    coordinatorStreamManager.start
-    coordinatorStreamManager.bootstrap
-    val changelogPartitionManager = new ChangelogStreamManager(coordinatorStreamManager)
-    val jobModelManager = JobModelManager(coordinatorStreamManager.getConfig, changelogPartitionManager.readPartitionMapping())
-    coordinatorStreamManager.stop()
-    jobModelManager
+    val coordinatorStreamTestUtil: CoordinatorStreamStoreTestUtil = new CoordinatorStreamStoreTestUtil(config, "coordinator")
+    val coordinatorStreamStore: CoordinatorStreamStore = coordinatorStreamTestUtil.getCoordinatorStreamStore
+    val namespaceAwareCoordinatorStore: NamespaceAwareCoordinatorStreamStore = new NamespaceAwareCoordinatorStreamStore(coordinatorStreamStore, SetChangelogMapping.TYPE)
+    try {
+      val changelogPartitionManager = new ChangelogStreamManager(namespaceAwareCoordinatorStore)
+      val jobModelManager = JobModelManager(config, changelogPartitionManager.readPartitionMapping(),
+        coordinatorStreamStore, new MetricsRegistryMap())
+      jobModelManager
+    } finally {
+      coordinatorStreamStore.close()
+    }
   }
 
   @Before
   def setUp() {
     // setup the test stream metadata
-    MockSystemFactory.MSG_QUEUES.put(new SystemStreamPartition("test", "stream1", new Partition(0)), new util.ArrayList[IncomingMessageEnvelope]());
-    MockSystemFactory.MSG_QUEUES.put(new SystemStreamPartition("test", "stream1", new Partition(1)), new util.ArrayList[IncomingMessageEnvelope]());
-    MockSystemFactory.MSG_QUEUES.put(new SystemStreamPartition("test", "stream1", new Partition(2)), new util.ArrayList[IncomingMessageEnvelope]());
+    MockSystemFactory.MSG_QUEUES.put(new SystemStreamPartition("test", "stream1", new Partition(0)), new util.ArrayList[IncomingMessageEnvelope]())
+    MockSystemFactory.MSG_QUEUES.put(new SystemStreamPartition("test", "stream1", new Partition(1)), new util.ArrayList[IncomingMessageEnvelope]())
+    MockSystemFactory.MSG_QUEUES.put(new SystemStreamPartition("test", "stream1", new Partition(2)), new util.ArrayList[IncomingMessageEnvelope]())
   }
 
   @After
