@@ -19,6 +19,7 @@
 package org.apache.samza.processor
 
 import java.util
+import java.util.Collections
 
 import org.apache.samza.Partition
 import org.apache.samza.config.MapConfig
@@ -26,10 +27,12 @@ import org.apache.samza.container._
 import org.apache.samza.context.{ContainerContext, JobContext}
 import org.apache.samza.job.model.TaskModel
 import org.apache.samza.serializers.SerdeManager
+import org.apache.samza.storage.ContainerStorageManager
 import org.apache.samza.system._
 import org.apache.samza.system.chooser.RoundRobinChooser
 import org.apache.samza.task.{StreamTask, TaskInstanceCollector}
 import org.mockito.Mockito
+import org.mockito.Mockito.when
 
 
 object StreamProcessorTestUtils {
@@ -40,12 +43,15 @@ object StreamProcessorTestUtils {
     val adminMultiplexer = new SystemAdmins(config)
     val consumerMultiplexer = new SystemConsumers(
       new RoundRobinChooser,
-      Map[String, SystemConsumer]())
+      Map[String, SystemConsumer](), SystemAdmins.empty())
     val producerMultiplexer = new SystemProducers(
       Map[String, SystemProducer](),
       new SerdeManager)
     val collector = new TaskInstanceCollector(producerMultiplexer)
     val containerContext = Mockito.mock(classOf[ContainerContext])
+    val jobContext = Mockito.mock(classOf[JobContext])
+    when(jobContext.getConfig).thenReturn(
+      new MapConfig(Collections.singletonMap("task.commit.ms", "-1")))
     val taskInstance: TaskInstance = new TaskInstance(
       streamTask,
       taskModel,
@@ -53,21 +59,25 @@ object StreamProcessorTestUtils {
       adminMultiplexer,
       consumerMultiplexer,
       collector,
-      jobContext = Mockito.mock(classOf[JobContext]),
+      jobContext = jobContext,
       containerContext = containerContext,
       applicationContainerContextOption = None,
-      applicationTaskContextFactoryOption = None)
+      applicationTaskContextFactoryOption = None,
+      externalContextOption = None)
 
     val container = new SamzaContainer(
       config = config,
       taskInstances = Map(taskName -> taskInstance),
+      taskInstanceMetrics = Map(taskName -> new TaskInstanceMetrics),
       runLoop = mockRunloop,
       systemAdmins = adminMultiplexer,
       consumerMultiplexer = consumerMultiplexer,
       producerMultiplexer = producerMultiplexer,
       metrics = new SamzaContainerMetrics,
       containerContext = containerContext,
-      applicationContainerContextOption = None)
+      applicationContainerContextOption = None,
+      externalContextOption = None,
+      containerStorageManager = Mockito.mock(classOf[ContainerStorageManager]))
     container
   }
 }
\ No newline at end of file
