@@ -25,11 +25,13 @@ import java.util.Collections
 import org.junit.Assert._
 import org.junit.Test
 import org.apache.samza.Partition
+import org.apache.samza.config.TaskConfig
 import org.apache.samza.serializers._
 import org.apache.samza.system.chooser.MessageChooser
 import org.apache.samza.system.chooser.DefaultChooser
 import org.apache.samza.system.chooser.MockMessageChooser
 import org.apache.samza.util.BlockingEnvelopeMap
+import org.mockito.Mockito
 
 import scala.collection.JavaConverters._
 
@@ -42,11 +44,14 @@ class TestSystemConsumers {
     val envelope = new IncomingMessageEnvelope(systemStreamPartition0, "1", "k", "v")
     val consumer = new CustomPollResponseSystemConsumer(envelope)
     var now = 0L
-    val consumers = new SystemConsumers(new MockMessageChooser, Map(system -> consumer),
+    val systemAdmins = Mockito.mock(classOf[SystemAdmins])
+    Mockito.doReturn(Mockito.mock(classOf[SystemAdmin])).when(systemAdmins.getSystemAdmin(system))
+
+    val consumers = new SystemConsumers(new MockMessageChooser, Map(system -> consumer), systemAdmins,
                                         new SerdeManager, new SystemConsumersMetrics,
                                         SystemConsumers.DEFAULT_NO_NEW_MESSAGES_TIMEOUT,
                                         SystemConsumers.DEFAULT_DROP_SERIALIZATION_ERROR,
-                                        SystemConsumers.DEFAULT_POLL_INTERVAL_MS, clock = () => now)
+                                        TaskConfig.DEFAULT_POLL_INTERVAL_MS, clock = () => now)
 
     consumers.register(systemStreamPartition0, "0")
     consumers.register(systemStreamPartition1, "1234")
@@ -70,7 +75,7 @@ class TestSystemConsumers {
 
     // Advance the clock to trigger a new poll even though there are still
     // messages.
-    now = SystemConsumers.DEFAULT_POLL_INTERVAL_MS
+    now = TaskConfig.DEFAULT_POLL_INTERVAL_MS
 
     assertEquals(envelope, consumers.choose())
 
@@ -104,11 +109,14 @@ class TestSystemConsumers {
     val envelope = new IncomingMessageEnvelope(systemStreamPartition, "1", "k", "v")
     val consumer = new CustomPollResponseSystemConsumer(envelope)
     var now = 0
-    val consumers = new SystemConsumers(new MockMessageChooser, Map(system -> consumer),
+    val systemAdmins = Mockito.mock(classOf[SystemAdmins])
+    Mockito.doReturn(Mockito.mock(classOf[SystemAdmin])).when(systemAdmins.getSystemAdmin(system))
+
+    val consumers = new SystemConsumers(new MockMessageChooser, Map(system -> consumer), systemAdmins,
                                         new SerdeManager, new SystemConsumersMetrics,
                                         SystemConsumers.DEFAULT_NO_NEW_MESSAGES_TIMEOUT,
                                         SystemConsumers.DEFAULT_DROP_SERIALIZATION_ERROR,
-                                        SystemConsumers.DEFAULT_POLL_INTERVAL_MS, clock = () => now)
+                                        TaskConfig.DEFAULT_POLL_INTERVAL_MS, clock = () => now)
 
     consumers.register(systemStreamPartition, "0")
     consumers.start
@@ -137,7 +145,7 @@ class TestSystemConsumers {
     assertNull(consumers.choose())
 
     // Increase clock interval.
-    now = SystemConsumers.DEFAULT_POLL_INTERVAL_MS
+    now = TaskConfig.DEFAULT_POLL_INTERVAL_MS
 
     // We get two messages now.
     assertEquals(envelope, consumers.choose())
@@ -165,13 +173,16 @@ class TestSystemConsumers {
       def poll(systemStreamPartitions: java.util.Set[SystemStreamPartition], timeout: Long) = Map[SystemStreamPartition, java.util.List[IncomingMessageEnvelope]]().asJava
     })
 
+    val systemAdmins = Mockito.mock(classOf[SystemAdmins])
+    Mockito.when(systemAdmins.getSystemAdmin(system)).thenReturn(Mockito.mock(classOf[SystemAdmin]))
+
     val consumers = new SystemConsumers(new MessageChooser {
       def update(envelope: IncomingMessageEnvelope) = Unit
       def choose = null
       def start = chooserStarted += 1
       def stop = chooserStopped += 1
       def register(systemStreamPartition: SystemStreamPartition, offset: String) = chooserRegistered += systemStreamPartition -> offset
-    }, consumer, null)
+    }, consumer, systemAdmins)
 
     consumers.register(systemStreamPartition, "0")
     consumers.start
@@ -229,15 +240,17 @@ class TestSystemConsumers {
     val systemStreamPartition = new SystemStreamPartition(system, "some-stream", new Partition(1))
     val msgChooser = new DefaultChooser
     val consumer = Map(system -> new SerializingConsumer)
-    val systemMessageSerdes = Map(system -> (new StringSerde("UTF-8")).asInstanceOf[Serde[Object]]);
+    val systemMessageSerdes = Map(system -> (new StringSerde("UTF-8")).asInstanceOf[Serde[Object]])
     val serdeManager = new SerdeManager(systemMessageSerdes = systemMessageSerdes)
+    val systemAdmins = Mockito.mock(classOf[SystemAdmins])
+    Mockito.when(systemAdmins.getSystemAdmin(system)).thenReturn(Mockito.mock(classOf[SystemAdmin]))
 
     // throw exceptions when the deserialization has error
-    val consumers = new SystemConsumers(msgChooser, consumer, serdeManager, dropDeserializationError = false)
+    val consumers = new SystemConsumers(msgChooser, consumer, systemAdmins, serdeManager, dropDeserializationError = false)
     consumers.register(systemStreamPartition, "0")
-    consumer(system).putBytesMessage
-    consumer(system).putStringMessage
     consumers.start
+    consumer(system).putStringMessage
+    consumer(system).putBytesMessage
 
     var caughtRightException = false
     try {
@@ -246,16 +259,16 @@ class TestSystemConsumers {
       case e: SystemConsumersException => caughtRightException = true
       case _: Throwable => caughtRightException = false
     }
-    assertTrue("suppose to throw SystemConsumersException", caughtRightException);
+    assertTrue("suppose to throw SystemConsumersException", caughtRightException)
     consumers.stop
 
     // it should not throw exceptions when deserializaion fails if dropDeserializationError is set to true
-    val consumers2 = new SystemConsumers(msgChooser, consumer, serdeManager, dropDeserializationError = true)
+    val consumers2 = new SystemConsumers(msgChooser, consumer, systemAdmins, serdeManager, dropDeserializationError = true)
     consumers2.register(systemStreamPartition, "0")
+    consumers2.start
     consumer(system).putBytesMessage
     consumer(system).putStringMessage
     consumer(system).putBytesMessage
-    consumers2.start
 
     var notThrowException = true;
     try {
@@ -297,11 +310,13 @@ class TestSystemConsumers {
     val normalEnvelope = new IncomingMessageEnvelope(systemStreamPartition1, "1", "k", "v")
     val endOfStreamEnvelope = IncomingMessageEnvelope.buildEndOfStreamEnvelope(systemStreamPartition2)
     val consumer = new CustomPollResponseSystemConsumer(normalEnvelope)
+    val systemAdmins = Mockito.mock(classOf[SystemAdmins])
+    Mockito.when(systemAdmins.getSystemAdmin(system)).thenReturn(Mockito.mock(classOf[SystemAdmin]))
     val consumers = new SystemConsumers(new MockMessageChooser, Map(system -> consumer),
-      new SerdeManager, new SystemConsumersMetrics,
+      systemAdmins, new SerdeManager, new SystemConsumersMetrics,
       SystemConsumers.DEFAULT_NO_NEW_MESSAGES_TIMEOUT,
       SystemConsumers.DEFAULT_DROP_SERIALIZATION_ERROR,
-      SystemConsumers.DEFAULT_POLL_INTERVAL_MS, clock = () => 0)
+      TaskConfig.DEFAULT_POLL_INTERVAL_MS, clock = () => 0)
 
     consumers.register(systemStreamPartition1, "0")
     consumers.register(systemStreamPartition2, "0")
@@ -339,6 +354,26 @@ class TestSystemConsumers {
     assertTrue(consumer.lastPoll.contains(systemStreamPartition1))
   }
 
+  @Test
+  def testSystemConsumersRegistration {
+    val system = "test-system"
+    val stream = "some-stream"
+    val systemStreamPartition1 = new SystemStreamPartition(system, stream, new Partition(1))
+    val systemStreamPartition2 = new SystemStreamPartition(system, stream, new Partition(2))
+
+    val consumer = Mockito.mock(classOf[SystemConsumer])
+    val systemAdmins = Mockito.mock(classOf[SystemAdmins])
+    Mockito.when(systemAdmins.getSystemAdmin(system)).thenReturn(Mockito.mock(classOf[SystemAdmin]))
+
+    val consumers = new SystemConsumers(new MockMessageChooser, Map(system -> consumer),
+      systemAdmins, new SerdeManager, new SystemConsumersMetrics,
+      SystemConsumers.DEFAULT_NO_NEW_MESSAGES_TIMEOUT,
+      SystemConsumers.DEFAULT_DROP_SERIALIZATION_ERROR,
+      TaskConfig.DEFAULT_POLL_INTERVAL_MS, clock = () => 0)
+
+    consumers.register(systemStreamPartition1, "0")
+  }
+
   /**
    * A simple MockSystemConsumer that keeps track of what was polled, and lets
    * you define how many envelopes to return in the poll response. You can
@@ -374,20 +409,23 @@ class TestSystemConsumers {
    */
   private class SerializingConsumer extends BlockingEnvelopeMap {
     val systemStreamPartition = new SystemStreamPartition("test-system", "some-stream", new Partition(1))
-    def putBytesMessage {
+    def putBytesMessage() {
       put(systemStreamPartition, new IncomingMessageEnvelope(systemStreamPartition, "0", "0", "test".getBytes()))
     }
-    def putStringMessage {
+    def putStringMessage() {
       put(systemStreamPartition, new IncomingMessageEnvelope(systemStreamPartition, "0", "1", "test"))
     }
-    def start {}
-    def stop {}
-    def register { super.register(systemStreamPartition, "0") }
+    def start() {}
+    def stop() {}
+
+    override def register(systemStreamPartition: SystemStreamPartition, offset: String): Unit = {
+       super[BlockingEnvelopeMap].register(systemStreamPartition, offset)
+    }
   }
 }
 
 object TestSystemConsumers {
-  def getSystemConsumers(consumers: java.util.Map[String, SystemConsumer]) : SystemConsumers = {
-    new SystemConsumers(new DefaultChooser, consumers.asScala.toMap)
+  def getSystemConsumers(consumers: java.util.Map[String, SystemConsumer], systemAdmins: SystemAdmins = SystemAdmins.empty()) : SystemConsumers = {
+    new SystemConsumers(new DefaultChooser, consumers.asScala.toMap, systemAdmins)
   }
 }
