@@ -21,28 +21,31 @@ package org.apache.samza.checkpoint.kafka
 
 import org.apache.samza.SamzaException
 import org.apache.samza.checkpoint.{CheckpointManager, CheckpointManagerFactory}
-import org.apache.samza.config.JobConfig.Config2Job
 import org.apache.samza.config._
 import org.apache.samza.metrics.MetricsRegistry
 import org.apache.samza.system.{StreamSpec, SystemFactory}
 import org.apache.samza.system.kafka.KafkaStreamSpec
-import org.apache.samza.util.{KafkaUtil, Logging, Util, _}
+import org.apache.samza.util.ScalaJavaUtil.JavaOptionals
+import org.apache.samza.util.{KafkaUtil, Logging, _}
 
 class KafkaCheckpointManagerFactory extends CheckpointManagerFactory with Logging {
 
   def getCheckpointManager(config: Config, registry: MetricsRegistry): CheckpointManager = {
-    val jobName = config.getName.getOrElse(throw new SamzaException("Missing job name in configs"))
-    val jobId = config.getJobId
+    val jobConfig = new JobConfig(config)
+    val jobName = JavaOptionals.toRichOptional(jobConfig.getName).toOption
+      .getOrElse(throw new SamzaException("Missing job name in configs"))
+    val jobId = jobConfig.getJobId
 
     val kafkaConfig = new KafkaConfig(config)
     val checkpointSystemName = kafkaConfig.getCheckpointSystem.getOrElse(
       throw new SamzaException("No system defined for Kafka's checkpoint manager."))
 
-    val checkpointSystemFactoryName = new SystemConfig(config)
-      .getSystemFactory(checkpointSystemName)
-      .getOrElse(throw new SamzaException("Missing configuration: " + SystemConfig.SYSTEM_FACTORY format checkpointSystemName))
+    val systemConfig = new SystemConfig(config)
+    val checkpointSystemFactoryName = JavaOptionals.toRichOptional(systemConfig.getSystemFactory(checkpointSystemName))
+      .toOption
+      .getOrElse(throw new SamzaException("Missing configuration: " + SystemConfig.SYSTEM_FACTORY_FORMAT format checkpointSystemName))
 
-    val checkpointSystemFactory = Util.getObj(checkpointSystemFactoryName, classOf[SystemFactory])
+    val checkpointSystemFactory = ReflectionUtil.getObj(checkpointSystemFactoryName, classOf[SystemFactory])
     val checkpointTopic = KafkaUtil.getCheckpointTopic(jobName, jobId, config)
 
     info(s"Creating a KafkaCheckpointManager to consume from $checkpointTopic")
@@ -50,7 +53,7 @@ class KafkaCheckpointManagerFactory extends CheckpointManagerFactory with Loggin
         .copyWithReplicationFactor(kafkaConfig.getCheckpointReplicationFactor.get.toInt)
         .copyWithProperties(kafkaConfig.getCheckpointTopicProperties)
 
-    new KafkaCheckpointManager(checkpointSpec, checkpointSystemFactory, config.failOnCheckpointValidation, config,
+    new KafkaCheckpointManager(checkpointSpec, checkpointSystemFactory, jobConfig.failOnCheckpointValidation, config,
       new NoOpMetricsRegistry)
   }
 }
