@@ -103,7 +103,7 @@ public class TestKafkaCheckpointManagerJava {
     KafkaStreamSpec checkpointSpec = new KafkaStreamSpec(CHECKPOINT_TOPIC, CHECKPOINT_TOPIC,
         CHECKPOINT_SYSTEM, 1);
     Config mockConfig = mock(Config.class);
-    when(mockConfig.get(JobConfig.SSP_GROUPER_FACTORY())).thenReturn(GROUPER_FACTORY_CLASS);
+    when(mockConfig.get(JobConfig.SSP_GROUPER_FACTORY)).thenReturn(GROUPER_FACTORY_CLASS);
 
     // mock out a consumer that returns a single checkpoint IME
     SystemStreamPartition ssp = new SystemStreamPartition("system-1", "input-topic", new Partition(0));
@@ -129,7 +129,7 @@ public class TestKafkaCheckpointManagerJava {
     KafkaStreamSpec checkpointSpec = new KafkaStreamSpec(CHECKPOINT_TOPIC, CHECKPOINT_TOPIC,
         CHECKPOINT_SYSTEM, 1);
     Config mockConfig = mock(Config.class);
-    when(mockConfig.get(JobConfig.SSP_GROUPER_FACTORY())).thenReturn(GROUPER_FACTORY_CLASS);
+    when(mockConfig.get(JobConfig.SSP_GROUPER_FACTORY)).thenReturn(GROUPER_FACTORY_CLASS);
 
     // mock out a consumer that returns a single checkpoint IME
     SystemStreamPartition ssp = new SystemStreamPartition("system-1", "input-topic", new Partition(0));
@@ -156,7 +156,7 @@ public class TestKafkaCheckpointManagerJava {
     KafkaStreamSpec checkpointSpec = new KafkaStreamSpec(CHECKPOINT_TOPIC, CHECKPOINT_TOPIC,
         CHECKPOINT_SYSTEM, 1);
     Config mockConfig = mock(Config.class);
-    when(mockConfig.get(JobConfig.SSP_GROUPER_FACTORY())).thenReturn(GROUPER_FACTORY_CLASS);
+    when(mockConfig.get(JobConfig.SSP_GROUPER_FACTORY)).thenReturn(GROUPER_FACTORY_CLASS);
 
     // mock out a consumer that returns a single checkpoint IME
     SystemStreamPartition ssp = new SystemStreamPartition("system-1", "input-topic", new Partition(0));
@@ -187,7 +187,7 @@ public class TestKafkaCheckpointManagerJava {
     KafkaStreamSpec checkpointSpec = new KafkaStreamSpec(CHECKPOINT_TOPIC, CHECKPOINT_TOPIC,
         CHECKPOINT_SYSTEM, 1);
     Config mockConfig = mock(Config.class);
-    when(mockConfig.get(JobConfig.SSP_GROUPER_FACTORY())).thenReturn(GROUPER_FACTORY_CLASS);
+    when(mockConfig.get(JobConfig.SSP_GROUPER_FACTORY)).thenReturn(GROUPER_FACTORY_CLASS);
 
     SystemStreamPartition ssp = new SystemStreamPartition("system-1", "input-topic", new Partition(0));
 
@@ -196,7 +196,7 @@ public class TestKafkaCheckpointManagerJava {
 
     // mock out a consumer that returns ten checkpoint IMEs for the same ssp
     List<List<IncomingMessageEnvelope>> pollOutputs = new ArrayList<>();
-    for(int offset = oldestOffset; offset <= newestOffset; offset++) {
+    for (int offset = oldestOffset; offset <= newestOffset; offset++) {
       pollOutputs.add(ImmutableList.of(newCheckpointEnvelope(TASK1, ssp, Integer.toString(offset))));
     }
 
@@ -247,9 +247,9 @@ public class TestKafkaCheckpointManagerJava {
 
   private SystemFactory newFactory(SystemProducer producer, SystemConsumer consumer, SystemAdmin admin) {
     SystemFactory factory = mock(SystemFactory.class);
-    when(factory.getProducer(anyString(), any(Config.class), any(MetricsRegistry.class))).thenReturn(producer);
-    when(factory.getConsumer(anyString(), any(Config.class), any(MetricsRegistry.class))).thenReturn(consumer);
-    when(factory.getAdmin(anyString(), any(Config.class))).thenReturn(admin);
+    when(factory.getProducer(anyString(), any(Config.class), any(MetricsRegistry.class), anyString())).thenReturn(producer);
+    when(factory.getConsumer(anyString(), any(Config.class), any(MetricsRegistry.class), anyString())).thenReturn(consumer);
+    when(factory.getAdmin(anyString(), any(Config.class), anyString())).thenReturn(admin);
     return factory;
   }
 
