@@ -20,12 +20,13 @@
 package org.apache.samza.storage.kv
 
 import java.io.File
-import java.util
-import java.util.Comparator
+import java.nio.file.{Path, Paths}
 import java.util.concurrent.TimeUnit
 import java.util.concurrent.locks.ReentrantReadWriteLock
+import java.util.{Comparator, Optional}
 
 import org.apache.samza.SamzaException
+import org.apache.samza.checkpoint.CheckpointId
 import org.apache.samza.config.Config
 import org.apache.samza.util.Logging
 import org.rocksdb.{TtlDB, _}
@@ -172,25 +173,27 @@ class RocksDbKeyValueStore(
     }
   }
 
-  // Write batch from RocksDB API is not used currently because of: https://github.com/facebook/rocksdb/issues/262
   def putAll(entries: java.util.List[Entry[Array[Byte], Array[Byte]]]): Unit = ifOpen {
     metrics.putAlls.inc()
     val iter = entries.iterator
     var wrote = 0
     var deletes = 0
+    val writeBatch = new WriteBatch()
     while (iter.hasNext) {
       val curr = iter.next()
       if (curr.getValue == null) {
         deletes += 1
-        db.delete(writeOptions, curr.getKey)
+        writeBatch.remove(curr.getKey)
       } else {
         wrote += 1
         val key = curr.getKey
         val value = curr.getValue
         metrics.bytesWritten.inc(key.length + value.length)
-        db.put(writeOptions, key, value)
+        writeBatch.put(key, value)
       }
     }
+    db.write(writeOptions, writeBatch)
+    writeBatch.close()
     metrics.puts.inc(wrote)
     metrics.deletes.inc(deletes)
   }
@@ -234,8 +237,23 @@ class RocksDbKeyValueStore(
     trace("Flushed store: %s" format storeName)
   }
 
+  override def checkpoint(id: CheckpointId): Optional[Path] = {
+    val checkpoint = Checkpoint.create(db)
+    val checkpointPath = dir.getPath + "-" + id.toString
+    checkpoint.createCheckpoint(checkpointPath)
+    Optional.of(Paths.get(checkpointPath))
+  }
+
   def close(): Unit = {
+    trace("Calling compact range.")
     stateChangeLock.writeLock().lock()
+
+    // if auto-compaction is disabled, e.g., when bulk-loading
+    if(options.disableAutoCompactions()) {
+      trace("Auto compaction is disabled, invoking compact range.")
+      db.compactRange()
+    }
+
     try {
       trace("Closing.")
       if (stackAtFirstClose == null) { // first close
