@@ -27,7 +27,7 @@ import org.apache.samza.SamzaException;
 import org.apache.samza.rest.model.Job;
 import org.apache.samza.rest.model.JobStatus;
 import org.apache.samza.rest.resources.JobsResourceConfig;
-import org.apache.samza.util.Util;
+import org.apache.samza.util.ReflectionUtil;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -50,7 +50,7 @@ public abstract class AbstractJobProxy implements JobProxy {
     String jobProxyFactoryClassName = config.getJobProxyFactory();
     if (jobProxyFactoryClassName != null && !jobProxyFactoryClassName.isEmpty()) {
       try {
-        JobProxyFactory factory = Util.getObj(jobProxyFactoryClassName, JobProxyFactory.class);
+        JobProxyFactory factory = ReflectionUtil.getObj(jobProxyFactoryClassName, JobProxyFactory.class);
         return factory.getJobProxy(config);
       } catch (Exception e) {
         throw new SamzaException(e);
@@ -74,8 +74,8 @@ public abstract class AbstractJobProxy implements JobProxy {
       throws IOException, InterruptedException {
     List<Job> allJobs = new ArrayList<>();
     Collection<JobInstance> jobInstances = getAllJobInstances();
-    for(JobInstance jobInstance : jobInstances) {
-        allJobs.add(new Job(jobInstance.getJobName(), jobInstance.getJobId()));
+    for (JobInstance jobInstance : jobInstances) {
+      allJobs.add(new Job(jobInstance.getJobName(), jobInstance.getJobId()));
     }
     getJobStatusProvider().getJobStatuses(allJobs);
 
