@@ -19,7 +19,6 @@
 
 package org.apache.samza.sql.client.impl;
 
-import com.google.common.base.Joiner;
 import kafka.utils.ZkUtils;
 import org.I0Itec.zkclient.ZkClient;
 import org.I0Itec.zkclient.ZkConnection;
@@ -31,22 +30,24 @@ import org.apache.samza.config.*;
 import org.apache.samza.container.grouper.task.SingleContainerGrouperFactory;
 import org.apache.samza.job.ApplicationStatus;
 import org.apache.samza.serializers.StringSerdeFactory;
-import org.apache.samza.sql.avro.AvroRelSchemaProvider;
+import org.apache.samza.sql.client.exceptions.ExecutorException;
+import org.apache.samza.sql.client.interfaces.ExecutionStatus;
 import org.apache.samza.sql.client.interfaces.*;
+import org.apache.samza.sql.client.util.Pair;
 import org.apache.samza.sql.client.util.RandomAccessQueue;
 import org.apache.samza.sql.dsl.SamzaSqlDslConverter;
 import org.apache.samza.sql.dsl.SamzaSqlDslConverterFactory;
-import org.apache.samza.sql.fn.FlattenUdf;
-import org.apache.samza.sql.fn.RegexMatchUdf;
 import org.apache.samza.sql.impl.ConfigBasedIOResolverFactory;
-import org.apache.samza.sql.impl.ConfigBasedUdfResolver;
 import org.apache.samza.sql.interfaces.RelSchemaProvider;
 import org.apache.samza.sql.interfaces.RelSchemaProviderFactory;
 import org.apache.samza.sql.interfaces.SqlIOConfig;
 import org.apache.samza.sql.interfaces.SqlIOResolver;
 import org.apache.samza.sql.runner.SamzaSqlApplicationConfig;
 import org.apache.samza.sql.runner.SamzaSqlApplicationRunner;
-import org.apache.samza.sql.testutil.JsonUtil;
+import org.apache.samza.sql.schema.SamzaSqlFieldType;
+import org.apache.samza.sql.schema.SqlFieldSchema;
+import org.apache.samza.sql.schema.SqlSchema;
+import org.apache.samza.sql.util.JsonUtil;
 import org.apache.samza.standalone.PassthroughJobCoordinatorFactory;
 import org.apache.samza.system.OutgoingMessageEnvelope;
 import org.apache.samza.system.kafka.KafkaSystemFactory;
@@ -88,7 +89,8 @@ public class SamzaExecutor implements SqlExecutor {
           new RandomAccessQueue<>(OutgoingMessageEnvelope.class, RANDOM_ACCESS_QUEUE_CAPACITY);
   private static AtomicInteger execIdSeq = new AtomicInteger(0);
   private Map<Integer, SamzaSqlApplicationRunner> executions = new HashMap<>();
-  private String lastErrorMsg = "";
+  private SamzaExecutorEnvironmentVariableHandler environmentVariableHandler =
+          new SamzaExecutorEnvironmentVariableHandler();
 
   // -- implementation of SqlExecutor ------------------------------------------
 
@@ -97,7 +99,7 @@ public class SamzaExecutor implements SqlExecutor {
   }
 
   @Override
-  public void stop(ExecutionContext context) {
+  public void stop(ExecutionContext context) throws ExecutorException {
     Iterator<Integer> iter = executions.keySet().iterator();
     while (iter.hasNext()) {
       stopExecution(context, iter.next());
@@ -107,78 +109,74 @@ public class SamzaExecutor implements SqlExecutor {
   }
 
   @Override
-  public List<String> listTables(ExecutionContext context) {
-    /**
-     * TODO: currently Shell can only talk to Kafka system, but we should use a general way
-     *       to connect to different systems.
-     */
-    lastErrorMsg = "";
-    String address = context.getConfigMap().getOrDefault(SAMZA_SQL_SYSTEM_KAFKA_ADDRESS, DEFAULT_SERVER_ADDRESS);
-    List<String> tables = null;
+  public EnvironmentVariableHandler getEnvironmentVariableHandler() {
+    return environmentVariableHandler;
+  }
+
+  @Override
+  public List<String> listTables(ExecutionContext context) throws ExecutorException {
+    String address = environmentVariableHandler.getEnvironmentVariable(SAMZA_SQL_SYSTEM_KAFKA_ADDRESS);
+    if (address == null || address.isEmpty()) {
+      address = DEFAULT_SERVER_ADDRESS;
+    }
     try {
       ZkUtils zkUtils = new ZkUtils(new ZkClient(address, DEFAULT_ZOOKEEPER_CLIENT_TIMEOUT),
           new ZkConnection(address), false);
-      tables = JavaConversions.seqAsJavaList(zkUtils.getAllTopics())
+      return JavaConversions.seqAsJavaList(zkUtils.getAllTopics())
         .stream()
         .map(x -> SAMZA_SYSTEM_KAFKA + "." + x)
         .collect(Collectors.toList());
     } catch (ZkTimeoutException ex) {
-      lastErrorMsg = ex.toString();
-      LOG.error(lastErrorMsg);
+      throw new ExecutorException(ex);
     }
-    return tables;
   }
 
   @Override
-  public SqlSchema getTableSchema(ExecutionContext context, String tableName) {
-    /**
+  public SqlSchema getTableSchema(ExecutionContext context, String tableName) throws ExecutorException {
+    /*
      *  currently Shell works only for systems that has Avro schemas
      */
-    lastErrorMsg = "";
     int execId = execIdSeq.incrementAndGet();
-    Map<String, String> staticConfigs = fetchSamzaSqlConfig(execId, context);
+    Map<String, String> staticConfigs = fetchSamzaSqlConfig(execId);
     Config samzaSqlConfig = new MapConfig(staticConfigs);
-    SqlSchema sqlSchema = null;
+    SqlSchema sqlSchema;
     try {
       SqlIOResolver ioResolver = SamzaSqlApplicationConfig.createIOResolver(samzaSqlConfig);
       SqlIOConfig sourceInfo = ioResolver.fetchSourceInfo(tableName);
       RelSchemaProvider schemaProvider =
-              SamzaSqlApplicationConfig.initializePlugin("RelSchemaProvider", sourceInfo.getRelSchemaProviderName(),
-                      samzaSqlConfig, SamzaSqlApplicationConfig.CFG_FMT_REL_SCHEMA_PROVIDER_DOMAIN,
-                      (o, c) -> ((RelSchemaProviderFactory) o).create(sourceInfo.getSystemStream(), c));
-      AvroRelSchemaProvider avroSchemaProvider = (AvroRelSchemaProvider) schemaProvider;
-      String schema = avroSchemaProvider.getSchema(sourceInfo.getSystemStream());
-      sqlSchema = AvroSqlSchemaConverter.convertAvroToSamzaSqlSchema(schema);
+        SamzaSqlApplicationConfig.initializePlugin("RelSchemaProvider", sourceInfo.getRelSchemaProviderName(),
+          samzaSqlConfig, SamzaSqlApplicationConfig.CFG_FMT_REL_SCHEMA_PROVIDER_DOMAIN,
+          (o, c) -> ((RelSchemaProviderFactory) o).create(sourceInfo.getSystemStream(), c));
+      sqlSchema =  schemaProvider.getSqlSchema();
     } catch (SamzaException ex) {
-      lastErrorMsg = ex.toString();
-      LOG.error(lastErrorMsg);
+      throw new ExecutorException(ex);
     }
     return sqlSchema;
   }
 
   @Override
-  public QueryResult executeQuery(ExecutionContext context, String statement) {
-    lastErrorMsg = "";
+  public QueryResult executeQuery(ExecutionContext context, String statement) throws ExecutorException {
     outputData.clear();
 
     int execId = execIdSeq.incrementAndGet();
-    Map<String, String> staticConfigs = fetchSamzaSqlConfig(execId, context);
+    Map<String, String> staticConfigs = fetchSamzaSqlConfig(execId);
     List<String> sqlStmts = formatSqlStmts(Collections.singletonList(statement));
     staticConfigs.put(SamzaSqlApplicationConfig.CFG_SQL_STMTS_JSON, JsonUtil.toJson(sqlStmts));
 
     SamzaSqlApplicationRunner runner;
     try {
       runner = new SamzaSqlApplicationRunner(true, new MapConfig(staticConfigs));
-      runner.run();
+      runner.run(null);
     } catch (SamzaException ex) {
-      lastErrorMsg = ex.toString();
-      LOG.error(lastErrorMsg);
-      return new QueryResult(execId, null, false);
+      throw new ExecutorException(ex);
     }
     executions.put(execId, runner);
     LOG.debug("Executing sql. Id ", execId);
 
-    return new QueryResult(execId, generateResultSchema(new MapConfig(staticConfigs)), true);
+    SqlSchema resultSchema = null;
+    // TODO: after fixing the TODO in generateResultSchema function, we can uncomment the following piece of code.
+    // resultSchema = generateResultSchema(new MapConfig(staticConfigs));
+    return new QueryResult(execId, resultSchema);
   }
 
   @Override
@@ -190,7 +188,7 @@ public class SamzaExecutor implements SqlExecutor {
   public List<String[]> retrieveQueryResult(ExecutionContext context, int startRow, int endRow) {
     List<String[]> results = new ArrayList<>();
     for (OutgoingMessageEnvelope row : outputData.get(startRow, endRow)) {
-      results.add(getFormattedRow(context, row));
+      results.add(getFormattedRow(row));
     }
     return results;
   }
@@ -199,23 +197,19 @@ public class SamzaExecutor implements SqlExecutor {
   public List<String[]> consumeQueryResult(ExecutionContext context, int startRow, int endRow) {
     List<String[]> results = new ArrayList<>();
     for (OutgoingMessageEnvelope row : outputData.consume(startRow, endRow)) {
-      results.add(getFormattedRow(context, row));
+      results.add(getFormattedRow(row));
     }
     return results;
   }
 
   @Override
-  public NonQueryResult executeNonQuery(ExecutionContext context, File sqlFile) {
-    lastErrorMsg = "";
-
+  public NonQueryResult executeNonQuery(ExecutionContext context, File sqlFile) throws ExecutorException {
     LOG.info("Sql file path: " + sqlFile.getPath());
-    List<String> executedStmts = new ArrayList<>();
+    List<String> executedStmts;
     try {
       executedStmts = Files.lines(Paths.get(sqlFile.getPath())).collect(Collectors.toList());
     } catch (IOException e) {
-      lastErrorMsg = String.format("Unable to parse the sql file %s. %s", sqlFile.getPath(), e.toString());
-      LOG.error(lastErrorMsg);
-      return new NonQueryResult(-1, false);
+      throw new ExecutorException(e);
     }
     LOG.info("Sql statements in Sql file: " + executedStmts.toString());
 
@@ -223,105 +217,71 @@ public class SamzaExecutor implements SqlExecutor {
     List<String> nonSubmittedStmts = new ArrayList<>();
     validateExecutedStmts(executedStmts, submittedStmts, nonSubmittedStmts);
     if (submittedStmts.isEmpty()) {
-      lastErrorMsg = "Nothing to execute. Note: SELECT statements are ignored.";
-      LOG.warn("Nothing to execute. Statements in the Sql file: {}", nonSubmittedStmts);
-      return new NonQueryResult(-1, false);
+      throw new ExecutorException("Nothing to execute. Note: SELECT statements are ignored.");
     }
     NonQueryResult result = executeNonQuery(context, submittedStmts);
-    return new NonQueryResult(result.getExecutionId(), result.succeeded(), submittedStmts, nonSubmittedStmts);
+    return new NonQueryResult(result.getExecutionId(), submittedStmts, nonSubmittedStmts);
   }
 
   @Override
-  public NonQueryResult executeNonQuery(ExecutionContext context, List<String> statement) {
-    lastErrorMsg = "";
-
+  public NonQueryResult executeNonQuery(ExecutionContext context, List<String> statement) throws ExecutorException {
     int execId = execIdSeq.incrementAndGet();
-    Map<String, String> staticConfigs = fetchSamzaSqlConfig(execId, context);
+    Map<String, String> staticConfigs = fetchSamzaSqlConfig(execId);
     staticConfigs.put(SamzaSqlApplicationConfig.CFG_SQL_STMTS_JSON, JsonUtil.toJson(formatSqlStmts(statement)));
 
     SamzaSqlApplicationRunner runner;
     try {
       runner = new SamzaSqlApplicationRunner(true, new MapConfig(staticConfigs));
-      runner.run();
+      runner.run(null);
     } catch (SamzaException ex) {
-      lastErrorMsg = ex.toString();
-      LOG.error(lastErrorMsg);
-      return new NonQueryResult(execId, false);
+      throw new ExecutorException(ex);
     }
     executions.put(execId, runner);
     LOG.debug("Executing sql. Id ", execId);
 
-    return new NonQueryResult(execId, true);
+    return new NonQueryResult(execId);
   }
 
   @Override
-  public boolean stopExecution(ExecutionContext context, int exeId) {
-    lastErrorMsg = "";
-
+  public void stopExecution(ExecutionContext context, int exeId) throws ExecutorException {
     SamzaSqlApplicationRunner runner = executions.get(exeId);
     if (runner != null) {
       LOG.debug("Stopping execution ", exeId);
-
       try {
         runner.kill();
       } catch (SamzaException ex) {
-        lastErrorMsg = ex.toString();
-        LOG.debug(lastErrorMsg);
-        return false;
+        throw new ExecutorException(ex);
       }
-
-      try {
-        Thread.sleep(500); // wait for a second
-      } catch (InterruptedException e) {
-        e.printStackTrace();
-      }
-
-      return true;
     } else {
-      lastErrorMsg = "Trying to stop a non-existing SQL execution " + exeId;
-      LOG.warn(lastErrorMsg);
-      return false;
+      throw new ExecutorException("Trying to stop a non-existing SQL execution " + exeId);
     }
   }
 
   @Override
-  public boolean removeExecution(ExecutionContext context, int exeId) {
-    lastErrorMsg = "";
-
+  public void removeExecution(ExecutionContext context, int exeId) throws ExecutorException {
     SamzaSqlApplicationRunner runner = executions.get(exeId);
     if (runner != null) {
       if (runner.status().getStatusCode().equals(ApplicationStatus.StatusCode.Running)) {
-        lastErrorMsg = "Trying to remove a ongoing execution " + exeId;
-        LOG.error(lastErrorMsg);
-        return false;
+        throw new ExecutorException("Trying to remove an ongoing execution " + exeId);
       }
       executions.remove(exeId);
-      LOG.debug("Stopping execution ", exeId);
-      return true;
     } else {
-      lastErrorMsg = "Trying to remove a non-existing SQL execution " + exeId;
-      LOG.warn(lastErrorMsg);
-      return false;
+      throw new ExecutorException("Trying to remove a non-existing SQL execution " + exeId);
     }
   }
 
   @Override
-  public ExecutionStatus queryExecutionStatus(int execId) {
+  public ExecutionStatus queryExecutionStatus(int execId) throws ExecutorException {
     SamzaSqlApplicationRunner runner = executions.get(execId);
     if (runner == null) {
-      return null;
+      throw new ExecutorException("Execution " + execId + " does not exist.");
     }
     return queryExecutionStatus(runner);
   }
 
-  @Override
-  public String getErrorMsg() {
-    return lastErrorMsg;
-  }
-
   @Override
   public List<SqlFunction> listFunctions(ExecutionContext context) {
-    /**
+    /*
      * TODO: currently the Shell only shows some UDFs supported by Samza internally. We may need to require UDFs
      *       to provide a function of getting their "SamzaSqlUdfDisplayInfo", then we can get the UDF information from
      *       SamzaSqlApplicationConfig.udfResolver(or SamzaSqlApplicationConfig.udfMetadata) instead of registering
@@ -329,24 +289,33 @@ public class SamzaExecutor implements SqlExecutor {
      */
     List<SqlFunction> udfs = new ArrayList<>();
     udfs.add(new SamzaSqlUdfDisplayInfo("RegexMatch", "Matches the string to the regex",
-            Arrays.asList(SamzaSqlFieldType.createPrimitiveFieldType(SamzaSqlFieldType.TypeName.STRING),
-                    SamzaSqlFieldType.createPrimitiveFieldType(SamzaSqlFieldType.TypeName.STRING)),
-            SamzaSqlFieldType.createPrimitiveFieldType(SamzaSqlFieldType.TypeName.BOOLEAN)));
+            Arrays.asList(SqlFieldSchema.createPrimitiveSchema(SamzaSqlFieldType.STRING, false, false),
+                SqlFieldSchema.createPrimitiveSchema(SamzaSqlFieldType.STRING, false, false)),
+        SqlFieldSchema.createPrimitiveSchema(SamzaSqlFieldType.BOOLEAN, false, false)));
 
     return udfs;
   }
 
+  @Override
+  public String getVersion() {
+    return this.getClass().getPackage().getImplementationVersion();
+  }
+
   static void saveOutputMessage(OutgoingMessageEnvelope messageEnvelope) {
     outputData.add(messageEnvelope);
   }
 
-  static Map<String, String> fetchSamzaSqlConfig(int execId, ExecutionContext executionContext) {
+  private boolean processEnvironmentVariable(String name, String value) {
+    return true;
+  }
+
+  Map<String, String> fetchSamzaSqlConfig(int execId) {
     HashMap<String, String> staticConfigs = new HashMap<>();
 
-    staticConfigs.put(JobConfig.JOB_NAME(), "sql-job-" + execId);
-    staticConfigs.put(JobConfig.PROCESSOR_ID(), String.valueOf(execId));
+    staticConfigs.put(JobConfig.JOB_NAME, "sql-job-" + execId);
+    staticConfigs.put(JobConfig.PROCESSOR_ID, String.valueOf(execId));
     staticConfigs.put(JobCoordinatorConfig.JOB_COORDINATOR_FACTORY, PassthroughJobCoordinatorFactory.class.getName());
-    staticConfigs.put(TaskConfig.GROUPER_FACTORY(), SingleContainerGrouperFactory.class.getName());
+    staticConfigs.put(TaskConfig.GROUPER_FACTORY, SingleContainerGrouperFactory.class.getName());
 
     staticConfigs.put(SamzaSqlApplicationConfig.CFG_IO_RESOLVER, "config");
     String configIOResolverDomain =
@@ -355,11 +324,6 @@ public class SamzaExecutor implements SqlExecutor {
         ConfigBasedIOResolverFactory.class.getName());
 
     staticConfigs.put(SamzaSqlApplicationConfig.CFG_UDF_RESOLVER, "config");
-    String configUdfResolverDomain = String.format(SamzaSqlApplicationConfig.CFG_FMT_UDF_RESOLVER_DOMAIN, "config");
-    staticConfigs.put(configUdfResolverDomain + SamzaSqlApplicationConfig.CFG_FACTORY,
-        ConfigBasedUdfResolver.class.getName());
-    staticConfigs.put(configUdfResolverDomain + ConfigBasedUdfResolver.CFG_UDF_CLASSES,
-        Joiner.on(",").join(RegexMatchUdf.class.getName(), FlattenUdf.class.getName()));
 
     staticConfigs.put("serializers.registry.string.class", StringSerdeFactory.class.getName());
     staticConfigs.put("serializers.registry.avro.class", AvroSerDeFactory.class.getName());
@@ -408,11 +372,10 @@ public class SamzaExecutor implements SqlExecutor {
         configAvroRelSchemaProviderDomain + FileSystemAvroRelSchemaProviderFactory.CFG_SCHEMA_DIR,
         "/tmp/schemas/");
 
-    /* TODO: we need to validate and read configurations from shell-defaults.conf (aka. "executionContext"),
-     *       and update their value if they've been included in staticConfigs. We could handle these logic
-     *       Shell level, or in Executor level.
-     */
-    staticConfigs.putAll(executionContext.getConfigMap());
+    List<Pair<String, String>> allEnvironmentVariables = environmentVariableHandler.getAllEnvironmentVariables();
+    for (Pair<String, String> p : allEnvironmentVariables) {
+      staticConfigs.put(p.getL(), p.getR());
+    }
 
     return staticConfigs;
   }
@@ -454,12 +417,14 @@ public class SamzaExecutor implements SqlExecutor {
       colTypeNames.add(dataTypeField.getType().toString());
     }
 
-    return new SqlSchema(colNames, colTypeNames);
+    // TODO: Need to find a way to convert the relational to SQL Schema. After fixing this TODO, please resolve the TODOs
+    // in QueryResult class and executeQuery().
+    return new SqlSchema(colNames, Collections.emptyList());
   }
 
-  private String[] getFormattedRow(ExecutionContext context, OutgoingMessageEnvelope row) {
+  private String[] getFormattedRow(OutgoingMessageEnvelope row) {
     String[] formattedRow = new String[1];
-    String outputFormat = context.getConfigMap().get(SAMZA_SQL_OUTPUT);
+    String outputFormat = environmentVariableHandler.getEnvironmentVariable(SAMZA_SQL_OUTPUT);
     if (outputFormat == null || !outputFormat.equalsIgnoreCase(MessageFormat.PRETTY.toString())) {
       formattedRow[0] = getCompressedFormat(row);
     } else {
@@ -468,9 +433,9 @@ public class SamzaExecutor implements SqlExecutor {
     return formattedRow;
   }
 
-  private ExecutionStatus queryExecutionStatus(SamzaSqlApplicationRunner runner) {
-    lastErrorMsg = "";
-    switch (runner.status().getStatusCode()) {
+  private ExecutionStatus queryExecutionStatus(SamzaSqlApplicationRunner runner) throws ExecutorException {
+    ApplicationStatus.StatusCode code = runner.status().getStatusCode();
+    switch (code) {
       case New:
         return ExecutionStatus.New;
       case Running:
@@ -479,11 +444,8 @@ public class SamzaExecutor implements SqlExecutor {
         return ExecutionStatus.SuccessfulFinish;
       case UnsuccessfulFinish:
         return ExecutionStatus.UnsuccessfulFinish;
-      default:
-        lastErrorMsg = String.format("Unsupported execution status %s",
-                runner.status().getStatusCode().toString());
-        return null;
     }
+    throw new ExecutorException("Unsupported status code: " + code);
   }
 
   private String getPrettyFormat(OutgoingMessageEnvelope envelope) {
@@ -493,9 +455,9 @@ public class SamzaExecutor implements SqlExecutor {
     try {
       Object json = mapper.readValue(value, Object.class);
       formattedValue = mapper.writerWithDefaultPrettyPrinter().writeValueAsString(json);
-    } catch (IOException e) {
+    } catch (IOException ex) {
       formattedValue = value;
-      LOG.error("Error while formatting json", e);
+      LOG.error("getPrettyFormat failed with exception while formatting json ", ex);
     }
     return formattedValue;
   }
@@ -508,4 +470,24 @@ public class SamzaExecutor implements SqlExecutor {
     PRETTY,
     COMPACT
   }
+
+  class SamzaExecutorEnvironmentVariableHandler extends EnvironmentVariableHandlerImpl {
+    SamzaExecutorEnvironmentVariableHandler() {
+    }
+
+    protected EnvironmentVariableSpecs initializeEnvironmentVariableSpecs() {
+      HashMap<String, EnvironmentVariableSpecs.Spec> specMap = new HashMap<>();
+      specMap.put("samza.sql.output",
+              new EnvironmentVariableSpecs.Spec(new String[]{"pretty", "compact"}, "compact"));
+      return new EnvironmentVariableSpecs(specMap);
+    }
+
+    protected boolean processEnvironmentVariable(String name, String value) {
+      return SamzaExecutor.this.processEnvironmentVariable(name, value);
+    }
+
+    protected boolean isAcceptUnknowName() {
+      return true;
+    }
+  }
 }
