@@ -21,19 +21,25 @@ package org.apache.samza.sql.fn;
 
 import java.util.regex.Pattern;
 import org.apache.samza.config.Config;
+import org.apache.samza.context.Context;
+import org.apache.samza.sql.schema.SamzaSqlFieldType;
+import org.apache.samza.sql.udfs.SamzaSqlUdf;
+import org.apache.samza.sql.udfs.SamzaSqlUdfMethod;
 import org.apache.samza.sql.udfs.ScalarUdf;
 
 
 /**
  * Simple RegexMatch Udf.
  */
-public class RegexMatchUdf implements ScalarUdf<Boolean> {
+@SamzaSqlUdf(name = "RegexMatch", description = "Function to perform the regex match.")
+public class RegexMatchUdf implements ScalarUdf {
   @Override
-  public void init(Config config) {
+  public void init(Config config, Context context) {
 
   }
 
-  public Boolean execute(Object... args) {
-    return Pattern.matches((String) args[0], (String) args[1]);
+  @SamzaSqlUdfMethod(params = {SamzaSqlFieldType.STRING, SamzaSqlFieldType.STRING})
+  public Boolean match(String regexPattern, String input) {
+    return Pattern.matches(regexPattern, input);
   }
 }
