@@ -32,6 +32,7 @@ import org.apache.samza.operators.windows.AccumulationMode;
 import org.apache.samza.operators.windows.Windows;
 import org.apache.samza.serializers.LongSerde;
 import org.apache.samza.sql.data.SamzaSqlRelMessage;
+import org.apache.samza.sql.data.SamzaSqlRelMsgMetadata;
 import org.apache.samza.sql.serializers.SamzaSqlRelMessageSerdeFactory;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
@@ -44,10 +45,12 @@ import org.slf4j.LoggerFactory;
 class LogicalAggregateTranslator {
 
   private static final Logger log = LoggerFactory.getLogger(JoinTranslator.class);
-  private int windowId;
+  private String logicalOpId;
+  private String changeLogStorePrefix;
 
-  LogicalAggregateTranslator(int windowId) {
-    this.windowId = windowId;
+  LogicalAggregateTranslator(String logicalOpId, String changeLogStorePrefix) {
+    this.logicalOpId = logicalOpId;
+    this.changeLogStorePrefix = changeLogStorePrefix + (changeLogStorePrefix.isEmpty() ? "" : "_");
   }
 
   void translate(final LogicalAggregate aggregate, final TranslatorContext context) {
@@ -63,26 +66,29 @@ class LogicalAggregateTranslator {
 
     MessageStream<SamzaSqlRelMessage> outputStream =
         inputStream
+            .map(new TranslatorInputMetricsMapFunction(logicalOpId))
             .window(Windows.keyedTumblingWindow(m -> m,
                 Duration.ofMillis(context.getExecutionContext().getSamzaSqlApplicationConfig().getWindowDurationMs()),
                 initialValue,
                 foldCountFn,
                 new SamzaSqlRelMessageSerdeFactory.SamzaSqlRelMessageSerde(),
                 new LongSerde())
-                .setAccumulationMode(AccumulationMode.DISCARDING), "tumblingWindow_" + windowId)
+                .setAccumulationMode(
+                    AccumulationMode.DISCARDING), changeLogStorePrefix + "_tumblingWindow_" + logicalOpId)
             .map(windowPane -> {
-                List<String> fieldNames = windowPane.getKey().getKey().getSamzaSqlRelRecord().getFieldNames();
-                List<Object> fieldValues = windowPane.getKey().getKey().getSamzaSqlRelRecord().getFieldValues();
-                fieldNames.add(aggFieldNames.get(0));
-                fieldValues.add(windowPane.getMessage());
-                return new SamzaSqlRelMessage(fieldNames, fieldValues);
-              });
+              List<String> fieldNames = windowPane.getKey().getKey().getSamzaSqlRelRecord().getFieldNames();
+              List<Object> fieldValues = windowPane.getKey().getKey().getSamzaSqlRelRecord().getFieldValues();
+              fieldNames.add(aggFieldNames.get(0));
+              fieldValues.add(windowPane.getMessage());
+              return new SamzaSqlRelMessage(fieldNames, fieldValues, new SamzaSqlRelMsgMetadata(0L, 0L));
+            });
     context.registerMessageStream(aggregate.getId(), outputStream);
+    outputStream.map(new TranslatorOutputMetricsMapFunction(logicalOpId));
   }
 
   private ArrayList<String> getAggFieldNames(LogicalAggregate aggregate) {
     return aggregate.getAggCallList().stream().collect(ArrayList::new, (names, aggCall) -> names.add(aggCall.getName()),
-        (n1, n2) -> n1.addAll(n2));
+      (n1, n2) -> n1.addAll(n2));
   }
 
   void validateAggregateFunctions(final LogicalAggregate aggregate) {
