@@ -1,25 +1,24 @@
 /*
-* Licensed to the Apache Software Foundation (ASF) under one
-* or more contributor license agreements.  See the NOTICE file
-* distributed with this work for additional information
-* regarding copyright ownership.  The ASF licenses this file
-* to you under the Apache License, Version 2.0 (the
-* "License"); you may not use this file except in compliance
-* with the License.  You may obtain a copy of the License at
-*
-*   http://www.apache.org/licenses/LICENSE-2.0
-*
-* Unless required by applicable law or agreed to in writing,
-* software distributed under the License is distributed on an
-* "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
-* KIND, either express or implied.  See the License for the
-* specific language governing permissions and limitations
-* under the License.
-*/
+ * Licensed to the Apache Software Foundation (ASF) under one
+ * or more contributor license agreements.  See the NOTICE file
+ * distributed with this work for additional information
+ * regarding copyright ownership.  The ASF licenses this file
+ * to you under the Apache License, Version 2.0 (the
+ * "License"); you may not use this file except in compliance
+ * with the License.  You may obtain a copy of the License at
+ *
+ *   http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing,
+ * software distributed under the License is distributed on an
+ * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+ * KIND, either express or implied.  See the License for the
+ * specific language governing permissions and limitations
+ * under the License.
+ */
 
 package org.apache.samza.sql.avro;
 
-import com.google.common.base.Joiner;
 import java.io.ByteArrayOutputStream;
 import java.io.IOException;
 import java.nio.ByteBuffer;
@@ -51,6 +50,7 @@ import org.apache.samza.config.MapConfig;
 import org.apache.samza.operators.KV;
 import org.apache.samza.sql.avro.schemas.AddressRecord;
 import org.apache.samza.sql.avro.schemas.ComplexRecord;
+import org.apache.samza.sql.avro.schemas.ComplexUnion;
 import org.apache.samza.sql.avro.schemas.Kind;
 import org.apache.samza.sql.avro.schemas.MyFixed;
 import org.apache.samza.sql.avro.schemas.PhoneNumber;
@@ -58,8 +58,11 @@ import org.apache.samza.sql.avro.schemas.Profile;
 import org.apache.samza.sql.avro.schemas.SimpleRecord;
 import org.apache.samza.sql.avro.schemas.StreetNumRecord;
 import org.apache.samza.sql.data.SamzaSqlRelMessage;
+import org.apache.samza.sql.planner.RelSchemaConverter;
+import org.apache.samza.sql.schema.SqlSchema;
 import org.apache.samza.system.SystemStream;
 import org.junit.Assert;
+import org.junit.Ignore;
 import org.junit.Test;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
@@ -69,14 +72,16 @@ public class TestAvroRelConversion {
 
   private static final Logger LOG = LoggerFactory.getLogger(TestAvroRelConversion.class);
   private static final byte[] DEFAULT_TRACKING_ID_BYTES =
-      {76, 75, -24, 10, 33, -117, 24, -52, -110, -39, -5, 102, 65, 57, -62, -1};
+    {76, 75, -24, 10, 33, -117, 24, -52, -110, -39, -5, 102, 65, 57, -62, -1};
 
   private final AvroRelConverter simpleRecordAvroRelConverter;
   private final AvroRelConverter complexRecordAvroRelConverter;
   private final AvroRelConverter nestedRecordAvroRelConverter;
+  private final AvroRelConverter complexUnionAvroRelConverter;
   private final AvroRelSchemaProvider simpleRecordSchemaProvider;
   private final AvroRelSchemaProvider complexRecordSchemaProvider;
   private final AvroRelSchemaProvider nestedRecordSchemaProvider;
+  private final AvroRelSchemaProvider complexUnionSchemaProvider;
 
   private int id = 1;
   private boolean boolValue = true;
@@ -87,18 +92,20 @@ public class TestAvroRelConversion {
   private MyFixed fixedBytes = new MyFixed();
   private long longValue = 200L;
 
-  private HashMap<String, String> mapValue = new HashMap<String, String>() {{
-    put("key1", "val1");
-    put("key2", "val2");
-    put("key3", "val3");
-  }};
+  private HashMap<String, String> mapValue = new HashMap<String, String>() { {
+      put("key1", "val1");
+      put("key2", "val2");
+      put("key3", "val3");
+    } };
   private List<String> arrayValue = Arrays.asList("val1", "val2", "val3");
+  RelSchemaConverter relSchemaConverter = new RelSchemaConverter();
 
   public TestAvroRelConversion() {
     Map<String, String> props = new HashMap<>();
     SystemStream ss1 = new SystemStream("test", "complexRecord");
     SystemStream ss2 = new SystemStream("test", "simpleRecord");
     SystemStream ss3 = new SystemStream("test", "nestedRecord");
+    SystemStream ss4 = new SystemStream("test", "complexUnion");
     props.put(
         String.format(ConfigBasedAvroRelSchemaProviderFactory.CFG_SOURCE_SCHEMA, ss1.getSystem(), ss1.getStream()),
         ComplexRecord.SCHEMA$.toString());
@@ -108,15 +115,20 @@ public class TestAvroRelConversion {
     props.put(
         String.format(ConfigBasedAvroRelSchemaProviderFactory.CFG_SOURCE_SCHEMA, ss3.getSystem(), ss3.getStream()),
         Profile.SCHEMA$.toString());
+    props.put(
+        String.format(ConfigBasedAvroRelSchemaProviderFactory.CFG_SOURCE_SCHEMA, ss4.getSystem(), ss4.getStream()),
+        ComplexUnion.SCHEMA$.toString());
 
     ConfigBasedAvroRelSchemaProviderFactory factory = new ConfigBasedAvroRelSchemaProviderFactory();
 
     complexRecordSchemaProvider = (AvroRelSchemaProvider) factory.create(ss1, new MapConfig(props));
     simpleRecordSchemaProvider = (AvroRelSchemaProvider) factory.create(ss2, new MapConfig(props));
     nestedRecordSchemaProvider = (AvroRelSchemaProvider) factory.create(ss3, new MapConfig(props));
+    complexUnionSchemaProvider = (AvroRelSchemaProvider) factory.create(ss3, new MapConfig(props));
     complexRecordAvroRelConverter = new AvroRelConverter(ss1, complexRecordSchemaProvider, new MapConfig());
     simpleRecordAvroRelConverter = new AvroRelConverter(ss2, simpleRecordSchemaProvider, new MapConfig());
     nestedRecordAvroRelConverter = new AvroRelConverter(ss3, nestedRecordSchemaProvider, new MapConfig());
+    complexUnionAvroRelConverter = new AvroRelConverter(ss4, complexUnionSchemaProvider, new MapConfig());
 
     fixedBytes.bytes(DEFAULT_TRACKING_ID_BYTES);
   }
@@ -125,7 +137,8 @@ public class TestAvroRelConversion {
   public void testSimpleSchemaConversion() {
     String streamName = "stream";
 
-    RelDataType dataType = simpleRecordSchemaProvider.getRelationalSchema();
+    SqlSchema sqlSchema = simpleRecordSchemaProvider.getSqlSchema();
+    RelDataType dataType = relSchemaConverter.convertToRelSchema(sqlSchema);
     junit.framework.Assert.assertTrue(dataType instanceof RelRecordType);
     RelRecordType recordType = (RelRecordType) dataType;
 
@@ -140,14 +153,14 @@ public class TestAvroRelConversion {
 
   @Test
   public void testComplexSchemaConversion() {
-    RelDataType relSchema = complexRecordSchemaProvider.getRelationalSchema();
+    RelDataType relSchema = relSchemaConverter.convertToRelSchema(complexRecordSchemaProvider.getSqlSchema());
 
     LOG.info("Relational schema " + relSchema);
   }
 
   @Test
   public void testNestedSchemaConversion() {
-    RelDataType relSchema = nestedRecordSchemaProvider.getRelationalSchema();
+    RelDataType relSchema = relSchemaConverter.convertToRelSchema(nestedRecordSchemaProvider.getSqlSchema());
 
     LOG.info("Relational schema " + relSchema);
   }
@@ -160,8 +173,7 @@ public class TestAvroRelConversion {
     record.put("name", "name1");
 
     SamzaSqlRelMessage message = simpleRecordAvroRelConverter.convertToRelMessage(new KV<>("key", record));
-    LOG.info(Joiner.on(",").join(message.getSamzaSqlRelRecord().getFieldValues()));
-    LOG.info(Joiner.on(",").join(message.getSamzaSqlRelRecord().getFieldNames()));
+    LOG.info(message.toString());
   }
 
   @Test
@@ -192,22 +204,24 @@ public class TestAvroRelConversion {
   @Test
   public void testComplexRecordConversion() throws IOException {
     GenericData.Record record = new GenericData.Record(ComplexRecord.SCHEMA$);
+
     record.put("id", id);
     record.put("bool_value", boolValue);
     record.put("double_value", doubleValue);
-    record.put("float_value", floatValue);
+    record.put("float_value0", floatValue);
     record.put("string_value", testStrValue);
     record.put("bytes_value", testBytes);
     record.put("fixed_value", fixedBytes);
     record.put("long_value", longValue);
     record.put("array_values", arrayValue);
     record.put("map_values", mapValue);
+    record.put("union_value", testStrValue);
 
     ComplexRecord complexRecord = new ComplexRecord();
     complexRecord.id = id;
     complexRecord.bool_value = boolValue;
     complexRecord.double_value = doubleValue;
-    complexRecord.float_value = floatValue;
+    complexRecord.float_value0 = floatValue;
     complexRecord.string_value = testStrValue;
     complexRecord.bytes_value = testBytes;
     complexRecord.fixed_value = fixedBytes;
@@ -216,12 +230,50 @@ public class TestAvroRelConversion {
     complexRecord.array_values.addAll(arrayValue);
     complexRecord.map_values = new HashMap<>();
     complexRecord.map_values.putAll(mapValue);
+    complexRecord.union_value = testStrValue;
+
 
     byte[] serializedData = bytesFromGenericRecord(record);
-    validateAvroSerializedData(serializedData);
+    validateAvroSerializedData(serializedData, testStrValue);
 
     serializedData = encodeAvroSpecificRecord(ComplexRecord.class, complexRecord);
-    validateAvroSerializedData(serializedData);
+    validateAvroSerializedData(serializedData, testStrValue);
+  }
+
+  @Test
+  public void testComplexUnionConversionShouldWorkWithBothStringAndIntTypes() throws Exception {
+    // ComplexUnion is a nested avro non-nullable union-type with both String and Integer type
+    // Test the complex-union conversion for String type.
+    GenericData.Record record = new GenericData.Record(ComplexUnion.SCHEMA$);
+    record.put("non_nullable_union_value", testStrValue);
+
+    ComplexUnion complexUnion = new ComplexUnion();
+    complexUnion.non_nullable_union_value = testStrValue;
+
+    byte[] serializedData = bytesFromGenericRecord(record);
+    GenericRecord genericRecord = genericRecordFromBytes(serializedData, ComplexUnion.SCHEMA$);
+    SamzaSqlRelMessage message = complexUnionAvroRelConverter.convertToRelMessage(new KV<>("key", genericRecord));
+
+    Assert.assertEquals(testStrValue, message.getSamzaSqlRelRecord().getField("non_nullable_union_value").get().toString());
+
+    serializedData = encodeAvroSpecificRecord(ComplexUnion.class, complexUnion);
+    genericRecord = genericRecordFromBytes(serializedData, ComplexUnion.SCHEMA$);
+    Assert.assertEquals(testStrValue, genericRecord.get("non_nullable_union_value").toString());
+
+    // Testing the complex-union conversion for Integer type
+    record.put("non_nullable_union_value", Integer.valueOf(123));
+
+    complexUnion.non_nullable_union_value = Integer.valueOf(123);
+
+    serializedData = bytesFromGenericRecord(record);
+    genericRecord = genericRecordFromBytes(serializedData, ComplexUnion.SCHEMA$);
+    message = complexUnionAvroRelConverter.convertToRelMessage(new KV<>("key", genericRecord));
+    Assert.assertEquals(Integer.valueOf(123), message.getSamzaSqlRelRecord().getField("non_nullable_union_value").get());
+
+    serializedData = encodeAvroSpecificRecord(ComplexUnion.class, complexUnion);
+    genericRecord = genericRecordFromBytes(serializedData, ComplexUnion.SCHEMA$);
+    Assert.assertEquals(Integer.valueOf(123), genericRecord.get("non_nullable_union_value"));
+
   }
 
   @Test
@@ -238,7 +290,6 @@ public class TestAvroRelConversion {
     record.put("address", addressRecord);
     record.put("selfEmployed", "True");
 
-
     GenericData.Record phoneNumberRecordH = new GenericData.Record(PhoneNumber.SCHEMA$);
     phoneNumberRecordH.put("kind", Kind.Home);
     phoneNumberRecordH.put("number", "111-111-1111");
@@ -263,8 +314,51 @@ public class TestAvroRelConversion {
 
     SamzaSqlRelMessage relMessage = nestedRecordAvroRelConverter.convertToRelMessage(new KV<>("key", record));
 
-    LOG.info(Joiner.on(",").join(relMessage.getSamzaSqlRelRecord().getFieldValues()));
-    LOG.info(Joiner.on(",").join(relMessage.getSamzaSqlRelRecord().getFieldNames()));
+    LOG.info(relMessage.toString());
+
+    KV<Object, Object> samzaMessage = nestedRecordAvroRelConverter.convertToSamzaMessage(relMessage);
+    GenericRecord recordPostConversion = (GenericRecord) samzaMessage.getValue();
+
+    for (Schema.Field field : Profile.SCHEMA$.getFields()) {
+      // equals() on GenericRecord does the nested record equality check as well.
+      Assert.assertEquals(record.get(field.name()), recordPostConversion.get(field.name()));
+    }
+  }
+
+  // SAMZA-2110 We need to enable this when we have a true support for Null records
+  @Ignore
+  @Test
+  public void testRecordConversionWithNullPayload() throws IOException {
+    GenericData.Record record = null;
+    SamzaSqlRelMessage relMessage = nestedRecordAvroRelConverter.convertToRelMessage(new KV<>("key", record));
+
+    LOG.info(relMessage.toString());
+
+    KV<Object, Object> samzaMessage = nestedRecordAvroRelConverter.convertToSamzaMessage(relMessage);
+    GenericRecord recordPostConversion = (GenericRecord) samzaMessage.getValue();
+
+    Assert.assertTrue(recordPostConversion == null);
+  }
+
+  @Test
+  public void testNestedRecordConversionWithSubRecordsBeingNull() throws IOException {
+    GenericData.Record record = new GenericData.Record(Profile.SCHEMA$);
+    record.put("id", 1);
+    record.put("name", "name1");
+    record.put("companyId", 0);
+    GenericData.Record addressRecord = null;
+    record.put("address", addressRecord);
+    record.put("selfEmployed", "True");
+
+    List<GenericData.Record> phoneNumbers = null;
+    record.put("phoneNumbers", phoneNumbers);
+
+    HashMap<String, IndexedRecord> mapValues = null;
+    record.put("mapValues", mapValues);
+
+    SamzaSqlRelMessage relMessage = nestedRecordAvroRelConverter.convertToRelMessage(new KV<>("key", record));
+
+    LOG.info(relMessage.toString());
 
     KV<Object, Object> samzaMessage = nestedRecordAvroRelConverter.convertToSamzaMessage(relMessage);
     GenericRecord recordPostConversion = (GenericRecord) samzaMessage.getValue();
@@ -292,23 +386,28 @@ public class TestAvroRelConversion {
     return outputStream.toByteArray();
   }
 
-  private void validateAvroSerializedData(byte[] serializedData) throws IOException {
+  private void validateAvroSerializedData(byte[] serializedData, Object unionValue) throws IOException {
     GenericRecord complexRecordValue = genericRecordFromBytes(serializedData, ComplexRecord.SCHEMA$);
 
     SamzaSqlRelMessage message = complexRecordAvroRelConverter.convertToRelMessage(new KV<>("key", complexRecordValue));
-    Assert.assertEquals(message.getSamzaSqlRelRecord().getFieldNames().size(), ComplexRecord.SCHEMA$.getFields().size() + 1);
+    Assert.assertEquals(message.getSamzaSqlRelRecord().getFieldNames().size(),
+        ComplexRecord.SCHEMA$.getFields().size() + 1);
 
     Assert.assertEquals(message.getSamzaSqlRelRecord().getField("id").get(), id);
     Assert.assertEquals(message.getSamzaSqlRelRecord().getField("bool_value").get(), boolValue);
     Assert.assertEquals(message.getSamzaSqlRelRecord().getField("double_value").get(), doubleValue);
     Assert.assertEquals(message.getSamzaSqlRelRecord().getField("string_value").get(), new Utf8(testStrValue));
-    Assert.assertEquals(message.getSamzaSqlRelRecord().getField("float_value").get(), floatValue);
+    Assert.assertEquals(message.getSamzaSqlRelRecord().getField("float_value0").get(), floatValue);
     Assert.assertEquals(message.getSamzaSqlRelRecord().getField("long_value").get(), longValue);
-    Assert.assertTrue(
-        arrayValue.stream()
-            .map(Utf8::new)
-            .collect(Collectors.toList())
-            .equals(message.getSamzaSqlRelRecord().getField("array_values").get()));
+    if (unionValue instanceof String) {
+      Assert.assertEquals(message.getSamzaSqlRelRecord().getField("union_value").get(), new Utf8((String) unionValue));
+    } else {
+      Assert.assertEquals(message.getSamzaSqlRelRecord().getField("union_value").get(), unionValue);
+    }
+    Assert.assertTrue(arrayValue.stream()
+        .map(Utf8::new)
+        .collect(Collectors.toList())
+        .equals(message.getSamzaSqlRelRecord().getField("array_values").get()));
     Assert.assertTrue(mapValue.entrySet()
         .stream()
         .collect(Collectors.toMap(x -> new Utf8(x.getKey()), y -> new Utf8(y.getValue())))
@@ -321,8 +420,7 @@ public class TestAvroRelConversion {
         Arrays.equals(((ByteString) message.getSamzaSqlRelRecord().getField("fixed_value").get()).getBytes(),
             DEFAULT_TRACKING_ID_BYTES));
 
-    LOG.info(Joiner.on(",").useForNull("null").join(message.getSamzaSqlRelRecord().getFieldValues()));
-    LOG.info(Joiner.on(",").join(message.getSamzaSqlRelRecord().getFieldNames()));
+    LOG.info(message.toString());
 
     KV<Object, Object> samzaMessage = complexRecordAvroRelConverter.convertToSamzaMessage(message);
     GenericRecord record = (GenericRecord) samzaMessage.getValue();
@@ -331,7 +429,8 @@ public class TestAvroRelConversion {
       if (field.name().equals("array_values")) {
         Assert.assertTrue(record.get(field.name()).equals(complexRecordValue.get(field.name())));
       } else {
-        Assert.assertEquals(record.get(field.name()), complexRecordValue.get(field.name()));
+        Object expected = complexRecordValue.get(field.name());
+        Assert.assertEquals(expected, record.get(field.name()));
       }
     }
   }
