@@ -26,7 +26,7 @@ import java.util.Set;
 import java.util.concurrent.atomic.AtomicInteger;
 import java.util.function.Function;
 import java.util.stream.Collectors;
-import org.apache.commons.lang.NotImplementedException;
+import org.apache.commons.lang3.NotImplementedException;
 import org.apache.samza.Partition;
 import org.apache.samza.config.Config;
 import org.apache.samza.metrics.MetricsRegistry;
@@ -55,7 +55,7 @@ public class ConsoleLoggingSystemFactory implements SystemFactory {
 
   @Override
   public SystemConsumer getConsumer(String systemName, Config config, MetricsRegistry registry) {
-    throw new NotImplementedException();
+    throw new NotImplementedException("Not Implemented");
   }
 
   @Override
@@ -85,7 +85,7 @@ public class ConsoleLoggingSystemFactory implements SystemFactory {
     @Override
     public void send(String source, OutgoingMessageEnvelope envelope) {
       String msg = String.format("OutputStream:%s Key:%s Value:%s", envelope.getSystemStream(), envelope.getKey(),
-          new String((byte[]) envelope.getMessage()));
+          envelope.getMessage() != null ? new String((byte[]) envelope.getMessage()) : null);
       LOG.info(msg);
 
       System.out.println(String.format("Message %d :", messageCounter.incrementAndGet()));
@@ -97,7 +97,7 @@ public class ConsoleLoggingSystemFactory implements SystemFactory {
     }
 
     private String getFormattedValue(OutgoingMessageEnvelope envelope) {
-      String value = new String((byte[]) envelope.getMessage());
+      String value = envelope.getMessage() != null ? new String((byte[]) envelope.getMessage()) : null;
       String formattedValue;
 
       try {
