@@ -60,8 +60,6 @@ import org.apache.http.impl.client.HttpClientBuilder
 import org.apache.samza.webapp.ApplicationMasterRestClient
 
 object ClientHelper {
-  val applicationType = "Samza"
-
   val CREDENTIALS_FILE = "credentials"
 
   val SOURCE = "yarn"
@@ -177,6 +175,10 @@ class ClientHelper(conf: Configuration) extends Logging {
     appCtx.setApplicationId(appId.get)
     info("set app ID to %s" format appId.get)
 
+    if (new JobConfig(config).getApplicationMasterHighAvailabilityEnabled) {
+      appCtx.setKeepContainersAcrossApplicationAttempts(true)
+      info("keep containers alive across application attempts for AM High availability")
+    }
     val localResources: HashMap[String, LocalResource] = HashMap[String, LocalResource]()
     localResources += "__package" -> packageResource
 
@@ -230,7 +232,7 @@ class ClientHelper(conf: Configuration) extends Logging {
     }
 
     appCtx.setAMContainerSpec(containerCtx)
-    appCtx.setApplicationType(ClientHelper.applicationType)
+    appCtx.setApplicationType(yarnConfig.getYarnApplicationType)
     info("submitting application request for %s" format appId.get)
     yarnClient.submitApplication(appCtx)
     appId
@@ -303,8 +305,8 @@ class ClientHelper(conf: Configuration) extends Logging {
   }
 
   private def isActiveApplication(applicationReport: ApplicationReport): Boolean = {
-    (Running.equals(toAppStatus(applicationReport).get)
-    || New.equals(toAppStatus(applicationReport).get))
+    val status = toAppStatus(applicationReport).get
+    Running.equals(status) || New.equals(status)
   }
 
   def toAppStatus(applicationReport: ApplicationReport): Option[ApplicationStatus] = {
@@ -350,15 +352,20 @@ class ClientHelper(conf: Configuration) extends Logging {
       } else {
         false
       }
+    } catch {
+      case e: Exception => {
+        // ignore any exceptions when querying the AM - likely due to YARN restarting the AM process
+        warn("Exception when querying AM metrics", e)
+        false
+      }
     } finally {
       amClient.close()
     }
   }
 
   private[yarn] def validateJobConfig(config: Config) = {
-    import org.apache.samza.config.JobConfig.Config2Job
-
-    if (config.getSecurityManagerFactory.isEmpty) {
+    val jobConfig = new JobConfig(config)
+    if (!jobConfig.getSecurityManagerFactory.isPresent) {
       throw new SamzaException(s"Job config ${JobConfig.JOB_SECURITY_MANAGER_FACTORY} not found. This config must be set for a secure cluster")
     }
   }
