@@ -24,6 +24,7 @@ import java.net.URL;
 import junit.framework.Assert;
 import org.apache.hadoop.yarn.util.ConverterUtils;
 import org.apache.samza.container.ContainerHeartbeatResponse;
+import org.apache.samza.coordinator.CoordinationConstants;
 import org.apache.samza.coordinator.server.HttpServer;
 import org.apache.samza.job.yarn.YarnAppState;
 import org.apache.samza.job.yarn.YarnContainer;
@@ -73,10 +74,10 @@ public class TestYarnContainerHeartbeatServlet {
   @Test
   public void testContainerHeartbeatWhenValid()
       throws IOException {
-    String VALID_CONTAINER_ID = "container_1350670447861_0003_01_000002";
-    when(container.id()).thenReturn(ConverterUtils.toContainerId(VALID_CONTAINER_ID));
-    yarnAppState.runningYarnContainers.put(VALID_CONTAINER_ID, container);
-    URL url = new URL(webApp.getUrl().toString() + "containerHeartbeat?executionContainerId=" + VALID_CONTAINER_ID);
+    String validContainerId = "container_1350670447861_0003_01_000002";
+    when(container.id()).thenReturn(ConverterUtils.toContainerId(validContainerId));
+    yarnAppState.runningProcessors.put(validContainerId, container);
+    URL url = new URL(String.format(CoordinationConstants.YARN_CONTAINER_HEARTBEAT_ENDPOINT_FORMAT, webApp.getUrl().toString(), validContainerId));
     String response = HttpUtil.read(url, 1000, new ExponentialSleepStrategy());
     heartbeat = mapper.readValue(response, ContainerHeartbeatResponse.class);
     Assert.assertTrue(heartbeat.isAlive());
@@ -85,11 +86,12 @@ public class TestYarnContainerHeartbeatServlet {
   @Test
   public void testContainerHeartbeatWhenInvalid()
       throws IOException {
-    String VALID_CONTAINER_ID = "container_1350670447861_0003_01_000003";
-    String INVALID_CONTAINER_ID = "container_1350670447861_0003_01_000002";
-    when(container.id()).thenReturn(ConverterUtils.toContainerId(VALID_CONTAINER_ID));
-    yarnAppState.runningYarnContainers.put(VALID_CONTAINER_ID, container);
-    URL url = new URL(webApp.getUrl().toString() + "containerHeartbeat?executionContainerId=" + INVALID_CONTAINER_ID);
+    String validContainerId = "container_1350670447861_0003_01_000003";
+    String invalidContainerId = "container_1350670447861_0003_01_000002";
+    when(container.id()).thenReturn(ConverterUtils.toContainerId(validContainerId));
+    yarnAppState.runningProcessors.put(validContainerId, container);
+    URL url = new URL(String.format(CoordinationConstants.YARN_CONTAINER_HEARTBEAT_ENDPOINT_FORMAT,
+        webApp.getUrl().toString(), invalidContainerId));
     String response = HttpUtil.read(url, 1000, new ExponentialSleepStrategy());
     heartbeat = mapper.readValue(response, ContainerHeartbeatResponse.class);
     Assert.assertFalse(heartbeat.isAlive());
